@@ -15,19 +15,17 @@
  *    limitations under the License.
  */
 
-#include "../stdafx.h"
+#include "stdafx.h"
+#include "../client/dbclient.h"
+#include "../db/dbhelpers.h"
 #include "../db/cmdline.h"
 
 namespace mongo {
 
-    const char * curNs = "in client mode";
-
-    bool quiet = false;
-
     CmdLine cmdLine;
 
-    //    Database* database = 0;
-    
+    const char * curNs = "in client mode";
+
     bool dbexitCalled = false;
 
     void dbexit( ExitCode returnCode, const char *whyMsg ) {
@@ -43,6 +41,10 @@ namespace mongo {
         return dbexitCalled;
     }
 
+    void setupSignals(){
+        // maybe should do SIGPIPE here, not sure
+    }
+
     string getDbContext() {
         return "in client only mode";
     }
@@ -50,4 +52,15 @@ namespace mongo {
     bool haveLocalShardingInfo( const string& ns ){
         return false;
     }
+
+    DBClientBase * createDirectClient(){
+        uassert( 10256 ,  "no createDirectClient in clientOnly" , 0 );
+        return 0;
+    }
+/*
+    auto_ptr<CursorIterator> Helpers::find( const char *ns , BSONObj query , bool requireIndex ){
+        uassert( 10000 ,  "Helpers::find can't be used in client" , 0 );
+        return auto_ptr<CursorIterator>(0);
+    }
+*/
 }
