@@ -105,8 +105,8 @@ namespace mongo {
 
     /* --- dbclientcommands --- */
 
-    inline bool DBClientWithCommands::isOk(const BSONObj& o) {
-        return o.getIntField("ok") == 1;
+    bool DBClientWithCommands::isOk(const BSONObj& o) {
+        return o["ok"].trueValue();
     }
 
     inline bool DBClientWithCommands::runCommand(const string &dbname, const BSONObj& cmd, BSONObj &info, int options) {
@@ -132,7 +132,7 @@ namespace mongo {
         BSONObj cmd = BSON( "count" << ns.coll << "query" << query );
         BSONObj res;
         if( !runCommand(ns.db.c_str(), cmd, res, options) )
-            uasserted(string("count fails:") + res.toString());
+            uasserted(11010,string("count fails:") + res.toString());
         return res.getIntField("n");
     }
 
@@ -233,11 +233,11 @@ namespace mongo {
         BSONObj o;
         if ( info == 0 )	info = &o;
         BSONObjBuilder b;
-        b.append("create", ns);
+        string db = nsToDatabase(ns.c_str());
+        b.append("create", ns.c_str() + db.length() + 1);
         if ( size ) b.append("size", size);
         if ( capped ) b.append("capped", true);
         if ( max ) b.append("max", max);
-        string db = nsToClient(ns.c_str());
         return runCommand(db.c_str(), b.done(), *info);
     }
 
@@ -314,8 +314,8 @@ namespace mongo {
 
     list<string> DBClientWithCommands::getDatabaseNames(){
         BSONObj info;
-        uassert( "listdatabases failed" , runCommand( "admin" , BSON( "listDatabases" << 1 ) , info ) );
-        uassert( "listDatabases.databases not array" , info["databases"].type() == Array );
+        uassert( 10005 ,  "listdatabases failed" , runCommand( "admin" , BSON( "listDatabases" << 1 ) , info ) );
+        uassert( 10006 ,  "listDatabases.databases not array" , info["databases"].type() == Array );
         
         list<string> names;
         
@@ -422,11 +422,11 @@ namespace mongo {
 		return DBClientBase::auth(dbname, username, password.c_str(), errmsg, false);
 	}
 
-    BSONObj DBClientBase::findOne(const string &ns, Query query, const BSONObj *fieldsToReturn, int queryOptions) {
+    BSONObj DBClientInterface::findOne(const string &ns, Query query, const BSONObj *fieldsToReturn, int queryOptions) {
         auto_ptr<DBClientCursor> c =
             this->query(ns, query, 1, 0, fieldsToReturn, queryOptions);
 
-        massert( "DBClientBase::findOne: transport error", c.get() );
+        massert( 10276 ,  "DBClientBase::findOne: transport error", c.get() );
 
         if ( !c->more() )
             return BSONObj();
@@ -448,7 +448,13 @@ namespace mongo {
             port = CmdLine::DefaultDBPort;
             ip = hostbyname( serverAddress.c_str() );
         }
-        massert( "Unable to parse hostname", !ip.empty() );
+        if( ip.empty() ) {
+            stringstream ss;
+            ss << "client connect: couldn't parse/resolve hostname: " << _serverAddress;
+            errmsg = ss.str();
+            failed = true;
+            return false;
+        }
 
         // we keep around SockAddr for connection life -- maybe MessagingPort
         // requires that?
@@ -494,10 +500,10 @@ namespace mongo {
     }
 
     auto_ptr<DBClientCursor> DBClientBase::query(const string &ns, Query query, int nToReturn,
-            int nToSkip, const BSONObj *fieldsToReturn, int queryOptions) {
+                                                 int nToSkip, const BSONObj *fieldsToReturn, int queryOptions , int batchSize ) {
         auto_ptr<DBClientCursor> c( new DBClientCursor( this,
-                                    ns, query.obj, nToReturn, nToSkip,
-                                    fieldsToReturn, queryOptions ) );
+                                                        ns, query.obj, nToReturn, nToSkip,
+                                                        fieldsToReturn, queryOptions , batchSize ) );
         if ( c->init() )
             return c;
         return auto_ptr< DBClientCursor >( 0 );
@@ -562,12 +568,12 @@ namespace mongo {
     void DBClientBase::update( const string & ns , Query query , BSONObj obj , bool upsert , bool multi ) {
 
         BufBuilder b;
-        b.append( (int)0 ); // reserverd
+        b.append( (int)0 ); // reserved
         b.append( ns );
 
         int flags = 0;
-        if ( upsert ) flags |= Option_Upsert;
-        if ( multi ) flags |= Option_Multi;
+        if ( upsert ) flags |= UpdateOption_Upsert;
+        if ( multi ) flags |= UpdateOption_Multi;
         b.append( flags );
 
         query.obj.appendSelfToBufBuilder( b );
@@ -590,18 +596,18 @@ namespace mongo {
 
     void DBClientWithCommands::dropIndex( const string& ns , const string& indexName ){
         BSONObj info;
-        if ( ! runCommand( nsToClient( ns.c_str() ) , 
+        if ( ! runCommand( nsToDatabase( ns.c_str() ) , 
                            BSON( "deleteIndexes" << NamespaceString( ns ).coll << "index" << indexName ) , 
                            info ) ){
             log() << "dropIndex failed: " << info << endl;
-            uassert( "dropIndex failed" , 0 );
+            uassert( 10007 ,  "dropIndex failed" , 0 );
         }
         resetIndexCache();
     }
     
     void DBClientWithCommands::dropIndexes( const string& ns ){
         BSONObj info;
-        uassert( "dropIndexes failed" , runCommand( nsToClient( ns.c_str() ) , 
+        uassert( 10008 ,  "dropIndexes failed" , runCommand( nsToDatabase( ns.c_str() ) , 
                                                     BSON( "deleteIndexes" << NamespaceString( ns ).coll << "index" << "*") , 
                                                     info ) );
         resetIndexCache();
@@ -716,7 +722,7 @@ namespace mongo {
             if ( !port().call(toSend, response) ) {
                 failed = true;
                 if ( assertOk )
-                    massert("dbclient error communicating with server", false);
+                    massert( 10278 , "dbclient error communicating with server", false);
                 return false;
             }
         }
@@ -740,10 +746,19 @@ namespace mongo {
         }
     }
 
+    int DBClientCursor::nextBatchSize(){
+        if ( nToReturn == 0 )
+            return batchSize;
+        if ( batchSize == 0 )
+            return nToReturn;
+        
+        return batchSize < nToReturn ? batchSize : nToReturn;
+    }
+
     bool DBClientCursor::init() {
         Message toSend;
         if ( !cursorId ) {
-            assembleRequest( ns, query, nToReturn, nToSkip, fieldsToReturn, opts, toSend );
+            assembleRequest( ns, query, nextBatchSize() , nToSkip, fieldsToReturn, opts, toSend );
         } else {
             BufBuilder b;
             b.append( opts );
@@ -754,6 +769,8 @@ namespace mongo {
         }
         if ( !connector->call( toSend, *m, false ) )
             return false;
+        if ( ! m->data )
+            return false;
         dataReceived();
         return true;
     }
@@ -761,10 +778,14 @@ namespace mongo {
     void DBClientCursor::requestMore() {
         assert( cursorId && pos == nReturned );
 
+        if (haveLimit){
+            nToReturn -= nReturned;
+            assert(nToReturn > 0);
+        }
         BufBuilder b;
         b.append(opts);
         b.append(ns.c_str());
-        b.append(nToReturn);
+        b.append(nextBatchSize());
         b.append(cursorId);
 
         Message toSend;
@@ -785,7 +806,7 @@ namespace mongo {
             cursorId = 0; // 0 indicates no longer valid (dead)
             // TODO: should we throw a UserException here???
         }
-        if ( cursorId == 0 || ! ( opts & Option_CursorTailable ) ) {
+        if ( cursorId == 0 || ! ( opts & QueryOption_CursorTailable ) ) {
             // only set initially: we don't want to kill it on end of data
             // if it's a tailable cursor
             cursorId = qr->cursorId;
@@ -800,7 +821,14 @@ namespace mongo {
         */
     }
 
+    /** If true, safe to call next().  Requests more from server if necessary. */
     bool DBClientCursor::more() {
+        if ( !_putBack.empty() )
+            return true;
+        
+        if (haveLimit && pos >= nToReturn)
+            return false;
+
         if ( pos < nReturned )
             return true;
 
@@ -813,6 +841,11 @@ namespace mongo {
 
     BSONObj DBClientCursor::next() {
         assert( more() );
+        if ( !_putBack.empty() ) {
+            BSONObj ret = _putBack.top();
+            _putBack.pop();
+            return ret;
+        }
         pos++;
         BSONObj o(data);
         data += o.objsize();
@@ -820,18 +853,19 @@ namespace mongo {
     }
 
     DBClientCursor::~DBClientCursor() {
-        if ( cursorId && ownCursor_ ) {
-            BufBuilder b;
-            b.append( (int)0 ); // reserved
-            b.append( (int)1 ); // number
-            b.append( cursorId );
+        DESTRUCTOR_GUARD (
+            if ( cursorId && _ownCursor ) {
+                BufBuilder b;
+                b.append( (int)0 ); // reserved
+                b.append( (int)1 ); // number
+                b.append( cursorId );
 
-            Message m;
-            m.setData( dbKillCursors , b.buf() , b.len() );
-
-            connector->sayPiggyBack( m );
-        }
+                Message m;
+                m.setData( dbKillCursors , b.buf() , b.len() );
 
+                connector->sayPiggyBack( m );
+            }
+        );
     }
 
     /* --- class dbclientpaired --- */
@@ -878,7 +912,7 @@ namespace mongo {
             sleepsecs(1);
         }
 
-        uassert("checkmaster: no master found", false);
+        uassert( 10009 , "checkmaster: no master found", false);
     }
 
     inline DBClientConnection& DBClientPaired::checkMaster() {
@@ -922,7 +956,7 @@ namespace mongo {
 
     bool DBClientPaired::connect(string hostpairstring) { 
         size_t comma = hostpairstring.find( "," );
-        uassert("bad hostpairstring", comma != string::npos);
+        uassert( 10010 , "bad hostpairstring", comma != string::npos);
         return connect( hostpairstring.substr( 0 , comma ) , hostpairstring.substr( comma + 1 ) );
     }
 
@@ -944,9 +978,9 @@ namespace mongo {
 	}
 
     auto_ptr<DBClientCursor> DBClientPaired::query(const string &a, Query b, int c, int d,
-            const BSONObj *e, int f)
+                                                   const BSONObj *e, int f, int g)
     {
-        return checkMaster().query(a,b,c,d,e,f);
+        return checkMaster().query(a,b,c,d,e,f,g);
     }
 
     BSONObj DBClientPaired::findOne(const string &a, Query b, const BSONObj *c, int d) {
