@@ -21,6 +21,7 @@
 #include "../util/message.h"
 #include "../db/jsobj.h"
 #include "../db/json.h"
+#include <stack>
 
 namespace mongo {
 
@@ -34,22 +35,41 @@ namespace mongo {
            final object it references were deleted.  Thus, you should be prepared to requery if you get back
            ResultFlag_CursorNotFound.
         */
-        Option_CursorTailable = 1 << 1,
+        QueryOption_CursorTailable = 1 << 1,
 
         /** allow query of replica slave.  normally these return an error except for namespace "local".
         */
-        Option_SlaveOk = 1 << 2,
+        QueryOption_SlaveOk = 1 << 2,
         
-        Option_OplogReplay = 1 << 3,
+        // findingStart mode is used to find the first operation of interest when
+        // we are scanning through a repl log.  For efficiency in the common case,
+        // where the first operation of interest is closer to the tail than the head,
+        // we start from the tail of the log and work backwards until we find the
+        // first operation of interest.  Then we scan forward from that first operation,
+        // actually returning results to the client.  During the findingStart phase,
+        // we release the db mutex occasionally to avoid blocking the db process for
+        // an extended period of time.
+        QueryOption_OplogReplay = 1 << 3,
+
+        /** The server normally times out idle cursors after an inactivy period to prevent excess memory use
+            Set this option to prevent that. 
+        */
+        QueryOption_NoCursorTimeout = 1 << 4,
+
+        /** Use with QueryOption_CursorTailable.  If we are at the end of the data, block for a while rather 
+            than returning no data. After a timeout period, we do return as normal.
+        */
+        QueryOption_AwaitData = 1 << 5
 
-        /** if there is a cursor, ignore the normal cursor timeout behavior and never time it out
-         */
-        Option_NoCursorTimeout = 1 << 4
     };
 
     enum UpdateOptions {
-        Option_Upsert = 1 << 0,
-        Option_Multi = 1 << 1
+        /** Upsert - that is, insert the item if no matching item is found. */
+        UpdateOption_Upsert = 1 << 0,
+
+        /** Update multiple documents (if multiple documents match query expression). 
+           (Default is update a single document and stop.) */
+        UpdateOption_Multi = 1 << 1
     };
 
     class BSONObj;
@@ -181,10 +201,18 @@ namespace mongo {
 	/** Queries return a cursor object */
     class DBClientCursor : boost::noncopyable {
         friend class DBClientBase;
+        bool init();
     public:
-		/** if true, safe to call next() */
+		/** If true, safe to call next().  Requests more from server if necessary. */
         bool more();
 
+        /** If true, there is more in our local buffers to be fetched via next(). Returns 
+            false when a getMore request back to server would be required.  You can use this 
+            if you want to exhaust whatever data has been fetched to the client already but 
+            then perhaps stop.
+        */
+        bool moreInCurrentBatch() { return !_putBack.empty() || pos < nReturned; }
+
         /** next
 		   @return next object in the result cursor.
            on an error at the remote server, you will get back:
@@ -192,6 +220,11 @@ namespace mongo {
            if you do not want to handle that yourself, call nextSafe().
         */
         BSONObj next();
+        
+        /** 
+            restore an object previously returned by next() to the cursor
+         */
+        void putBack( const BSONObj &o ) { _putBack.push( o.getOwned() ); }
 
 		/** throws AssertionException if get back { $err : ... } */
         BSONObj nextSafe() {
@@ -223,59 +256,74 @@ namespace mongo {
         }
 
         bool tailable() const {
-            return (opts & Option_CursorTailable) != 0;
+            return (opts & QueryOption_CursorTailable) != 0;
         }
         
+        /** see QueryResult::ResultFlagType (db/dbmessage.h) for flag values 
+            mostly these flags are for internal purposes - 
+            ResultFlag_ErrSet is the possible exception to that
+        */
         bool hasResultFlag( int flag ){
             return (resultFlags & flag) != 0;
         }
-    private:
-        bool init();
-    public:
+
         DBClientCursor( DBConnector *_connector, const string &_ns, BSONObj _query, int _nToReturn,
-                        int _nToSkip, const BSONObj *_fieldsToReturn, int queryOptions ) :
+                        int _nToSkip, const BSONObj *_fieldsToReturn, int queryOptions , int bs ) :
                 connector(_connector),
                 ns(_ns),
                 query(_query),
                 nToReturn(_nToReturn),
+                haveLimit( _nToReturn > 0 && !(queryOptions & QueryOption_CursorTailable)),
                 nToSkip(_nToSkip),
                 fieldsToReturn(_fieldsToReturn),
                 opts(queryOptions),
+                batchSize(bs),
                 m(new Message()),
                 cursorId(),
                 nReturned(),
                 pos(),
                 data(),
-                ownCursor_( true ) {
+                _ownCursor( true ) {
         }
         
         DBClientCursor( DBConnector *_connector, const string &_ns, long long _cursorId, int _nToReturn, int options ) :
                 connector(_connector),
                 ns(_ns),
                 nToReturn( _nToReturn ),
+                haveLimit( _nToReturn > 0 && !(options & QueryOption_CursorTailable)),
                 opts( options ),
                 m(new Message()),
                 cursorId( _cursorId ),
                 nReturned(),
                 pos(),
                 data(),
-                ownCursor_( true ) {
+                _ownCursor( true ) {
         }            
 
         virtual ~DBClientCursor();
 
         long long getCursorId() const { return cursorId; }
-        void decouple() { ownCursor_ = false; }
+
+        /** by default we "own" the cursor and will send the server a KillCursor
+            message when ~DBClientCursor() is called. This function overrides that.
+        */
+        void decouple() { _ownCursor = false; }
         
     private:
+        
+        int nextBatchSize();
+
         DBConnector *connector;
         string ns;
         BSONObj query;
         int nToReturn;
+        bool haveLimit;
         int nToSkip;
         const BSONObj *fieldsToReturn;
         int opts;
+        int batchSize;
         auto_ptr<Message> m;
+        stack< BSONObj > _putBack;
 
         int resultFlags;
         long long cursorId;
@@ -284,22 +332,19 @@ namespace mongo {
         const char *data;
         void dataReceived();
         void requestMore();
-        bool ownCursor_;
+        bool _ownCursor; // see decouple()
     };
     
-
     /**
        The interface that any db connection should implement
      */
     class DBClientInterface : boost::noncopyable {
     public:
         virtual auto_ptr<DBClientCursor> query(const string &ns, Query query, int nToReturn = 0, int nToSkip = 0,
-                                               const BSONObj *fieldsToReturn = 0, int queryOptions = 0) = 0;
+                                               const BSONObj *fieldsToReturn = 0, int queryOptions = 0 , int batchSize = 0 ) = 0;
 
         virtual auto_ptr<DBClientCursor> getMore( const string &ns, long long cursorId, int nToReturn = 0, int options = 0 ) = 0;
         
-        virtual BSONObj findOne(const string &ns, Query query, const BSONObj *fieldsToReturn = 0, int queryOptions = 0) = 0;
-
         virtual void insert( const string &ns, BSONObj obj ) = 0;
         
         virtual void insert( const string &ns, const vector< BSONObj >& v ) = 0;
@@ -309,6 +354,14 @@ namespace mongo {
         virtual void update( const string &ns , Query query , BSONObj obj , bool upsert = 0 , bool multi = 0 ) = 0;
 
         virtual ~DBClientInterface() { }
+
+        /**
+           @return a single object that matches the query.  if none do, then the object is empty
+           @throws AssertionException
+        */
+        virtual BSONObj findOne(const string &ns, Query query, const BSONObj *fieldsToReturn = 0, int queryOptions = 0);
+
+
     };
 
     /**
@@ -316,7 +369,6 @@ namespace mongo {
        Basically just invocations of connection.$cmd.findOne({...});
     */
     class DBClientWithCommands : public DBClientInterface {
-        bool isOk(const BSONObj&);
         set<string> _seenIndexes;
     public:
 
@@ -338,7 +390,7 @@ namespace mongo {
 			       set.
 			@return true if the command returned "ok".
         */
-        bool runCommand(const string &dbname, const BSONObj& cmd, BSONObj &info, int options=0);
+        virtual bool runCommand(const string &dbname, const BSONObj& cmd, BSONObj &info, int options=0);
 
         /** Authorize access to a particular database.
 			Authentication is separate for each database on the server -- you may authenticate for any 
@@ -412,7 +464,7 @@ namespace mongo {
         virtual bool dropCollection( const string &ns ){
             string db = nsGetDB( ns );
             string coll = nsGetCollection( ns );
-            uassert( "no collection name", coll.size() );
+            uassert( 10011 ,  "no collection name", coll.size() );
 
             BSONObj info;
             
@@ -457,6 +509,7 @@ namespace mongo {
             ProfileOff = 0,
             ProfileSlow = 1, // log very slow (>100ms) operations
             ProfileAll = 2
+            
         };
         bool setDbProfilingLevel(const string &dbname, ProfilingLevel level, BSONObj *info = 0);
         bool getDbProfilingLevel(const string &dbname, ProfilingLevel& level, BSONObj *info = 0);
@@ -611,6 +664,9 @@ namespace mongo {
             return ns.substr( pos + 1 );            
         }
 
+    protected:
+        bool isOk(const BSONObj&);
+
     };
     
     /**
@@ -634,7 +690,7 @@ namespace mongo {
          @throws AssertionException
         */
         virtual auto_ptr<DBClientCursor> query(const string &ns, Query query, int nToReturn = 0, int nToSkip = 0,
-                                               const BSONObj *fieldsToReturn = 0, int queryOptions = 0);
+                                               const BSONObj *fieldsToReturn = 0, int queryOptions = 0 , int batchSize = 0 );
 
         /** @param cursorId id of cursor to retrieve
             @return an handle to a previously allocated cursor
@@ -642,12 +698,6 @@ namespace mongo {
          */
         virtual auto_ptr<DBClientCursor> getMore( const string &ns, long long cursorId, int nToReturn = 0, int options = 0 );
         
-        /**
-           @return a single object that matches the query.  if none do, then the object is empty
-           @throws AssertionException
-        */
-        virtual BSONObj findOne(const string &ns, Query query, const BSONObj *fieldsToReturn = 0, int queryOptions = 0);
-        
         /**
            insert an object into the database
          */
@@ -673,13 +723,20 @@ namespace mongo {
         
         virtual bool isFailed() const = 0;
 
+        static int countCommas( const string& s ){
+            int n = 0;
+            for ( unsigned i=0; i<s.size(); i++ )
+                if ( s[i] == ',' )
+                    n++;
+            return n;
+        }
     };
     
     class DBClientPaired;
     
     class ConnectException : public UserException { 
     public:
-        ConnectException(string msg) : UserException(msg) { }
+        ConnectException(string msg) : UserException(9000,msg) { }
     };
 
     /** 
@@ -734,9 +791,9 @@ namespace mongo {
         virtual bool auth(const string &dbname, const string &username, const string &pwd, string& errmsg, bool digestPassword = true);
 
         virtual auto_ptr<DBClientCursor> query(const string &ns, Query query, int nToReturn = 0, int nToSkip = 0,
-                                               const BSONObj *fieldsToReturn = 0, int queryOptions = 0) {
+                                               const BSONObj *fieldsToReturn = 0, int queryOptions = 0 , int batchSize = 0 ) {
             checkConnection();
-            return DBClientBase::query( ns, query, nToReturn, nToSkip, fieldsToReturn, queryOptions );
+            return DBClientBase::query( ns, query, nToReturn, nToSkip, fieldsToReturn, queryOptions , batchSize );
         }
 
         /**
@@ -767,7 +824,6 @@ namespace mongo {
             return serverAddress;
         }
 
-    protected:
         virtual bool call( Message &toSend, Message &response, bool assertOk = true );
         virtual void say( Message &toSend );
         virtual void sayPiggyBack( Message &toSend );
@@ -814,7 +870,7 @@ namespace mongo {
         /** throws userassertion "no master found" */
         virtual
         auto_ptr<DBClientCursor> query(const string &ns, Query query, int nToReturn = 0, int nToSkip = 0,
-                                       const BSONObj *fieldsToReturn = 0, int queryOptions = 0);
+                                       const BSONObj *fieldsToReturn = 0, int queryOptions = 0 , int batchSize = 0 );
 
         /** throws userassertion "no master found" */
         virtual
@@ -863,7 +919,7 @@ namespace mongo {
         
         bool isFailed() const {
             // TODO: this really should check isFailed on current master as well
-            return master > NotSetR;
+            return master < Left;
         }
     };
     
