@@ -24,6 +24,8 @@
 #include "clientcursor.h"
 #include "client.h"
 #include "dbhelpers.h"
+#include "curop.h"
+#include "stats/counters.h"
 
 namespace mongo {
 
@@ -40,6 +42,11 @@ namespace mongo {
     const int split_debug = 0;
     const int insert_debug = 0;
 
+    static void alreadyInIndex() { 
+        // we don't use massert() here as that does logging and this is 'benign' - see catches in _indexRecord()
+        throw MsgAssertionException(10287, "btree: key+recloc already in index");
+    }
+
     /* BucketBasics --------------------------------------------------- */
 
     inline void BucketBasics::modified(const DiskLoc& thisLoc) {
@@ -81,10 +88,10 @@ namespace mongo {
         {
             bool f = false;
             assert( f = true );
-            massert("assert is misdefined", f);
+            massert( 10281 , "assert is misdefined", f);
         }
 
-        checkForInterrupt();
+        killCurrentOp.checkForInterrupt();
         assertValid(order, true);
 //	if( bt_fv==0 )
 //		return;
@@ -222,14 +229,14 @@ namespace mongo {
 	   does not bother returning that value.
     */
     void BucketBasics::popBack(DiskLoc& recLoc, BSONObj& key) { 
-        massert( "n==0 in btree popBack()", n > 0 );
+        massert( 10282 ,  "n==0 in btree popBack()", n > 0 );
         assert( k(n-1).isUsed() ); // no unused skipping in this function at this point - btreebuilder doesn't require that
         KeyNode kn = keyNode(n-1);
         recLoc = kn.recordLoc;
         key = kn.key;
         int keysize = kn.key.objsize();
 
-		massert("rchild not null in btree popBack()", nextChild.isNull());
+		massert( 10283 , "rchild not null in btree popBack()", nextChild.isNull());
 
 		/* weirdly, we also put the rightmost down pointer in nextchild, even when bucket isn't full. */
 		nextChild = kn.prevChildBucket;
@@ -355,9 +362,36 @@ namespace mongo {
         return false;
     }
 
+    /* @param self - don't complain about ourself already being in the index case.
+       @return true = there is a duplicate.
+    */
+    bool BtreeBucket::wouldCreateDup(
+        const IndexDetails& idx, DiskLoc thisLoc, 
+        const BSONObj& key, BSONObj order,
+        DiskLoc self) 
+    { 
+        int pos;
+        bool found;
+        DiskLoc b = locate(idx, thisLoc, key, order, pos, found, minDiskLoc);
+
+        while ( !b.isNull() ) {
+            // we skip unused keys
+            BtreeBucket *bucket = b.btree();
+            _KeyNode& kn = bucket->k(pos);
+            if ( kn.isUsed() ) {
+                if( bucket->keyAt(pos).woEqual(key) )
+                    return kn.recordLoc != self;
+                break;
+            }
+            b = bucket->advance(b, pos, 1, "BtreeBucket::dupCheck");
+        }
+
+        return false;
+    }
+
     string BtreeBucket::dupKeyError( const IndexDetails& idx , const BSONObj& key ){
         stringstream ss;
-        ss << "E11000 duplicate key error";
+        ss << "E11000 duplicate key error ";
         ss << "index: " << idx.indexNamespace() << "  ";
         ss << "dup key: " << key;
         return ss.str();
@@ -390,6 +424,9 @@ namespace mongo {
 			}
 		}
 #endif
+        
+        globalIndexCounters.btree( (char*)this );
+        
         /* binary search for this key */
         bool dupsChecked = false;
         int l=0;
@@ -406,12 +443,19 @@ namespace mongo {
                         // coding effort in here to make this particularly fast
                         if( !dupsChecked ) { 
                             dupsChecked = true;
-                            if( idx.head.btree()->exists(idx, idx.head, key, order) )
-                                uasserted( dupKeyError( idx , key ) );
+                            if( idx.head.btree()->exists(idx, idx.head, key, order) ) {
+                                if( idx.head.btree()->wouldCreateDup(idx, idx.head, key, order, recordLoc) )
+                                    uasserted( ASSERT_ID_DUPKEY , dupKeyError( idx , key ) );
+                                else
+                                    alreadyInIndex();
+                            }
                         }
                     }
-                    else
-                        uasserted( dupKeyError( idx , key ) );
+                    else {
+                        if( M.recordLoc == recordLoc ) 
+                            alreadyInIndex();
+                        uasserted( ASSERT_ID_DUPKEY , dupKeyError( idx , key ) );
+                    }
                 }
 
                 // dup keys allowed.  use recordLoc as if it is part of the key
@@ -443,7 +487,7 @@ namespace mongo {
     }
 
     void BtreeBucket::delBucket(const DiskLoc& thisLoc, IndexDetails& id) {
-        ClientCursor::informAboutToDeleteBucket(thisLoc);
+        ClientCursor::informAboutToDeleteBucket(thisLoc); // slow...
         assert( !isHead() );
 
         BtreeBucket *p = parent.btreemod();
@@ -479,7 +523,7 @@ found:
         //defensive:
         n = -1;
         parent.Null();
-        massert("todo: use RecStoreInterface instead", false);
+        massert( 10284 , "todo: use RecStoreInterface instead", false);
         // TODO: this was broken anyway as deleteRecord does unindexRecord() call which assumes the data is a BSONObj, 
         // and it isn't.
         assert(false);
@@ -621,13 +665,18 @@ found:
         if ( split_debug )
             out() << "    " << thisLoc.toString() << ".split" << endl;
 
-        int mid = n / 2;
+        int split = n / 2;
+        if ( keypos == n ) { // see SERVER-983
+            split = 0.9 * n;
+            if ( split > n - 2 )
+                split = n - 2;
+        }
 
         DiskLoc rLoc = addBucket(idx);
         BtreeBucket *r = rLoc.btreemod();
         if ( split_debug )
-            out() << "     mid:" << mid << ' ' << keyNode(mid).key.toString() << " n:" << n << endl;
-        for ( int i = mid+1; i < n; i++ ) {
+            out() << "     split:" << split << ' ' << keyNode(split).key.toString() << " n:" << n << endl;
+        for ( int i = split+1; i < n; i++ ) {
             KeyNode kn = keyNode(i);
             r->pushBack(kn.recordLoc, kn.key, order, kn.prevChildBucket);
         }
@@ -640,18 +689,18 @@ found:
         rLoc.btree()->fixParentPtrs(rLoc);
 
         {
-            KeyNode middle = keyNode(mid);
-            nextChild = middle.prevChildBucket; // middle key gets promoted, its children will be thisLoc (l) and rLoc (r)
+            KeyNode splitkey = keyNode(split);
+            nextChild = splitkey.prevChildBucket; // splitkey key gets promoted, its children will be thisLoc (l) and rLoc (r)
             if ( split_debug ) {
-                out() << "    middle key:" << middle.key.toString() << endl;
+                out() << "    splitkey key:" << splitkey.key.toString() << endl;
             }
 
-            // promote middle to a parent node
+            // promote splitkey to a parent node
             if ( parent.isNull() ) {
                 // make a new parent if we were the root
                 DiskLoc L = addBucket(idx);
                 BtreeBucket *p = L.btreemod();
-                p->pushBack(middle.recordLoc, middle.key, order, thisLoc);
+                p->pushBack(splitkey.recordLoc, splitkey.key, order, thisLoc);
                 p->nextChild = rLoc;
                 p->assertValid( order );
                 parent = idx.head = L;
@@ -664,22 +713,22 @@ found:
                 */
                 rLoc.btreemod()->parent = parent;
                 if ( split_debug )
-                    out() << "    promoting middle key " << middle.key.toString() << endl;
-                parent.btree()->_insert(parent, middle.recordLoc, middle.key, order, /*dupsallowed*/true, thisLoc, rLoc, idx);
+                    out() << "    promoting splitkey key " << splitkey.key.toString() << endl;
+                parent.btree()->_insert(parent, splitkey.recordLoc, splitkey.key, order, /*dupsallowed*/true, thisLoc, rLoc, idx);
             }
         }
 
-        truncateTo(mid, order);  // note this may trash middle.key.  thus we had to promote it before finishing up here.
+        truncateTo(split, order);  // note this may trash splitkey.key.  thus we had to promote it before finishing up here.
 
         // add our new key, there is room now
         {
 
-            if ( keypos <= mid ) {
+            if ( keypos <= split ) {
                 if ( split_debug )
-                    out() << "  keypos<mid, insertHere() the new key" << endl;
+                    out() << "  keypos<split, insertHere() the new key" << endl;
                 insertHere(thisLoc, keypos, recordLoc, key, order, lchild, rchild, idx);
             } else {
-                int kp = keypos-mid-1;
+                int kp = keypos-split-1;
                 assert(kp>=0);
                 rLoc.btree()->insertHere(rLoc, kp, recordLoc, key, order, lchild, rchild, idx);
             }
@@ -804,19 +853,21 @@ found:
             _KeyNode& kn = k(pos);
             if ( kn.isUnused() ) {
                 log(4) << "btree _insert: reusing unused key" << endl;
-                massert("_insert: reuse key but lchild is not null", lChild.isNull());
-                massert("_insert: reuse key but rchild is not null", rChild.isNull());
+                massert( 10285 , "_insert: reuse key but lchild is not null", lChild.isNull());
+                massert( 10286 , "_insert: reuse key but rchild is not null", rChild.isNull());
                 kn.setUsed();
                 return 0;
             }
 
-            out() << "_insert(): key already exists in index\n";
-            out() << "  " << idx.indexNamespace().c_str() << " thisLoc:" << thisLoc.toString() << '\n';
-            out() << "  " << key.toString() << '\n';
-            out() << "  " << "recordLoc:" << recordLoc.toString() << " pos:" << pos << endl;
-            out() << "  old l r: " << childForPos(pos).toString() << ' ' << childForPos(pos+1).toString() << endl;
-            out() << "  new l r: " << lChild.toString() << ' ' << rChild.toString() << endl;
-            massert("btree: key+recloc already in index", false);
+            DEV { 
+                out() << "_insert(): key already exists in index (ok for background:true)\n";
+                out() << "  " << idx.indexNamespace().c_str() << " thisLoc:" << thisLoc.toString() << '\n';
+                out() << "  " << key.toString() << '\n';
+                out() << "  " << "recordLoc:" << recordLoc.toString() << " pos:" << pos << endl;
+                out() << "  old l r: " << childForPos(pos).toString() << ' ' << childForPos(pos+1).toString() << endl;
+                out() << "  new l r: " << lChild.toString() << ' ' << rChild.toString() << endl;
+            }
+            alreadyInIndex();
         }
 
         DEBUGGING out() << "TEMP: key: " << key.toString() << endl;
@@ -929,12 +980,11 @@ namespace mongo {
         b->k(1).setUnused();
 
         b->dumpTree(id.head, order);
-        cout << "---\n";
 
         b->bt_insert(id.head, A, key, order, false, id);
 
         b->dumpTree(id.head, order);
-        cout << "---\n";*/
+        */
 
         // this should assert.  does it? (it might "accidentally" though, not asserting proves a problem, asserting proves nothing)
         b->bt_insert(id.head, C, key, order, false, id);
@@ -964,10 +1014,10 @@ namespace mongo {
         if( !dupsAllowed ) {
             if( n > 0 ) {
                 int cmp = keyLast.woCompare(key, order);
-                massert( "bad key order in BtreeBuilder - server internal error", cmp <= 0 );
+                massert( 10288 ,  "bad key order in BtreeBuilder - server internal error", cmp <= 0 );
                 if( cmp == 0 ) {
                     //if( !dupsAllowed )
-                    uasserted( BtreeBucket::dupKeyError( idx , keyLast ) );
+                    uasserted( ASSERT_ID_DUPKEY , BtreeBucket::dupKeyError( idx , keyLast ) );
                 }
             }
             keyLast = key;
