@@ -46,6 +46,7 @@ namespace mongo {
            snapshot    - use $snapshot mode for copying collections.  note this should not be used when it isn't required, as it will be slower.
                          for example repairDatabase need not use it.
         */
+        void setConnection( DBClientWithCommands *c ) { conn.reset( c ); }
         bool go(const char *masterHost, string& errmsg, const string& fromdb, bool logForRepl, bool slaveOk, bool useReplAuth, bool snapshot);
         bool startCloneCollection( const char *fromhost, const char *ns, const BSONObj &query, string& errmsg, bool logForRepl, bool copyIndexes, int logSizeMb, long long &cursorId );
         bool finishCloneCollection( const char *fromhost, const char *ns, const BSONObj &query, long long cursorId, string &errmsg );
@@ -64,9 +65,9 @@ namespace mongo {
             if ( e.eoo() )
                 break;
             if ( string("ns") == e.fieldName() ) {
-                uassert("bad ns field for index during dbcopy", e.type() == String);
+                uassert( 10024 , "bad ns field for index during dbcopy", e.type() == String);
                 const char *p = strchr(e.valuestr(), '.');
-                uassert("bad ns field for index during dbcopy [2]", p);
+                uassert( 10025 , "bad ns field for index during dbcopy [2]", p);
                 string newname = cc().database()->name + p;
                 b.append("ns", newname);
             }
@@ -92,16 +93,16 @@ namespace mongo {
         auto_ptr<DBClientCursor> c;
         {
             dbtemprelease r;
-            c = conn->query( from_collection, query, 0, 0, 0, Option_NoCursorTimeout | ( slaveOk ? Option_SlaveOk : 0 ) );
+            c = conn->query( from_collection, query, 0, 0, 0, QueryOption_NoCursorTimeout | ( slaveOk ? QueryOption_SlaveOk : 0 ) );
         }
         
         list<BSONObj> storedForLater;
         
-        assert( c.get() );
+        massert( 13055 , "socket error in Cloner:copy" , c.get() );
         long long n = 0;
         time_t saveLast = time( 0 );
         while ( 1 ) {
-            {
+            if( !c->moreInCurrentBatch() || n % 128 == 127 /*yield some*/ ) {
                 dbtemprelease r;
                 if ( !c->more() )
                     break;
@@ -110,7 +111,17 @@ namespace mongo {
 
             /* assure object is valid.  note this will slow us down a little. */
             if ( !tmp.valid() ) {
-                out() << "skipping corrupt object from " << from_collection << '\n';
+                stringstream ss;
+                ss << "Cloner: skipping corrupt object from " << from_collection;
+                BSONElement e = tmp.firstElement();
+                try {
+                    e.validate();
+                    ss << " firstElement: " << e;
+                }
+                catch( ... ){
+                    ss << " firstElement corrupt";
+                }
+                out() << ss.str() << endl;
                 continue;
             }
 
@@ -156,7 +167,7 @@ namespace mongo {
     
     bool Cloner::go(const char *masterHost, string& errmsg, const string& fromdb, bool logForRepl, bool slaveOk, bool useReplAuth, bool snapshot) {
 
-		massert( "useReplAuth is not written to replication log", !useReplAuth || !logForRepl );
+		massert( 10289 ,  "useReplAuth is not written to replication log", !useReplAuth || !logForRepl );
 
         string todb = cc().database()->name;
         stringstream a,b;
@@ -181,7 +192,9 @@ namespace mongo {
 		
             auto_ptr<DBClientCursor> c;
             {
-                if ( !masterSameProcess ) {
+                if ( conn.get() ) {
+                    // nothing to do
+                } else if ( !masterSameProcess ) {
                     auto_ptr< DBClientConnection > c( new DBClientConnection() );
                     if ( !c->connect( masterHost, errmsg ) )
                         return false;
@@ -192,7 +205,7 @@ namespace mongo {
                 } else {
                     conn.reset( new DBDirectClient() );
                 }
-                c = conn->query( ns.c_str(), BSONObj(), 0, 0, 0, slaveOk ? Option_SlaveOk : 0 );
+                c = conn->query( ns.c_str(), BSONObj(), 0, 0, 0, slaveOk ? QueryOption_SlaveOk : 0 );
             }
 
             if ( c.get() == 0 ) {
@@ -205,10 +218,10 @@ namespace mongo {
 
                 log(2) << "\t cloner got " << collection << endl;
 
-                BSONElement e = collection.findElement("name");
+                BSONElement e = collection.getField("name");
                 if ( e.eoo() ) {
                     string s = "bad system.namespaces object " + collection.toString();
-                    massert(s.c_str(), false);
+                    massert( 10290 , s.c_str(), false);
                 }
                 assert( !e.eoo() );
                 assert( e.type() == String );
@@ -221,12 +234,11 @@ namespace mongo {
                         continue;
                     }
                 }
-                else if( strchr(from_name, '$') ) {
+                if( strchr(from_name, '$') ) {
                     // don't clone index namespaces -- we take care of those separately below.
                     log(2) << "\t\t not cloning because has $ " << endl;
                     continue;
                 }            
-                
                 toClone.push_back( collection.getOwned() );
             }
         }
@@ -271,7 +283,7 @@ namespace mongo {
 
     bool Cloner::startCloneCollection( const char *fromhost, const char *ns, const BSONObj &query, string &errmsg, bool logForRepl, bool copyIndexes, int logSizeMb, long long &cursorId ) {
         char db[256];
-        nsToClient( ns, db );
+        nsToDatabase( ns, db );
 
         NamespaceDetails *nsd = nsdetails( ns );
         if ( nsd ){
@@ -322,14 +334,14 @@ namespace mongo {
         {
             dbtemprelease r;
             string logNS = "local.temp.oplog." + string( ns );
-            c = conn->query( logNS.c_str(), Query(), 0, 0, 0, Option_CursorTailable );
+            c = conn->query( logNS.c_str(), Query(), 0, 0, 0, QueryOption_CursorTailable );
         }
         if ( c->more() ) {
             replayOpLog( c.get(), query );
             cursorId = c->getCursorId();
-            massert( "Expected valid tailing cursor", cursorId != 0 );
+            massert( 10291 ,  "Expected valid tailing cursor", cursorId != 0 );
         } else {
-            massert( "Did not expect valid cursor for empty query result", c->getCursorId() == 0 );
+            massert( 10292 ,  "Did not expect valid cursor for empty query result", c->getCursorId() == 0 );
             cursorId = 0;
         }
         c->decouple();
@@ -337,7 +349,7 @@ namespace mongo {
     }
     
     void Cloner::replayOpLog( DBClientCursor *c, const BSONObj &query ) {
-        JSMatcher matcher( query );
+        Matcher matcher( query );
         while( 1 ) {
             BSONObj op;
             {
@@ -355,7 +367,7 @@ namespace mongo {
     
     bool Cloner::finishCloneCollection( const char *fromhost, const char *ns, const BSONObj &query, long long cursorId, string &errmsg ) {
         char db[256];
-        nsToClient( ns, db );
+        nsToDatabase( ns, db );
 
         auto_ptr< DBClientCursor > cur;
         {
@@ -404,6 +416,7 @@ namespace mongo {
         virtual bool slaveOk() {
             return false;
         }
+        virtual LockType locktype(){ return WRITE; }
         virtual void help( stringstream &help ) const {
             help << "clone this database from an instance of the db on another host\n";
             help << "example: { clone : \"host13\" }";
@@ -426,6 +439,7 @@ namespace mongo {
         virtual bool slaveOk() {
             return false;
         }
+        virtual LockType locktype(){ return WRITE; }
         CmdCloneCollection() : Command("cloneCollection") { }
         virtual void help( stringstream &help ) const {
             help << " example: { cloneCollection: <collection ns>, from: <hostname>, query: <query> }";
@@ -452,9 +466,9 @@ namespace mongo {
             /* replication note: we must logOp() not the command, but the cloned data -- if the slave
              were to clone it would get a different point-in-time and not match.
              */
-            setClient( collection.c_str() );
+            Client::Context ctx( collection );
             
-            log() << "cloneCollection.  db:" << ns << " collection:" << collection << " from: " << fromhost << " query: " << query << ( copyIndexes ? "" : ", not copying indexes" ) << endl;
+            log() << "cloneCollection.  db:" << ns << " collection:" << collection << " from: " << fromhost << " query: " << query << " logSizeMb: " << logSizeMb << ( copyIndexes ? "" : ", not copying indexes" ) << endl;
             
             Cloner c;
             long long cursorId;
@@ -469,6 +483,7 @@ namespace mongo {
         virtual bool slaveOk() {
             return false;
         }
+        virtual LockType locktype(){ return WRITE; }
         CmdStartCloneCollection() : Command("startCloneCollection") { }
         virtual void help( stringstream &help ) const {
             help << " example: { startCloneCollection: <collection ns>, from: <hostname>, query: <query> }";
@@ -496,7 +511,7 @@ namespace mongo {
             /* replication note: we must logOp() not the command, but the cloned data -- if the slave
              were to clone it would get a different point-in-time and not match.
              */
-            setClient( collection.c_str() );
+            Client::Context ctx(collection);
             
             log() << "startCloneCollection.  db:" << ns << " collection:" << collection << " from: " << fromhost << " query: " << query << endl;
             
@@ -522,6 +537,7 @@ namespace mongo {
         virtual bool slaveOk() {
             return false;
         }
+        virtual LockType locktype(){ return WRITE; }
         CmdFinishCloneCollection() : Command("finishCloneCollection") { }
         virtual void help( stringstream &help ) const {
             help << " example: { finishCloneCollection: <finishToken> }";
@@ -549,10 +565,10 @@ namespace mongo {
             long long cursorId = 0;
             BSONElement cursorIdToken = fromToken.getField( "cursorId" );
             if ( cursorIdToken.type() == Date ) {
-                cursorId = cursorIdToken.date();
+                cursorId = cursorIdToken._numberLong();
             }
             
-            setClient( collection.c_str() );
+            Client::Context ctx( collection );
             
             log() << "finishCloneCollection.  db:" << ns << " collection:" << collection << " from: " << fromhost << " query: " << query << endl;
             
@@ -561,8 +577,50 @@ namespace mongo {
         }
     } cmdfinishclonecollection;
 
+    thread_specific_ptr< DBClientConnection > authConn_;
     /* Usage:
-       admindb.$cmd.findOne( { copydb: 1, fromhost: <hostname>, fromdb: <db>, todb: <db> } );
+     admindb.$cmd.findOne( { copydbgetnonce: 1, fromhost: <hostname> } );
+     */
+    class CmdCopyDbGetNonce : public Command {
+    public:
+        CmdCopyDbGetNonce() : Command("copydbgetnonce") { }
+        virtual bool adminOnly() {
+            return true;
+        }
+        virtual bool slaveOk() {
+            return false;
+        }
+        virtual LockType locktype(){ return WRITE; }
+        virtual void help( stringstream &help ) const {
+            help << "get a nonce for subsequent copy db request from secure server\n";
+            help << "usage: {copydbgetnonce: 1, fromhost: <hostname>}";
+        }
+        virtual bool run(const char *ns, BSONObj& cmdObj, string& errmsg, BSONObjBuilder& result, bool fromRepl) {
+            string fromhost = cmdObj.getStringField("fromhost");
+            if ( fromhost.empty() ) {
+                /* copy from self */
+                stringstream ss;
+                ss << "localhost:" << cmdLine.port;
+                fromhost = ss.str();
+            }
+            authConn_.reset( new DBClientConnection() );
+            BSONObj ret;
+            {
+                dbtemprelease t;
+                if ( !authConn_->connect( fromhost, errmsg ) )
+                    return false;
+                if( !authConn_->runCommand( "admin", BSON( "getnonce" << 1 ), ret ) ) {
+                    errmsg = "couldn't get nonce " + string( ret );
+                    return false;
+                }
+            }
+            result.appendElements( ret );
+            return true;
+        }
+    } cmdcopydbgetnonce;
+
+    /* Usage:
+       admindb.$cmd.findOne( { copydb: 1, fromhost: <hostname>, fromdb: <db>, todb: <db>[, username: <username>, nonce: <nonce>, key: <key>] } );
     */
     class CmdCopyDb : public Command {
     public:
@@ -573,9 +631,10 @@ namespace mongo {
         virtual bool slaveOk() {
             return false;
         }
+        virtual LockType locktype(){ return WRITE; }
         virtual void help( stringstream &help ) const {
-            help << "copy a database from antoher host to this host\n";
-            help << "usage: {copydb: 1, fromhost: <hostname>, fromdb: <db>, todb: <db>}";
+            help << "copy a database from another host to this host\n";
+            help << "usage: {copydb: 1, fromhost: <hostname>, fromdb: <db>, todb: <db>[, username: <username>, nonce: <nonce>, key: <key>]}";
         }
         virtual bool run(const char *ns, BSONObj& cmdObj, string& errmsg, BSONObjBuilder& result, bool fromRepl) {
             string fromhost = cmdObj.getStringField("fromhost");
@@ -591,9 +650,24 @@ namespace mongo {
                 errmsg = "parms missing - {copydb: 1, fromhost: <hostname>, fromdb: <db>, todb: <db>}";
                 return false;
             }
-            setClient(todb.c_str());
-            bool res = cloneFrom(fromhost.c_str(), errmsg, fromdb, /*logForReplication=*/!fromRepl, /*slaveok*/false, /*replauth*/false, /*snapshot*/true);
-            cc().clearns();
+            Cloner c;
+            string username = cmdObj.getStringField( "username" );
+            string nonce = cmdObj.getStringField( "nonce" );
+            string key = cmdObj.getStringField( "key" );
+            if ( !username.empty() && !nonce.empty() && !key.empty() ) {
+                uassert( 13008, "must call copydbgetnonce first", authConn_.get() );
+                BSONObj ret;
+                {
+                    dbtemprelease t;
+                    if ( !authConn_->runCommand( fromdb, BSON( "authenticate" << 1 << "user" << username << "nonce" << nonce << "key" << key ), ret ) ) {
+                        errmsg = "unable to login " + string( ret );
+                        return false;
+                    }
+                }
+                c.setConnection( authConn_.release() );
+            }
+            Client::Context ctx(todb);
+            bool res = c.go(fromhost.c_str(), errmsg, fromdb, /*logForReplication=*/!fromRepl, /*slaveok*/false, /*replauth*/false, /*snapshot*/true);
             return res;
         }
     } cmdcopydb;
@@ -607,6 +681,7 @@ namespace mongo {
         virtual bool slaveOk() {
             return false;
         }
+        virtual LockType locktype(){ return WRITE; }
         virtual bool logTheOp() {
             return true; // can't log steps when doing fast rename within a db, so always log the op rather than individual steps comprising it.
         }
@@ -621,23 +696,34 @@ namespace mongo {
                 return false;
             }
             
-            setClient( source.c_str() );
-            NamespaceDetails *nsd = nsdetails( source.c_str() );
-            uassert( "source namespace does not exist", nsd );
-            bool capped = nsd->capped;
+            bool capped = false;
             long long size = 0;
-            if ( capped )
-                for( DiskLoc i = nsd->firstExtent; !i.isNull(); i = i.ext()->xnext )
-                    size += i.ext()->length;
+            {
+                Client::Context ctx( source );
+                NamespaceDetails *nsd = nsdetails( source.c_str() );
+                uassert( 10026 ,  "source namespace does not exist", nsd );
+                capped = nsd->capped;
+                if ( capped )
+                    for( DiskLoc i = nsd->firstExtent; !i.isNull(); i = i.ext()->xnext )
+                        size += i.ext()->length;
+            }
             
-            setClient( target.c_str() );
-            uassert( "target namespace exists", !nsdetails( target.c_str() ) );
+            Client::Context ctx( target );
+            
+            if ( nsdetails( target.c_str() ) ){
+                uassert( 10027 ,  "target namespace exists", cmdObj["dropTarget"].trueValue() );
+                BSONObjBuilder bb( result.subobjStart( "dropTarget" ) );
+                dropCollection( target , errmsg , bb );
+                bb.done();
+                if ( errmsg.size() > 0 )
+                    return false;
+            }
 
             {
                 char from[256];
-                nsToClient( source.c_str(), from );
+                nsToDatabase( source.c_str(), from );
                 char to[256];
-                nsToClient( target.c_str(), to );
+                nsToDatabase( target.c_str(), to );
                 if ( strcmp( from, to ) == 0 ) {
                     renameNamespace( source.c_str(), target.c_str() );
                     return true;
@@ -668,9 +754,9 @@ namespace mongo {
             }
             
             char cl[256];
-            nsToClient( source.c_str(), cl );
+            nsToDatabase( source.c_str(), cl );
             string sourceIndexes = string( cl ) + ".system.indexes";
-            nsToClient( target.c_str(), cl );
+            nsToDatabase( target.c_str(), cl );
             string targetIndexes = string( cl ) + ".system.indexes";
             {
                 c = bridge.query( sourceIndexes, QUERY( "ns" << source ) );
@@ -697,8 +783,10 @@ namespace mongo {
                 theDataFileMgr.insert( targetIndexes.c_str(), n );
             }
 
-            setClient( source.c_str() );
-            dropCollection( source, errmsg, result );
+            {
+                Client::Context ctx( source );
+                dropCollection( source, errmsg, result );
+            }
             return true;
         }
     } cmdrenamecollection;
