@@ -29,6 +29,7 @@
 #include "instance.h"
 #include "clientcursor.h"
 #include "pdfile.h"
+#include "stats/counters.h"
 #if !defined(_WIN32)
 #include <sys/file.h>
 #endif
@@ -40,9 +41,12 @@
 #include "../scripting/engine.h"
 #include "module.h"
 #include "cmdline.h"
+#include "stats/snapshots.h"
 
 namespace mongo {
 
+    CmdLine cmdLine;
+
     bool useJNI = true;
 
     /* only off if --nocursors which is for debugging. */
@@ -54,10 +58,11 @@ namespace mongo {
 
     extern string bind_ip;
     extern char *appsrvPath;
-    extern bool autoresync;
     extern int diagLogging;
     extern int lenForNewNsFiles;
     extern int lockFile;
+    
+    extern string repairpath;
 
     void setupSignals();
     void closeAllSockets();
@@ -65,9 +70,14 @@ namespace mongo {
     void pairWith(const char *remoteEnd, const char *arb);
     void setRecCacheSize(unsigned MB);
 
+    void exitCleanly( ExitCode code );
+
     const char *ourgetns() { 
         Client *c = currentClient.get();
-        return c ? c->ns() : "";
+        if ( ! c )
+            return "";
+        Client::Context* cc = c->getContext();
+        return cc ? cc->ns() : "";
     }
 
     struct MyStartupTests {
@@ -79,9 +89,8 @@ namespace mongo {
     QueryResult* emptyMoreResult(long long);
 
     void testTheDb() {
-        stringstream ss;
-
-        setClient("sys.unittest.pdfile");
+        OpDebug debug;
+        Client::Context ctx("sys.unittest.pdfile");
 
         /* this is not validly formatted, if you query this namespace bad things will happen */
         theDataFileMgr.insert("sys.unittest.pdfile", (void *) "hello worldx", 13);
@@ -91,8 +100,8 @@ namespace mongo {
         deleteObjects("sys.unittest.delete", j1, false);
         theDataFileMgr.insert("sys.unittest.delete", &js1, sizeof(js1));
         deleteObjects("sys.unittest.delete", j1, false);
-        updateObjects("sys.unittest.delete", j1, j1, true,false,ss,true);
-        updateObjects("sys.unittest.delete", j1, j1, false,false,ss,true);
+        updateObjects("sys.unittest.delete", j1, j1, true,false,true,debug);
+        updateObjects("sys.unittest.delete", j1, j1, false,false,true,debug);
 
         auto_ptr<Cursor> c = theDataFileMgr.findAll("sys.unittest.pdfile");
         while ( c->ok() ) {
@@ -100,40 +109,47 @@ namespace mongo {
             c->advance();
         }
         out() << endl;
-
-        cc().clearns();
     }
 
-    MessagingPort *grab = 0;
+    MessagingPort *connGrab = 0;
     void connThread();
 
     class OurListener : public Listener {
     public:
         OurListener(const string &ip, int p) : Listener(ip, p) { }
         virtual void accepted(MessagingPort *mp) {
-            assert( grab == 0 );
-            grab = mp;
+            assert( connGrab == 0 );
+            if ( ! connTicketHolder.tryAcquire() ){
+                log() << "connection refused because too many open connections" << endl;
+                // TODO: would be nice if we notified them...
+                mp->shutdown();
+                return;
+            }
+            connGrab = mp;
             try {
                 boost::thread thr(connThread);
-                while ( grab )
+                while ( connGrab )
                     sleepmillis(1);
             }
-            catch ( boost::thread_resource_error& e ){
+            catch ( boost::thread_resource_error& ){
                 log() << "can't create new thread, closing connection" << endl;
                 mp->shutdown();
-                grab = 0;
+                connGrab = 0;
+            }
+            catch ( ... ){
+                log() << "unkonwn exception starting connThread" << endl;
+                mp->shutdown();
+                connGrab = 0;
             }
         }
     };
 
     void webServerThread();
-    void pdfileInit();
 
     void listen(int port) {
         log() << mongodVersion() << endl;
         printGitVersion();
         printSysInfo();
-        pdfileInit();
         //testTheDb();
         log() << "waiting for connections on port " << port << endl;
         OurListener l(bind_ip, port);
@@ -141,7 +157,7 @@ namespace mongo {
         if ( !noHttpInterface )
             boost::thread thr(webServerThread);
         if ( l.init() ) {
-            registerListenerSocket( l.socket() );
+            ListeningSockets::get()->add( l.socket() );
             l.listen();
         }
     }
@@ -170,38 +186,45 @@ namespace mongo {
     */
     void connThread()
     {
+        TicketHolderReleaser connTicketReleaser( &connTicketHolder );
         Client::initThread("conn");
 
         /* todo: move to Client object */
         LastError *le = new LastError();
         lastError.reset(le);
 
-        MessagingPort& dbMsgPort = *grab;
-        grab = 0;
+        auto_ptr<MessagingPort> dbMsgPort( connGrab );
+        connGrab = 0;
+        Client& c = cc();
 
         try {
 
-            currentClient.get()->ai->isLocalHost = dbMsgPort.farEnd.isLocalHost();
+            c.getAuthenticationInfo()->isLocalHost = dbMsgPort->farEnd.isLocalHost();
 
             Message m;
             while ( 1 ) {
                 m.reset();
 
-                if ( !dbMsgPort.recv(m) ) {
+                if ( !dbMsgPort->recv(m) ) {
                     if( !cmdLine.quiet )
-                    log() << "end connection " << dbMsgPort.farEnd.toString() << endl;
-                    dbMsgPort.shutdown();
+                      log() << "end connection " << dbMsgPort->farEnd.toString() << endl;
+                    dbMsgPort->shutdown();
                     break;
                 }
 
+                if ( inShutdown() ) {
+                    log() << "got request after shutdown()" << endl;
+                    break;
+                }
+                
                 lastError.startRequest( m , le );
 
                 DbResponse dbresponse;
-                if ( !assembleResponse( m, dbresponse, dbMsgPort.farEnd.sa ) ) {
-                    out() << curTimeMillis() % 10000 << "   end msg " << dbMsgPort.farEnd.toString() << endl;
+                if ( !assembleResponse( m, dbresponse, dbMsgPort->farEnd.sa ) ) {
+                    out() << curTimeMillis() % 10000 << "   end msg " << dbMsgPort->farEnd.toString() << endl;
                     /* todo: we may not wish to allow this, even on localhost: very low priv accounts could stop us. */
-                    if ( dbMsgPort.farEnd.isLocalHost() ) {
-                        dbMsgPort.shutdown();
+                    if ( dbMsgPort->farEnd.isLocalHost() ) {
+                        dbMsgPort->shutdown();
                         sleepmillis(50);
                         problem() << "exiting end msg" << endl;
                         dbexit(EXIT_CLEAN);
@@ -212,18 +235,21 @@ namespace mongo {
                 }
 
                 if ( dbresponse.response )
-                    dbMsgPort.reply(m, *dbresponse.response, dbresponse.responseTo);
+                    dbMsgPort->reply(m, *dbresponse.response, dbresponse.responseTo);
             }
 
         }
         catch ( AssertionException& ) {
             problem() << "AssertionException in connThread, closing client connection" << endl;
-            dbMsgPort.shutdown();
+            dbMsgPort->shutdown();
         }
         catch ( SocketException& ) {
             problem() << "SocketException in connThread, closing client connection" << endl;
-            dbMsgPort.shutdown();
+            dbMsgPort->shutdown();
         }
+        catch ( const ClockSkewException & ) {
+            exitCleanly( EXIT_CLOCK_SKEW );
+        }        
         catch ( std::exception &e ) {
             problem() << "Uncaught std::exception: " << e.what() << ", terminating" << endl;
             dbexit( EXIT_UNCAUGHT );
@@ -251,8 +277,10 @@ namespace mongo {
 //  SockAddr db("172.16.0.179", MessagingPort::DBPort);
 
         MessagingPort p;
-        if ( !p.connect(db) )
+        if ( !p.connect(db) ){
+            out() << "msg couldn't connect" << endl;
             return;
+        }
 
         const int Loops = 1;
         for ( int q = 0; q < Loops; q++ ) {
@@ -268,8 +296,9 @@ namespace mongo {
             Timer t;
             bool ok = p.call(send, response);
             double tm = ((double) t.micros()) + 1;
-            out() << " ****ok. response.data:" << ok << " time:" << tm / 1000.0 << "ms " <<
-                 ((double) len) * 8 / 1000000 / (tm/1000000) << "Mbps" << endl;
+            out() << " ****ok. response.data:" << ok << " time:" << tm / 1000.0 << "ms "
+                  << "len: " << len << " data: " << response.data->_data << endl;
+
             if (  q+1 < Loops ) {
                 out() << "\t\tSLEEP 8 then sending again as a test" << endl;
                 sleepsecs(8);
@@ -290,7 +319,7 @@ namespace mongo {
     bool doDBUpgrade( const string& dbName , string errmsg , MDFHeader * h ){
         static DBDirectClient db;
         
-        if ( forceRepair == false && h->version == 4 && h->versionMinor == 4 ){
+        if ( h->version == 4 && h->versionMinor == 4 ){
             assert( VERSION == 4 );
             assert( VERSION_MINOR == 5 );
             
@@ -315,15 +344,22 @@ namespace mongo {
         return repairDatabase( dbName.c_str(), errmsg );
     }
     
+    extern bool checkNsFilesOnLoad;
+
     void repairDatabases() {
+        Client::GodScope gs;
         log(1) << "enter repairDatabases" << endl;
+        
+        assert(checkNsFilesOnLoad);
+        checkNsFilesOnLoad = false; // we are mainly just checking the header - don't scan the whole .ns file for every db here.
+
         dblock lk;
         vector< string > dbNames;
         getDatabaseNames( dbNames );
         for ( vector< string >::iterator i = dbNames.begin(); i != dbNames.end(); ++i ) {
             string dbName = *i;
             log(1) << "\t" << dbName << endl;
-            assert( !setClient( dbName.c_str() ) );
+            Client::Context ctx( dbName );
             MongoDataFile *p = cc().database()->getFile( 0 );
             MDFHeader *h = p->getHeader();
             if ( !h->currentVersion() || forceRepair ) {
@@ -346,7 +382,7 @@ namespace mongo {
                     return;
                 }
             } else {
-                closeClient( dbName.c_str() );
+                closeDatabase( dbName.c_str() );
             }
         }
 
@@ -357,6 +393,8 @@ namespace mongo {
             cc().shutdown();
             dbexit( EXIT_CLEAN );
         }
+
+        checkNsFilesOnLoad = true;
     }
 
     void clearTmpFiles() {
@@ -365,39 +403,52 @@ namespace mongo {
                 i != boost::filesystem::directory_iterator(); ++i ) {
             string fileName = boost::filesystem::path(*i).leaf();
             if ( boost::filesystem::is_directory( *i ) &&
-                    fileName.length() > 2 && fileName.substr( 0, 3 ) == "tmp" )
+                fileName.length() && fileName[ 0 ] == '$' )
                 boost::filesystem::remove_all( *i );
         }
     }
-
+    
     void clearTmpCollections() {
+        Client::GodScope gs;
         vector< string > toDelete;
         DBDirectClient cli;
         auto_ptr< DBClientCursor > c = cli.query( "local.system.namespaces", Query( fromjson( "{name:/^local.temp./}" ) ) );
-        while( c->more() )
-            toDelete.push_back( c->next().getStringField( "name" ) );
+        while( c->more() ) {
+            BSONObj o = c->next();
+            toDelete.push_back( o.getStringField( "name" ) );
+        }
         for( vector< string >::iterator i = toDelete.begin(); i != toDelete.end(); ++i ) {
             log() << "Dropping old temporary collection: " << *i << endl;
             cli.dropCollection( *i );
         }
     }
-
+    
     /**
      * does background async flushes of mmapped files
      */
     class DataFileSync : public BackgroundJob {
     public:
         void run(){
+            if ( _sleepsecs > 2100 )
+                _sleepsecs = 2100;
             log(1) << "will flush memory every: " << _sleepsecs << " seconds" << endl;
+            int time_flushing = 0;
             while ( ! inShutdown() ){
                 if ( _sleepsecs == 0 ){
                     // in case at some point we add an option to change at runtime
                     sleepsecs(5);
                     continue;
                 }
-                sleepmillis( (int)(_sleepsecs * 1000) );
-                MemoryMappedFile::flushAll( false );
-                log(1) << "flushing mmmap" << endl;
+
+                sleepmillis( (int)(std::max(0.0, (_sleepsecs * 1000) - time_flushing)) );
+
+                Date_t start = jsTime();
+                MemoryMappedFile::flushAll( true );
+                time_flushing = (int) (jsTime() - start);
+
+                globalFlushCounters.flushed(time_flushing);
+
+                log(1) << "flushing mmap took " << time_flushing << "ms" << endl;
             }
         }
         
@@ -406,8 +457,7 @@ namespace mongo {
 
     void show_32_warning(){
 #if BOOST_VERSION < 103500
-#warning built with boost version <= 1.34, limited concurrency
-        //cout << \n** NOTE: built with boost version <= 1.34, limited concurrency" << endl;
+        cout << "\nwarning: built with boost version <= 1.34, limited concurrency" << endl;
 #endif
 
         if ( sizeof(int*) != 4 )
@@ -432,14 +482,21 @@ namespace mongo {
         bool is32bit = sizeof(int*) == 4;
 
         log() << "Mongo DB : starting : pid = " << pid << " port = " << cmdLine.port << " dbpath = " << dbpath
-              <<  " master = " << master << " slave = " << (int) slave << "  " << ( is32bit ? "32" : "64" ) << "-bit " << endl;
-
+              <<  " master = " << replSettings.master << " slave = " << (int) replSettings.slave << "  " << ( is32bit ? "32" : "64" ) << "-bit " << endl;
+        DEV log() << " FULL DEBUG ENABLED " << endl;
         show_32_warning();
 
-        stringstream ss;
-        ss << "dbpath (" << dbpath << ") does not exist";
-        massert( ss.str().c_str(), boost::filesystem::exists( dbpath ) );
-
+        {
+            stringstream ss;
+            ss << "dbpath (" << dbpath << ") does not exist";
+            massert( 10296 ,  ss.str().c_str(), boost::filesystem::exists( dbpath ) );
+        }
+        {
+            stringstream ss;
+            ss << "repairpath (" << repairpath << ") does not exist";
+            massert( 12590 ,  ss.str().c_str(), boost::filesystem::exists( repairpath ) );
+        }
+        
         acquirePathLock();
         remove_all( dbpath + "/_tmp/" );
 
@@ -448,11 +505,10 @@ namespace mongo {
         BOOST_CHECK_EXCEPTION( clearTmpFiles() );
 
         Client::initThread("initandlisten");
+        _diaglog.init();
 
         clearTmpCollections();
 
-        _diaglog.init();
-
         Module::initAll();
 
 #if 0
@@ -468,12 +524,19 @@ namespace mongo {
         }
 
         repairDatabases();
+
+        /* we didn't want to pre-open all fiels for the repair check above. for regular
+           operation we do for read/write lock concurrency reasons.
+        */        
+        Database::_openAllFiles = true;
+
         if ( shouldRepairDatabases )
             return;
 
         /* this is for security on certain platforms (nonce generation) */
         srand((unsigned) (curTimeMicros() ^ startupSrandTimer.micros()));
 
+        snapshotThread.go();
         listen(listenPort);
 
         // listen() will return when exit code closes its socket.
@@ -538,6 +601,7 @@ string arg_error_check(int argc, char* argv[]) {
 
 int main(int argc, char* argv[], char *envp[] )
 {
+    static StaticObserver staticObserver;
     getcurns = ourgetns;
 
     po::options_description general_options("General options");
@@ -545,25 +609,17 @@ int main(int argc, char* argv[], char *envp[] )
     po::options_description sharding_options("Sharding options");
     po::options_description visible_options("Allowed options");
     po::options_description hidden_options("Hidden options");
-    po::options_description cmdline_options("Command line options");
 
     po::positional_options_description positional_options;
 
+    CmdLine::addGlobalOptions( general_options , hidden_options );
+
     general_options.add_options()
-        ("help,h", "show this usage information")
-        ("version", "show version information")
-        ("config,f", po::value<string>(), "configuration file specifying additional options")
-        ("port", po::value<int>(&cmdLine.port)/*->default_value(CmdLine::DefaultDBPort)*/, "specify port number")
         ("bind_ip", po::value<string>(&bind_ip),
          "local ip address to bind listener - all local ips bound by default")
-        ("verbose,v", "be more verbose (include multiple times for more verbosity e.g. -vvvvv)")
         ("dbpath", po::value<string>()->default_value("/data/db/"), "directory for datafiles")
-        ("quiet", "quieter output")
-        ("logpath", po::value<string>() , "file to send all output to instead of stdout" )
-        ("logappend" , "appnd to logpath instead of over-writing" )
-#ifndef _WIN32
-        ("fork" , "fork server process" )
-#endif
+        ("directoryperdb", "each database will be stored in a separate directory")
+        ("repairpath", po::value<string>() , "root directory for repair files - defaults to dbpath" )
         ("cpu", "periodically show cpu and iowait utilization")
         ("noauth", "run without security")
         ("auth", "run with security")
@@ -574,6 +630,7 @@ int main(int argc, char* argv[], char *envp[] )
         ("nocursors", "diagnostic/debugging option")
         ("nohints", "ignore query hints")
         ("nohttpinterface", "disable http interface")
+        ("rest","turn on simple rest api")
         ("noscripting", "disable scripting engine")
         ("noprealloc", "disable data file preallocation")
         ("smallfiles", "use a smaller default file size")
@@ -584,6 +641,9 @@ int main(int argc, char* argv[], char *envp[] )
         ("repair", "run repair on all dbs")
         ("notablescan", "do not allow table scans")
         ("syncdelay",po::value<double>(&dataFileSync._sleepsecs)->default_value(60), "seconds between disk syncs (0 for never)")
+        ("profile",po::value<int>(), "0=off 1=slow, 2=all")
+        ("slowms",po::value<int>(&cmdLine.slowMS)->default_value(100), "value of slow for profile and console log" )
+        ("maxConns",po::value<int>(), "max number of simultaneous connections")
 #if defined(_WIN32)
         ("install", "install mongodb service")
         ("remove", "remove mongodb service")
@@ -598,8 +658,10 @@ int main(int argc, char* argv[], char *envp[] )
         ("only", po::value<string>(), "when slave: specify a single database to replicate")
         ("pairwith", po::value<string>(), "address of server to pair with")
         ("arbiter", po::value<string>(), "address of arbiter server")
+        ("slavedelay", po::value<int>(), "specify delay (in seconds) to be used when applying master ops to slave")
+        ("fastsync", "indicate that this instance is starting from a dbpath snapshot of the repl peer")
         ("autoresync", "automatically resync if slave data is stale")
-        ("oplogSize", po::value<long>(), "size limit (in MB) for op log")
+        ("oplogSize", po::value<int>(), "size limit (in MB) for op log")
         ("opIdMem", po::value<long>(), "size limit (in bytes) for in memory storage of op ids")
         ;
 
@@ -613,18 +675,12 @@ int main(int argc, char* argv[], char *envp[] )
         ("cacheSize", po::value<long>(), "cache size (in MB) for rec store")
         ;
 
-    /* support for -vv -vvvv etc. */
-    for (string s = "vv"; s.length() <= 10; s.append("v")) {
-        hidden_options.add_options()(s.c_str(), "verbose");
-    }
 
     positional_options.add("command", 3);
     visible_options.add(general_options);
     visible_options.add(replication_options);
     visible_options.add(sharding_options);
     Module::addOptions( visible_options );
-    cmdline_options.add(visible_options);
-    cmdline_options.add(hidden_options);
 
     setupSignals();
 
@@ -655,7 +711,7 @@ int main(int argc, char* argv[], char *envp[] )
         bool removeService = false;
         bool startService = false;
         po::variables_map params;
-
+        
         string error_message = arg_error_check(argc, argv);
         if (error_message != "") {
             cout << error_message << endl << endl;
@@ -663,37 +719,9 @@ int main(int argc, char* argv[], char *envp[] )
             return 0;
         }
 
-        /* don't allow guessing - creates ambiguities when some options are
-         * prefixes of others. allow long disguises and don't allow guessing
-         * to get away with our vvvvvvv trick. */
-        int command_line_style = (((po::command_line_style::unix_style ^
-                                    po::command_line_style::allow_guessing) |
-                                   po::command_line_style::allow_long_disguise) ^
-                                  po::command_line_style::allow_sticky);
 
-        try {
-            po::store(po::command_line_parser(argc, argv).options(cmdline_options).
-                      positional(positional_options).
-                      style(command_line_style).run(), params);
-
-            if (params.count("config")) {
-                ifstream config_file (params["config"].as<string>().c_str());
-                if (config_file.is_open()) {
-                    po::store(po::parse_config_file(config_file, cmdline_options), params);
-                    config_file.close();
-                } else {
-                    cout << "ERROR: could not read from config file" << endl << endl;
-                    cout << visible_options << endl;
-                    return 0;
-                }
-            }
-
-            po::notify(params);
-        } catch (po::error &e) {
-            cout << "ERROR: " << e.what() << endl << endl;
-            cout << visible_options << endl;
+        if ( ! CmdLine::store( argc , argv , visible_options , hidden_options , positional_options , params ) )
             return 0;
-        }
 
         if (params.count("help")) {
             show_help_text(visible_options);
@@ -705,16 +733,8 @@ int main(int argc, char* argv[], char *envp[] )
             return 0;
         }
         dbpath = params["dbpath"].as<string>();
-        if (params.count("quiet")) {
-            cmdLine.quiet = true;
-        }
-        if (params.count("verbose")) {
-            logLevel = 1;
-        }
-        for (string s = "vv"; s.length() <= 10; s.append("v")) {
-            if (params.count(s)) {
-                logLevel = s.length();
-            }
+        if ( params.count("directoryperdb")) {
+            directoryperdb = true;
         }
         if (params.count("cpu")) {
             cmdLine.cpu = true;
@@ -739,25 +759,11 @@ int main(int argc, char* argv[], char *envp[] )
             /* casting away the const-ness here */
             appsrvPath = (char*)(params["appsrvpath"].as<string>().c_str());
         }
-#ifndef _WIN32
-        if (params.count("fork")) {
-            if ( ! params.count( "logpath" ) ){
-                cout << "--fork has to be used with --logpath" << endl;
-                return -1;
-            }
-            pid_t c = fork();
-            if ( c ){
-                cout << "forked process: " << c << endl;
-                ::exit(0);
-            }
-            setsid();
-            setupSignals();
-        }
-#endif
-        if (params.count("logpath")) {
-            string lp = params["logpath"].as<string>();
-            uassert( "logpath has to be non-zero" , lp.size() );
-            initLogging( lp , params.count( "logappend" ) );
+        if (params.count("repairpath")) {
+            repairpath = params["repairpath"].as<string>();
+            uassert( 12589, "repairpath has to be non-zero", repairpath.size() );
+        } else {
+            repairpath = dbpath;
         }
         if (params.count("nocursors")) {
             useCursors = false;
@@ -768,6 +774,9 @@ int main(int argc, char* argv[], char *envp[] )
         if (params.count("nohttpinterface")) {
             noHttpInterface = true;
         }
+        if (params.count("rest")) {
+            cmdLine.rest = true;
+        }
         if (params.count("noscripting")) {
             useJNI = false;
         }
@@ -809,10 +818,19 @@ int main(int argc, char* argv[], char *envp[] )
             startService = true;
         }
         if (params.count("master")) {
-            master = true;
+            replSettings.master = true;
         }
         if (params.count("slave")) {
-            slave = SimpleSlave;
+            replSettings.slave = SimpleSlave;
+        }
+        if (params.count("slavedelay")) {
+            replSettings.slavedelay = params["slavedelay"].as<int>();
+        }
+        if (params.count("fastsync")) {
+            replSettings.fastsync = true;
+        }
+        if (params.count("autoresync")) {
+            replSettings.autoresync = true;
         }
         if (params.count("source")) {
             /* specifies what the source in local.sources should be */
@@ -830,32 +848,29 @@ int main(int argc, char* argv[], char *envp[] )
                 pairWith(paired.c_str(), "-");
             }
         } else if (params.count("arbiter")) {
-            uasserted("specifying --arbiter without --pairwith");
-        }
-        if (params.count("autoresync")) {
-            autoresync = true;
+            uasserted(10999,"specifying --arbiter without --pairwith");
         }
         if( params.count("nssize") ) {
             int x = params["nssize"].as<int>();
-            uassert("bad --nssize arg", x > 0 && x <= (0x7fffffff/1024/1024));
+            uassert( 10034 , "bad --nssize arg", x > 0 && x <= (0x7fffffff/1024/1024));
             lenForNewNsFiles = x * 1024 * 1024;
             assert(lenForNewNsFiles > 0);
         }
         if (params.count("oplogSize")) {
-            long x = params["oplogSize"].as<long>();
-            uassert("bad --oplogSize arg", x > 0);
+            long x = params["oplogSize"].as<int>();
+            uassert( 10035 , "bad --oplogSize arg", x > 0);
             cmdLine.oplogSize = x * 1024 * 1024;
             assert(cmdLine.oplogSize > 0);
         }
         if (params.count("opIdMem")) {
             long x = params["opIdMem"].as<long>();
-            uassert("bad --opIdMem arg", x > 0);
-            opIdMem = x;
-            assert(opIdMem > 0);
+            uassert( 10036 , "bad --opIdMem arg", x > 0);
+            replSettings.opIdMem = x;
+            assert(replSettings.opIdMem > 0);
         }
         if (params.count("cacheSize")) {
             long x = params["cacheSize"].as<long>();
-            uassert("bad --cacheSize arg", x > 0);
+            uassert( 10037 , "bad --cacheSize arg", x > 0);
             setRecCacheSize(x);
         }
 		if (params.count("port") == 0 ) { 
@@ -868,7 +883,16 @@ int main(int argc, char* argv[], char *envp[] )
         if ( params.count("configsvr" ) && params.count( "diaglog" ) == 0 ){
             _diaglog.level = 1;
         }
-
+        if ( params.count( "profile" ) ){
+            cmdLine.defaultProfile = params["profile"].as<int>();
+        }
+        if ( params.count( "maxConns" ) ){
+            int newSize = params["maxConns"].as<int>();
+            uassert( 12507 , "maxConns has to be at least 5" , newSize >= 5 );
+            uassert( 12508 , "maxConns can't be greater than 10000000" , newSize < 10000000 );
+            connTicketHolder.resize( newSize );
+        }
+        
         Module::configAll( params );
         dataFileSync.go();
 
@@ -943,13 +967,13 @@ namespace mongo {
 
 #undef out
 
-    void exitCleanly() {
+    void exitCleanly( ExitCode code ) {
         goingAway = true;
-        killCurrentOp = 1;
+        killCurrentOp.killAll();
         {
             dblock lk;
             log() << "now exiting" << endl;
-            dbexit( EXIT_KILL );        
+            dbexit( code );        
         }
     }
 
@@ -995,9 +1019,18 @@ namespace mongo {
         int x;
         sigwait( &asyncSignals, &x );
         log() << "got kill or ctrl c signal " << x << " (" << strsignal( x ) << "), will terminate after current cmd ends" << endl;
-        exitCleanly();
+        Client::initThread( "interruptThread" );
+        exitCleanly( EXIT_KILL );
     }
 
+    // this will be called in certain c++ error cases, for example if there are two active
+    // exceptions
+    void myterminate() {
+        rawOut( "terminate() called, printing stack:\n" );
+        printStackTrace();
+        abort();
+    }
+    
     void setupSignals() {
         assert( signal(SIGSEGV, abruptQuit) != SIG_ERR );
         assert( signal(SIGFPE, abruptQuit) != SIG_ERR );
@@ -1013,12 +1046,15 @@ namespace mongo {
         sigaddset( &asyncSignals, SIGTERM );
         assert( pthread_sigmask( SIG_SETMASK, &asyncSignals, 0 ) == 0 );
         boost::thread it( interruptThread );
+        
+        set_terminate( myterminate );
     }
 
 #else
 void ctrlCTerminate() {
-    log() << "got kill or ctrl c signal, will terminate after current cmd ends" << endl;
-    exitCleanly();
+    log() << "got kill or ctrl-c signal, will terminate after current cmd ends" << endl;
+    Client::initThread( "ctrlCTerminate" );
+    exitCleanly( EXIT_KILL );
 }
 BOOL CtrlHandler( DWORD fdwCtrlType )
 {
@@ -1051,18 +1087,10 @@ BOOL CtrlHandler( DWORD fdwCtrlType )
         if( SetConsoleCtrlHandler( (PHANDLER_ROUTINE) CtrlHandler, TRUE ) )
             ;
         else
-            massert("Couldn't register Windows Ctrl-C handler", false);
+            massert( 10297 , "Couldn't register Windows Ctrl-C handler", false);
     }
 #endif
 
-void temptestfoo() {
-    MongoMutex m;
-    m.lock();
-//    m.lock_upgrade();
-    m.lock_shared();
-}
-
-
 } // namespace mongo
 
 #include "recstore.h"
