@@ -20,6 +20,7 @@
 #include "stdafx.h"
 #include "jsobj.h"
 #include "nonce.h"
+#include "../util/atomic_int.h"
 #include "../util/goodies.h"
 #include "../util/base64.h"
 #include "../util/md5.hpp"
@@ -30,6 +31,7 @@
 #include "jsobjmanipulator.h"
 #include "../util/optime.h"
 #include <boost/static_assert.hpp>
+#include <boost/any.hpp>
 #undef assert
 #define assert xassert
 
@@ -37,6 +39,7 @@
 BOOST_STATIC_ASSERT( sizeof(int) == 4 );
 BOOST_STATIC_ASSERT( sizeof(long long) == 8 );
 BOOST_STATIC_ASSERT( sizeof(double) == 8 );
+BOOST_STATIC_ASSERT( sizeof(mongo::Date_t) == 8 );
 BOOST_STATIC_ASSERT( sizeof(mongo::OID) == 12 );
 
 namespace mongo {
@@ -49,12 +52,6 @@ namespace mongo {
     }
 
     IDLabeler GENOID;
-    BSONObjBuilder& operator<<(BSONObjBuilder& b, IDLabeler& id) {
-        OID oid;
-        oid.init();
-        b.appendOID("_id", &oid);
-        return b;
-    }
 
     DateNowLabeler DATENOW;
 
@@ -155,7 +152,7 @@ namespace mongo {
         return s.str();
     }
 
-    string escape( string s ) {
+    string escape( string s , bool escape_slash=false) {
         stringstream ret;
         for ( string::iterator i = s.begin(); i != s.end(); ++i ) {
             switch ( *i ) {
@@ -166,7 +163,7 @@ namespace mongo {
                 ret << "\\\\";
                 break;
             case '/':
-                ret << "\\/";
+                ret << (escape_slash ? "\\/" : "/");
                 break;
             case '\b':
                 ret << "\\b";
@@ -220,7 +217,7 @@ namespace mongo {
                 stringstream ss;
                 ss << "Number " << number() << " cannot be represented in JSON";
                 string message = ss.str();
-                massert( message.c_str(), false );
+                massert( 10311 ,  message.c_str(), false );
             }
             break;
         case Bool:
@@ -305,17 +302,13 @@ namespace mongo {
                 s << " )";
             break;
         case RegEx:
-            if ( format == Strict )
-                s << "{ \"$regex\" : \"";
-            else
-                s << "/";
-            s << escape( regex() );
-            if ( format == Strict )
+            if ( format == Strict ){
+                s << "{ \"$regex\" : \"" << escape( regex() );
                 s << "\", \"$options\" : \"" << regexFlags() << "\" }";
-            else {
-                s << "/";
+            } else {
+                s << "/" << escape( regex() , true ) << "/";
                 // FIXME Worry about alpha order?
-                for ( const char *f = regexFlags(); *f; ++f )
+                for ( const char *f = regexFlags(); *f; ++f ){
                     switch ( *f ) {
                     case 'g':
                     case 'i':
@@ -324,14 +317,24 @@ namespace mongo {
                     default:
                         break;
                     }
+                }
             }
             break;
+
+        case Code:
+            s << ascode();
+            break;
+
+        case Timestamp:
+            s << "{ \"t\" : " << timestampTime() << " , \"i\" : " << timestampInc() << " }";
+            break;
+
         default:
             stringstream ss;
             ss << "Cannot create a properly formatted JSON string with "
             << "element: " << toString() << " of type: " << type();
             string message = ss.str();
-            massert( message.c_str(), false );
+            massert( 10312 ,  message.c_str(), false );
         }
         return s.str();
     }
@@ -368,42 +371,43 @@ namespace mongo {
         case Symbol:
         case Code:
         case String:
-            massert( "Insufficient bytes to calculate element size", maxLen == -1 || remain > 3 );
+            massert( 10313 ,  "Insufficient bytes to calculate element size", maxLen == -1 || remain > 3 );
             x = valuestrsize() + 4;
             break;
         case CodeWScope:
-            massert( "Insufficient bytes to calculate element size", maxLen == -1 || remain > 3 );
+            massert( 10314 ,  "Insufficient bytes to calculate element size", maxLen == -1 || remain > 3 );
             x = objsize();
             break;
 
         case DBRef:
-            massert( "Insufficient bytes to calculate element size", maxLen == -1 || remain > 3 );
+            massert( 10315 ,  "Insufficient bytes to calculate element size", maxLen == -1 || remain > 3 );
             x = valuestrsize() + 4 + 12;
             break;
         case Object:
         case Array:
-            massert( "Insufficient bytes to calculate element size", maxLen == -1 || remain > 3 );
+            massert( 10316 ,  "Insufficient bytes to calculate element size", maxLen == -1 || remain > 3 );
             x = objsize();
             break;
         case BinData:
-            massert( "Insufficient bytes to calculate element size", maxLen == -1 || remain > 3 );
+            massert( 10317 ,  "Insufficient bytes to calculate element size", maxLen == -1 || remain > 3 );
             x = valuestrsize() + 4 + 1/*subtype*/;
             break;
         case RegEx:
         {
             const char *p = value();
             int len1 = ( maxLen == -1 ) ? strlen( p ) : strnlen( p, remain );
-            massert( "Invalid regex string", len1 != -1 );
+            massert( 10318 ,  "Invalid regex string", len1 != -1 );
             p = p + len1 + 1;
             int len2 = ( maxLen == -1 ) ? strlen( p ) : strnlen( p, remain - len1 - 1 );
-            massert( "Invalid regex options string", len2 != -1 );
+            massert( 10319 ,  "Invalid regex options string", len2 != -1 );
             x = len1 + 1 + len2 + 1;
         }
         break;
         default: {
             stringstream ss;
             ss << "BSONElement: bad type " << (int) type();
-            massert(ss.str().c_str(),false);
+            string msg = ss.str();
+            massert( 10320 , msg.c_str(),false);
         }
         }
         totalSize =  x + fieldNameSize() + 1; // BSONType
@@ -424,8 +428,12 @@ namespace mongo {
                     else if ( fn[3] == 'e' && fn[4] == 0 ) return BSONObj::LTE;
                 }
             }
-            else if ( fn[1] == 'n' && fn[2] == 'e' && fn[3] == 0)
-                return BSONObj::NE;
+            else if ( fn[1] == 'n' && fn[2] == 'e' ){
+                if ( fn[3] == 0 )
+                    return BSONObj::NE;
+                if ( fn[3] == 'a' && fn[4] == 'r' && fn[5] == 0 )
+                    return BSONObj::opNEAR;
+            }
             else if ( fn[1] == 'm' && fn[2] == 'o' && fn[3] == 'd' && fn[4] == 0 )
                 return BSONObj::opMOD;
             else if ( fn[1] == 't' && fn[2] == 'y' && fn[3] == 'p' && fn[4] == 'e' && fn[5] == 0 )
@@ -438,12 +446,18 @@ namespace mongo {
                 return BSONObj::opALL;
             else if ( fn[1] == 's' && fn[2] == 'i' && fn[3] == 'z' && fn[4] == 'e' && fn[5] == 0 )
                 return BSONObj::opSIZE;
-            else if ( fn[1] == 'e' && fn[2] == 'x' && fn[3] == 'i' && fn[4] == 's' && fn[5] == 't' && fn[6] == 's' && fn[7] == 0 )
-                return BSONObj::opEXISTS;
+            else if ( fn[1] == 'e' ){
+                if ( fn[2] == 'x' && fn[3] == 'i' && fn[4] == 's' && fn[5] == 't' && fn[6] == 's' && fn[7] == 0 )
+                    return BSONObj::opEXISTS;
+                if ( fn[2] == 'l' && fn[3] == 'e' && fn[4] == 'm' && fn[5] == 'M' && fn[6] == 'a' && fn[7] == 't' && fn[8] == 'c' && fn[9] == 'h' && fn[10] == 0 )
+                    return BSONObj::opELEM_MATCH;
+            }
             else if ( fn[1] == 'r' && fn[2] == 'e' && fn[3] == 'g' && fn[4] == 'e' && fn[5] == 'x' && fn[6] == 0 )
                 return BSONObj::opREGEX;
             else if ( fn[1] == 'o' && fn[2] == 'p' && fn[3] == 't' && fn[4] == 'i' && fn[5] == 'o' && fn[6] == 'n' && fn[7] == 's' && fn[8] == 0 )
                 return BSONObj::opOPTIONS;
+            else if ( fn[1] == 'w' && fn[2] == 'i' && fn[3] == 't' && fn[4] == 'h' && fn[5] == 'i' && fn[6] == 'n' && fn[7] == 0 )
+                return BSONObj::opWITHIN;
         }
         return def;
     }
@@ -527,13 +541,18 @@ namespace mongo {
         case Object:
         case Array:
             return l.embeddedObject().woCompare( r.embeddedObject() );
-        case DBRef:
-        case BinData: {
+        case DBRef: {
             int lsz = l.valuesize();
             int rsz = r.valuesize();
             if ( lsz - rsz != 0 ) return lsz - rsz;
             return memcmp(l.value(), r.value(), lsz);
         }
+        case BinData: {
+            int lsz = l.objsize(); // our bin data size in bytes, not including the subtype byte
+            int rsz = r.objsize();
+            if ( lsz - rsz != 0 ) return lsz - rsz;
+            return memcmp(l.value()+4, r.value()+4, lsz+1);
+        }
         case RegEx:
         {
             int c = strcmp(l.regex(), r.regex());
@@ -560,80 +579,41 @@ namespace mongo {
         return -1;
     }
 
-    /** returns a string that when used as a matcher, would match a super set of regex()
-        returns "" for complex regular expressions
-        used to optimize queries in some simple regex cases that start with '^'
-    */
-    string BSONElement::simpleRegex() const {
-
-        string r = "";
-
-        if ( *regexFlags() )
-            return r;
-
-        const char *i = regex();
-        if ( *i != '^' )
-            return r;
-        ++i;
-
-        // Empty string matches everything, won't limit our search.
-        if ( !*i )
-            return r;
-
-        stringstream ss;
-        for( ; *i; ++i ){
-            char c = *i;
-            if ( c == '*' || c == '?' ){
-                r = ss.str();
-                r = r.substr( 0 , r.size() - 1 );
-                break;
-            }
-            else if ( *i == ' ' || (*i>='0'&&*i<='9') || (*i>='@'&&*i<='Z') || (*i>='a'&&*i<='z') ){
-                ss << *i;
-            }
-            else {
-                r = ss.str();
-                break;
-            }
-        }
-
-        if ( r.size() == 0 && *i == 0 )
-            r = ss.str();
-
-        return r;
-    }
-
     void BSONElement::validate() const {
         switch( type() ) {
-            case DBRef:
-            case Code:
-            case Symbol:
-            case String:
-                massert( "Invalid dbref/code/string/symbol size",
-                        valuestrsize() > 0 &&
-                        valuestrsize() - 1 == strnlen( valuestr(), valuestrsize() ) );
-                break;
-            case CodeWScope: {
-                int totalSize = *( int * )( value() );
-                massert( "Invalid CodeWScope size", totalSize >= 8 );
-                int strSizeWNull = *( int * )( value() + 4 );
-                massert( "Invalid CodeWScope string size", totalSize >= strSizeWNull + 4 + 4 );
-                massert( "Invalid CodeWScope string size",
-                        strSizeWNull > 0 &&
-                        strSizeWNull - 1 == strnlen( codeWScopeCode(), strSizeWNull ) );
-                massert( "Invalid CodeWScope size", totalSize >= strSizeWNull + 4 + 4 + 4 );
-                int objSize = *( int * )( value() + 4 + 4 + strSizeWNull );
-                massert( "Invalid CodeWScope object size", totalSize == 4 + 4 + strSizeWNull + objSize );
-                // Subobject validation handled elsewhere.
-            }
-            case Object:
-                // We expect Object size validation to be handled elsewhere.
-            default:
-                break;
+        case DBRef:
+        case Code:
+        case Symbol:
+        case String: {
+            int x = valuestrsize();
+            if ( x > 0 && valuestr()[x-1] == 0 )
+                return;
+            StringBuilder buf;
+            buf <<  "Invalid dbref/code/string/symbol size: " << x << " strnlen:" << strnlen( valuestr() , x );
+            massert( 10321 , buf.str() , 0 );
+            break;
+        }
+        case CodeWScope: {
+            int totalSize = *( int * )( value() );
+            massert( 10322 ,  "Invalid CodeWScope size", totalSize >= 8 );
+            int strSizeWNull = *( int * )( value() + 4 );
+            massert( 10323 ,  "Invalid CodeWScope string size", totalSize >= strSizeWNull + 4 + 4 );
+            massert( 10324 ,  "Invalid CodeWScope string size",
+                     strSizeWNull > 0 &&
+                     strSizeWNull - 1 == strnlen( codeWScopeCode(), strSizeWNull ) );
+            massert( 10325 ,  "Invalid CodeWScope size", totalSize >= strSizeWNull + 4 + 4 + 4 );
+            int objSize = *( int * )( value() + 4 + 4 + strSizeWNull );
+            massert( 10326 ,  "Invalid CodeWScope object size", totalSize == 4 + 4 + strSizeWNull + objSize );
+            // Subobject validation handled elsewhere.
+        }
+        case Object:
+            // We expect Object size validation to be handled elsewhere.
+        default:
+            break;
         }
     }
 
-    /* JSMatcher --------------------------------------*/
+    /* Matcher --------------------------------------*/
 
 // If the element is something like:
 //   a : { $gt : 3 }
@@ -682,7 +662,7 @@ namespace mongo {
             const string& c = l.substr( lstart , lend - lstart );
             const string& d = r.substr( rstart , rend - rstart );
 
-            int x = c.compare( d );
+            int x = lexNumCmp( c.c_str(), d.c_str() );
 
             if ( x < 0 )
                 return LEFT_BEFORE;
@@ -706,17 +686,17 @@ namespace mongo {
         BSONObjIterator i(*this);
         bool first = true;
         while ( 1 ) {
-            massert( "Object does not end with EOO", i.moreWithEOO() );
+            massert( 10327 ,  "Object does not end with EOO", i.moreWithEOO() );
             BSONElement e = i.next( true );
-            massert( "Invalid element size", e.size() > 0 );
-            massert( "Element too large", e.size() < ( 1 << 30 ) );
+            massert( 10328 ,  "Invalid element size", e.size() > 0 );
+            massert( 10329 ,  "Element too large", e.size() < ( 1 << 30 ) );
             int offset = e.rawdata() - this->objdata();
-            massert( "Element extends past end of object",
+            massert( 10330 ,  "Element extends past end of object",
                     e.size() + offset <= this->objsize() );
             e.validate();
             bool end = ( e.size() + offset == this->objsize() );
             if ( e.eoo() ) {
-                massert( "EOO Before end of object", end );
+                massert( 10331 ,  "EOO Before end of object", end );
                 break;
             }
             if ( first )
@@ -795,9 +775,18 @@ namespace mongo {
             if ( r.eoo() )
                 return 1;
 
-            int x = l.woCompare( r, considerFieldName );
-            if ( ordered && o.number() < 0 )
-                x = -x;
+            int x;
+/*
+            if( ordered && o.type() == String && strcmp(o.valuestr(), "ascii-proto") == 0 && 
+                l.type() == String && r.type() == String ) { 
+                // note: no negative support yet, as this is just sort of a POC
+                x = _stricmp(l.valuestr(), r.valuestr());
+            }
+            else*/ {
+                x = l.woCompare( r, considerFieldName );
+                if ( ordered && o.number() < 0 )
+                    x = -x;
+            }
             if ( x != 0 )
                 return x;
         }
@@ -813,7 +802,7 @@ namespace mongo {
         if ( other.isEmpty() )
             return 1;
 
-        uassert( "woSortOrder needs a non-empty sortKey" , ! sortKey.isEmpty() );
+        uassert( 10060 ,  "woSortOrder needs a non-empty sortKey" , ! sortKey.isEmpty() );
 
         BSONObjIterator i(sortKey);
         while ( 1 ){
@@ -838,18 +827,6 @@ namespace mongo {
     }
 
 
-    BSONElement BSONObj::getField(const char *name) const {
-        BSONObjIterator i(*this);
-        while ( i.moreWithEOO() ) {
-            BSONElement e = i.next();
-            if ( e.eoo() )
-                break;
-            if ( strcmp(e.fieldName(), name) == 0 )
-                return e;
-        }
-        return nullElement;
-    }
-
     /* return has eoo() true if no match
        supports "." notation to reach into embedded objects
     */
@@ -867,49 +844,62 @@ namespace mongo {
         return e;
     }
 
-    /* jul09 : 'deep' and this function will be going away in the future - kept only for backward compatibility of datafiles for now. */
-    void trueDat( bool *deep ) {
-        if( deep )
-            *deep = true;
-    }
+    void BSONObj::getFieldsDotted(const char *name, BSONElementSet &ret ) const {
+        BSONObjIterator i(*this);
+        while ( i.more() ){
+            BSONElement e = i.next();
+            FieldCompareResult cmp = compareDottedFieldNames( name , e.fieldName() );
+            switch ( cmp ){
 
-    void BSONObj::getFieldsDotted(const char *name, BSONElementSet &ret, bool *deep ) const {
-        BSONElement e = getField( name );
-        if ( e.eoo() ) {
-            const char *p = strchr(name, '.');
-            if ( p ) {
-                string left(name, p-name);
-                BSONElement e = getField( left );
-                if ( e.type() == Array ) {
-                    trueDat( deep );
-                    BSONObjIterator i( e.embeddedObject() );
-                    while( i.moreWithEOO() ) {
-                        BSONElement f = i.next();
-                        if ( f.eoo() )
-                            break;
+            case LEFT_BEFORE: 
+            case RIGHT_BEFORE:
+                break;
+
+            case RIGHT_SUBFIELD: 
+                assert(0); 
+                break;
+
+            case LEFT_SUBFIELD: {
+                const char * next = name + strlen( e.fieldName() ) + 1;
+                bool allDigits = false;
+                if ( isdigit( *next ) ){
+                    const char * temp = next + 1;
+                    while ( isdigit( *temp ) )
+                        temp++;
+                    allDigits = *temp == '.';
+                }
+
+                if ( e.type() == Object || allDigits ){
+                    e.embeddedObject().getFieldsDotted( next , ret );
+                }
+                else if ( e.type() == Array ){
+                    BSONObjIterator j( e.embeddedObject() );
+                    while ( j.more() ){
+                        BSONElement f = j.next();
                         if ( f.type() == Object )
-                            f.embeddedObject().getFieldsDotted(p+1, ret);
+                            f.embeddedObject().getFieldsDotted( next , ret );
                     }
-                } else if ( e.type() == Object ) {
-                    e.embeddedObject().getFieldsDotted(p+1, ret);
                 }
+                else {
+                    // intentially left blank, this means no match
+                }
+                return;
             }
-        } else {
-            if ( e.type() == Array ) {
-                trueDat( deep );
-                BSONObjIterator i( e.embeddedObject() );
-                while( i.moreWithEOO() ) {
-                    BSONElement f = i.next();
-                    if ( f.eoo() )
-                        break;
-                    ret.insert( f );
+
+            case SAME: {
+                if ( e.type() == Array ){
+                    BSONObjIterator j( e.embeddedObject() );
+                    while ( j.more() )
+                        ret.insert( j.next() );
                 }
-            } else {
-                ret.insert( e );
+                else {
+                    ret.insert( e );
+                }
+                return;
+            }
+
             }
         }
-        if ( ret.empty() && deep )
-            *deep = false;
     }
 
     BSONElement BSONObj::getFieldDottedOrArray(const char *&name) const {
@@ -935,29 +925,25 @@ namespace mongo {
 
     /* makes a new BSONObj with the fields specified in pattern.
        fields returned in the order they appear in pattern.
-       if any field missing from the original object, that field
-       in the key will be null.
+       if any field missing or undefined in the original object, that field
+       in the output will be null.
 
        n^2 implementation bad if pattern and object have lots
        of fields - normally pattern doesn't so should be fine.
     */
-    BSONObj BSONObj::extractFieldsDotted(BSONObj pattern, BSONObjBuilder& b, const char *&nameWithinArray) const {
-        nameWithinArray = "";
+    BSONObj BSONObj::extractFieldsDotted(BSONObj pattern) const {
+        BSONObjBuilder b;
         BSONObjIterator i(pattern);
-        while ( i.moreWithEOO() ) {
+        while (i.more()) {
             BSONElement e = i.next();
-            if ( e.eoo() )
-                break;
             const char *name = e.fieldName();
-            BSONElement x = getFieldDottedOrArray( name );
-            if ( x.eoo() ) {
-                b.appendNull( "" );
-                continue;
-            } else if ( x.type() == Array ) {
-                // NOTE: Currently set based on last array discovered.
-                nameWithinArray = name;
+
+            BSONElement x = getFieldDotted( name );
+            if ( x.eoo() || x.type() == Undefined ) {
+                b.appendNull(name);
+            } else {
+                b.appendAs(x, name);
             }
-            b.appendAs(x, "");
         }
         return b.done();
     }
@@ -1166,6 +1152,40 @@ namespace mongo {
         return b.obj();
     }
 
+    bool BSONObj::okForStorage() const {
+        BSONObjIterator i( *this );
+        while ( i.more() ){
+            BSONElement e = i.next();
+            const char * name = e.fieldName();
+            
+            if ( strchr( name , '.' ) ||
+                 strchr( name , '$' ) ){
+                return 
+                    strcmp( name , "$ref" ) == 0 ||
+                    strcmp( name , "$id" ) == 0
+                    ;
+            }
+            
+            if ( e.mayEncapsulate() ){
+                switch ( e.type() ){
+                case Object:
+                case Array:
+                    if ( ! e.embeddedObject().okForStorage() )
+                        return false;
+                    break;
+                case CodeWScope:
+                    if ( ! e.codeWScopeObject().okForStorage() )
+                        return false;
+                    break;
+                default:
+                    uassert( 12579, "unhandled cases in BSONObj okForStorage" , 0 );
+                }
+                
+            }
+        }
+        return true;
+    }
+
     string BSONObj::hexDump() const {
         stringstream ss;
         const char *d = objdata();
@@ -1275,24 +1295,6 @@ namespace mongo {
             assert( !o.woEqual( p ) );
             assert( o.woCompare( p ) < 0 );
 
-            {
-                BSONObjBuilder b;
-                b.appendRegex("r", "^foo");
-                BSONObj o = b.done();
-                assert( o.firstElement().simpleRegex() == "foo" );
-            }
-            {
-                BSONObjBuilder b;
-                b.appendRegex("r", "^f?oo");
-                BSONObj o = b.done();
-                assert( o.firstElement().simpleRegex() == "" );
-            }
-            {
-                BSONObjBuilder b;
-                b.appendRegex("r", "^fz?oo");
-                BSONObj o = b.done();
-                assert( o.firstElement().simpleRegex() == "f" );
-            }
         }
         void testoid() {
             OID id;
@@ -1430,7 +1432,7 @@ namespace mongo {
     }
     
     void OID::init() {
-        static WrappingInt inc = (unsigned) security.getNonce();
+        static AtomicUInt inc = (unsigned) security.getNonce();
         unsigned t = (unsigned) time(0);
         char *T = (char *) &t;
         data[0] = T[3];
@@ -1440,7 +1442,7 @@ namespace mongo {
 
         (unsigned&) data[4] = _machine;
 
-        int new_inc = inc.atomicIncrement();
+        int new_inc = inc++;
         T = (char *) &new_inc;
         char * raw = (char*)&b;
         raw[0] = T[3];
@@ -1484,7 +1486,7 @@ namespace mongo {
     Labeler::Label SIZE( "$size" );
 
     void BSONElementManipulator::initTimestamp() {
-        massert( "Expected CurrentTime type", element_.type() == Timestamp );
+        massert( 10332 ,  "Expected CurrentTime type", _element.type() == Timestamp );
         unsigned long long &timestamp = *( reinterpret_cast< unsigned long long* >( value() ) );
         if ( timestamp == 0 )
             timestamp = OpTime::now().asDate();
@@ -1532,7 +1534,7 @@ namespace mongo {
 
         };
         log() << "type not support for appendMinElementForType: " << t << endl;
-        uassert( "type not supported for appendMinElementForType" , false );
+        uassert( 10061 ,  "type not supported for appendMinElementForType" , false );
     }
 
     void BSONObjBuilder::appendMaxForType( const string& field , int t ){
@@ -1630,4 +1632,38 @@ namespace mongo {
 
     }
 
+    void BSONObjBuilder::appendKeys( const BSONObj& keyPattern , const BSONObj& values ){
+        BSONObjIterator i(keyPattern);
+        BSONObjIterator j(values);
+        
+        while ( i.more() && j.more() ){
+            appendAs( j.next() , i.next().fieldName() );
+        }
+        
+        assert( ! i.more() );
+        assert( ! j.more() );
+    }
+
+    int BSONElementFieldSorter( const void * a , const void * b ){
+        const char * x = *((const char**)a);
+        const char * y = *((const char**)b);
+        x++; y++;
+        return lexNumCmp( x , y );
+    }
+    
+    BSONObjIteratorSorted::BSONObjIteratorSorted( const BSONObj& o ){
+        _nfields = o.nFields();
+        _fields = new const char*[_nfields];
+        int x = 0;
+        BSONObjIterator i( o );
+        while ( i.more() ){
+            _fields[x++] = i.next().rawdata();
+            assert( _fields[x-1] );
+        }
+        assert( x == _nfields );
+        qsort( _fields , _nfields , sizeof(char*) , BSONElementFieldSorter );
+        _cur = 0;
+    }
+
+
 } // namespace mongo
