@@ -20,6 +20,7 @@
 #include "json.h"
 #include "../util/builder.h"
 #include "../util/base64.h"
+#include "../util/hex.h"
 
 using namespace boost::spirit;
 
@@ -84,7 +85,7 @@ namespace mongo {
         BinDataType binDataType;
         string regex;
         string regexOptions;
-        unsigned long long date;
+        Date_t date;
     };
 
     struct objectStart {
@@ -156,6 +157,9 @@ namespace mongo {
             case 't':
                 o = '\t';
                 break;
+            case 'v':
+                o = '\v';
+                break;
             default:
                 assert( false );
             }
@@ -164,27 +168,11 @@ namespace mongo {
         ObjectBuilder &b;
     };
 
-    namespace hex {
-        int val( char c ) {
-            if ( '0' <= c && c <= '9' )
-                return c - '0';
-            if ( 'a' <= c && c <= 'f' )
-                return c - 'a' + 10;
-            if ( 'A' <= c && c <= 'F' )
-                return c - 'A' + 10;
-            assert( false );
-            return 0xff;
-        }
-        char val( const char *c ) {
-            return ( val( c[ 0 ] ) << 4 ) | val( c[ 1 ] );
-        }
-    } // namespace hex
-
     struct chU {
         chU( ObjectBuilder &_b ) : b( _b ) {}
         void operator() ( const char *start, const char *end ) const {
-            unsigned char first = hex::val( start );
-            unsigned char second = hex::val( start + 2 );
+            unsigned char first = fromHex( start );
+            unsigned char second = fromHex( start + 2 );
             if ( first == 0 && second < 0x80 )
                 b.ss << second;
             else if ( first < 0x08 ) {
@@ -211,7 +199,7 @@ namespace mongo {
         fieldNameEnd( ObjectBuilder &_b ) : b( _b ) {}
         void operator() ( const char *start, const char *end ) const {
             string name = b.popString();
-            massert( "Invalid use of reserved field name",
+            massert( 10338 ,  "Invalid use of reserved field name",
                      name != "$oid" &&
                      name != "$binary" &&
                      name != "$type" &&
@@ -312,7 +300,7 @@ namespace mongo {
         OID oid;
         char *oidP = (char *)( &oid );
         for ( int i = 0; i < 12; ++i )
-            oidP[ i ] = hex::val( s + ( i * 2 ) );
+            oidP[ i ] = fromHex( s + ( i * 2 ) );
         return oid;
     }
 
@@ -343,7 +331,7 @@ namespace mongo {
     struct binDataBinary {
         binDataBinary( ObjectBuilder &_b ) : b( _b ) {}
         void operator() ( const char *start, const char *end ) const {
-            massert( "Badly formatted bindata", ( end - start ) % 4 == 0 );
+            massert( 10339 ,  "Badly formatted bindata", ( end - start ) % 4 == 0 );
             string encoded( start, end );
             b.binData = base64::decode( encoded );
         }
@@ -353,7 +341,7 @@ namespace mongo {
     struct binDataType {
         binDataType( ObjectBuilder &_b ) : b( _b ) {}
         void operator() ( const char *start, const char *end ) const {
-            b.binDataType = BinDataType( hex::val( start ) );
+            b.binDataType = BinDataType( fromHex( start ) );
         }
         ObjectBuilder &b;
     };
@@ -369,7 +357,7 @@ namespace mongo {
 
     struct dateValue {
         dateValue( ObjectBuilder &_b ) : b( _b ) {}
-        void operator() ( unsigned long long v ) const {
+        void operator() ( Date_t v ) const {
             b.date = v;
         }
         ObjectBuilder &b;
@@ -456,28 +444,32 @@ public:
                 // NOTE We use range_p rather than cntrl_p, because the latter is locale dependent.
                 str = lexeme_d[ ch_p( '"' )[ chClear( self.b ) ] >>
                                 *( ( ch_p( '\\' ) >>
-                                     ( ch_p( '"' )[ chE( self.b ) ] |
-                                       ch_p( '\\' )[ chE( self.b ) ] |
-                                       ch_p( '/' )[ chE( self.b ) ] |
+                                     (
                                        ch_p( 'b' )[ chE( self.b ) ] |
                                        ch_p( 'f' )[ chE( self.b ) ] |
                                        ch_p( 'n' )[ chE( self.b ) ] |
                                        ch_p( 'r' )[ chE( self.b ) ] |
                                        ch_p( 't' )[ chE( self.b ) ] |
-                                       ( ch_p( 'u' ) >> ( repeat_p( 4 )[ xdigit_p ][ chU( self.b ) ] ) ) ) ) |
+                                       ch_p( 'v' )[ chE( self.b ) ] |
+                                       ( ch_p( 'u' ) >> ( repeat_p( 4 )[ xdigit_p ][ chU( self.b ) ] ) ) |
+                                       ( ~ch_p('x') & (~range_p('0','9'))[ ch( self.b ) ] ) // hex and octal aren't supported
+                                     )
+                                   ) |
                                    ( ~range_p( 0x00, 0x1f ) & ~ch_p( '"' ) & ( ~ch_p( '\\' ) )[ ch( self.b ) ] ) ) >> '"' ];
 
                 singleQuoteStr = lexeme_d[ ch_p( '\'' )[ chClear( self.b ) ] >>
                                 *( ( ch_p( '\\' ) >>
-                                     ( ch_p( '\'' )[ chE( self.b ) ] |
-                                       ch_p( '\\' )[ chE( self.b ) ] |
-                                       ch_p( '/' )[ chE( self.b ) ] |
+                                     (
                                        ch_p( 'b' )[ chE( self.b ) ] |
                                        ch_p( 'f' )[ chE( self.b ) ] |
                                        ch_p( 'n' )[ chE( self.b ) ] |
                                        ch_p( 'r' )[ chE( self.b ) ] |
                                        ch_p( 't' )[ chE( self.b ) ] |
-                                       ( ch_p( 'u' ) >> ( repeat_p( 4 )[ xdigit_p ][ chU( self.b ) ] ) ) ) ) |
+                                       ch_p( 'v' )[ chE( self.b ) ] |
+                                       ( ch_p( 'u' ) >> ( repeat_p( 4 )[ xdigit_p ][ chU( self.b ) ] ) ) |
+                                       ( ~ch_p('x') & (~range_p('0','9'))[ ch( self.b ) ] ) // hex and octal aren't supported
+                                     )
+                                   ) |
                                    ( ~range_p( 0x00, 0x1f ) & ~ch_p( '\'' ) & ( ~ch_p( '\\' ) )[ ch( self.b ) ] ) ) >> '\'' ];
 
                 // real_p accepts numbers with nonsignificant zero prefixes, which
@@ -506,9 +498,10 @@ public:
                           lexeme_d[ '"' >> ( *( range_p( 'A', 'Z' ) | range_p( 'a', 'z' ) | range_p( '0', '9' ) | ch_p( '+' ) | ch_p( '/' ) ) >> *ch_p( '=' ) )[ binDataBinary( self.b ) ] >> '"' ] >> ',' >> "\"$type\"" >> ':' >>
                           lexeme_d[ '"' >> ( repeat_p( 2 )[ xdigit_p ] )[ binDataType( self.b ) ] >> '"' ] >> '}';
 
+                // TODO: this will need to use a signed parser at some point
                 date = dateS | dateT;
-                dateS = ch_p( '{' ) >> "\"$date\"" >> ':' >> uint_parser< unsigned long long >()[ dateValue( self.b ) ] >> '}';
-                dateT = str_p( "Date" ) >> '(' >> uint_parser< unsigned long long >()[ dateValue( self.b ) ] >> ')';
+                dateS = ch_p( '{' ) >> "\"$date\"" >> ':' >> uint_parser< Date_t >()[ dateValue( self.b ) ] >> '}';
+                dateT = !str_p("new") >> str_p( "Date" ) >> '(' >> uint_parser< Date_t >()[ dateValue( self.b ) ] >> ')';
 
                 regex = regexS | regexT;
                 regexS = ch_p( '{' ) >> "\"$regex\"" >> ':' >> str[ regexValue( self.b ) ] >> ',' >> "\"$options\"" >> ':' >> lexeme_d[ '"' >> ( *( alpha_p ) )[ regexOptions( self.b ) ] >> '"' ] >> '}';
@@ -549,7 +542,7 @@ public:
                 len = 10;
             stringstream ss;
             ss << "Failure parsing JSON string near: " << string( result.stop, len );
-            massert( ss.str(), false );
+            massert( 10340 ,  ss.str(), false );
         }
         return b.pop();
     }
