@@ -47,11 +47,43 @@ namespace mongo {
     }
     
     boost::filesystem::path NamespaceIndex::path() const {
-        return boost::filesystem::path( dir_ ) / ( database_ + ".ns" );
+        boost::filesystem::path ret( dir_ );
+        if ( directoryperdb )
+            ret /= database_;
+        ret /= ( database_ + ".ns" );
+        return ret;
     }
 
+    void NamespaceIndex::maybeMkdir() const {
+        if ( !directoryperdb )
+            return;
+        boost::filesystem::path dir( dir_ );
+        dir /= database_;
+        if ( !boost::filesystem::exists( dir ) )
+            BOOST_CHECK_EXCEPTION( boost::filesystem::create_directory( dir ) );
+    }
+    
 	int lenForNewNsFiles = 16 * 1024 * 1024;
     
+    void NamespaceDetails::onLoad(const Namespace& k) { 
+        if( k.isExtra() ) { 
+            /* overflow storage for indexes - so don't treat as a NamespaceDetails object. */
+            return;
+        }
+
+        assertInWriteLock();
+        if( backgroundIndexBuildInProgress ) { 
+            log() << "backgroundIndexBuildInProgress was " << backgroundIndexBuildInProgress << " for " << k << ", indicating an abnormal db shutdown" << endl;
+            backgroundIndexBuildInProgress = 0;
+        }
+    }
+
+    static void callback(const Namespace& k, NamespaceDetails& v) { 
+        v.onLoad(k);
+    }
+
+    bool checkNsFilesOnLoad = true;
+
     void NamespaceIndex::init() {
         if ( ht )
             return;
@@ -59,11 +91,12 @@ namespace mongo {
            we need to be sure to clear any cached info for the database in
            local.*.
         */
+		/*
         if ( "local" != database_ ) {
             DBInfo i(database_.c_str());
             i.dbDropped();
         }
-
+		*/
 		int len = -1;
         boost::filesystem::path nsPath = path();
         string pathString = nsPath.string();
@@ -74,13 +107,14 @@ namespace mongo {
                 len = f.length();
                 if ( len % (1024*1024) != 0 ){
                     log() << "bad .ns file: " << pathString << endl;
-                    uassert( "bad .ns file length, cannot open database", len % (1024*1024) == 0 );
+                    uassert( 10079 ,  "bad .ns file length, cannot open database", len % (1024*1024) == 0 );
                 }
             }
 		}
 		else {
 			// use lenForNewNsFiles, we are making a new database
-			massert( "bad lenForNewNsFiles", lenForNewNsFiles >= 1024*1024 );
+			massert( 10343 ,  "bad lenForNewNsFiles", lenForNewNsFiles >= 1024*1024 );
+            maybeMkdir();
 			long l = lenForNewNsFiles;
 			p = f.map(pathString.c_str(), l);
             if( p ) { 
@@ -94,6 +128,8 @@ namespace mongo {
             dbexit( EXIT_FS );
         }
         ht = new HashTable<Namespace,NamespaceDetails>(p, len, "namespace index");
+        if( checkNsFilesOnLoad )
+            ht->iterAll(callback);
     }
 
     void NamespaceDetails::addDeletedRec(DeletedRecord *d, DiskLoc dloc) {
@@ -445,9 +481,14 @@ namespace mongo {
         // signal done allocating new extents.
         if ( !deletedList[ 1 ].isValid() )
             deletedList[ 1 ] = DiskLoc();
-
+        
         assert( len < 400000000 );
         int passes = 0;
+        int maxPasses = ( len / 30 ) + 2; // 30 is about the smallest entry that could go in the oplog
+        if ( maxPasses < 5000 ){
+            // this is for bacwards safety since 5000 was the old value
+            maxPasses = 5000;
+        }
         DiskLoc loc;
 
         // delete records until we have room and the max # objects limit achieved.
@@ -492,14 +533,14 @@ namespace mongo {
                 continue;
             }
 
-            massert( "Capped collection full and delete not allowed", cappedMayDelete() );
+            massert( 10344 ,  "Capped collection full and delete not allowed", cappedMayDelete() );
             DiskLoc fr = theCapExtent()->firstRecord;
             theDataFileMgr.deleteRecord(ns, fr.rec(), fr, true);
             compact();
-            if( ++passes >= 5000 ) {
-                log() << "passes ns:" << ns << " len:" << len << '\n';
+            if( ++passes > maxPasses ) {
+                log() << "passes ns:" << ns << " len:" << len << " maxPasses: " << maxPasses << '\n';
                 log() << "passes max:" << max << " nrecords:" << nrecords << " datasize: " << datasize << endl;
-                massert( "passes >= 5000 in capped collection alloc", false );
+                massert( 10345 ,  "passes >= maxPasses in capped collection alloc", false );
             }
         }
 
@@ -511,7 +552,7 @@ namespace mongo {
     }
 
     /* you MUST call when adding an index.  see pdfile.cpp */
-    IndexDetails& NamespaceDetails::addIndex(const char *thisns) {
+    IndexDetails& NamespaceDetails::addIndex(const char *thisns, bool resetTransient) {
         assert( nsdetails(thisns) == this );
 
         if( nIndexes == NIndexesBase && extraOffset == 0 ) { 
@@ -520,7 +561,8 @@ namespace mongo {
 
         IndexDetails& id = idx(nIndexes);
         nIndexes++;
-        NamespaceDetailsTransient::get(thisns).addedIndex();
+        if ( resetTransient )
+            NamespaceDetailsTransient::get_w(thisns).addedIndex();
         return id;
     }
 
@@ -537,95 +579,113 @@ namespace mongo {
        (aug08 - this method not currently used)
     */
     int NamespaceDetails::fieldIsIndexed(const char *fieldName) {
-        massert("not implemented", false);
+        massert( 10346 , "not implemented", false);
         /*
         for ( int i = 0; i < nIndexes; i++ ) {
             IndexDetails& idx = indexes[i];
             BSONObj idxKey = idx.info.obj().getObjectField("key"); // e.g., { ts : -1 }
-            if ( !idxKey.findElement(fieldName).eoo() )
+            if ( !idxKey.getField(fieldName).eoo() )
                 return i;
         }*/
         return -1;
     }
     
-    long long NamespaceDetails::storageSize(){
+    long long NamespaceDetails::storageSize( int * numExtents ){
         Extent * e = firstExtent.ext();
         assert( e );
         
         long long total = 0;
+        int n = 0;
         while ( e ){
-                total += e->length;
-                e = e->getNextExtent();
+            total += e->length;
+            e = e->getNextExtent();
+            n++;
         }
+        
+        if ( numExtents )
+            *numExtents = n;
+        
         return total;
     }
     
     /* ------------------------------------------------------------------------- */
 
-    map< string, shared_ptr< NamespaceDetailsTransient > > NamespaceDetailsTransient::map_;
+    mongo::mutex NamespaceDetailsTransient::_qcMutex;
+    mongo::mutex NamespaceDetailsTransient::_isMutex;
+    map< string, shared_ptr< NamespaceDetailsTransient > > NamespaceDetailsTransient::_map;
     typedef map< string, shared_ptr< NamespaceDetailsTransient > >::iterator ouriter;
 
     void NamespaceDetailsTransient::reset() {
+        DEV assertInWriteLock();
         clearQueryCache();
-        haveIndexKeys = false;
+        _keysComputed = false;
+        _indexSpecs.clear();
     }
     
-    NamespaceDetailsTransient& NamespaceDetailsTransient::get(const char *ns) {
+/*    NamespaceDetailsTransient& NamespaceDetailsTransient::get(const char *ns) {
         shared_ptr< NamespaceDetailsTransient > &t = map_[ ns ];
         if ( t.get() == 0 )
             t.reset( new NamespaceDetailsTransient(ns) );
         return *t;
     }
-
-    void NamespaceDetailsTransient::drop(const char *prefix) {
+*/
+    void NamespaceDetailsTransient::clearForPrefix(const char *prefix) {
+        assertInWriteLock();
         vector< string > found;
-        for( ouriter i = map_.begin(); i != map_.end(); ++i )
+        for( ouriter i = _map.begin(); i != _map.end(); ++i )
             if ( strncmp( i->first.c_str(), prefix, strlen( prefix ) ) == 0 )
                 found.push_back( i->first );
         for( vector< string >::iterator i = found.begin(); i != found.end(); ++i ) {
-            map_[ *i ].reset();
+            _map[ *i ].reset();
         }
     }
     
     void NamespaceDetailsTransient::computeIndexKeys() {
-        allIndexKeys.clear();
-        NamespaceDetails *d = nsdetails(ns.c_str());
+        _keysComputed = true;
+        _indexKeys.clear();
+        NamespaceDetails *d = nsdetails(_ns.c_str());
+        if ( ! d )
+            return;
         NamespaceDetails::IndexIterator i = d->ii();
         while( i.more() )
-            i.next().keyPattern().getFieldNames(allIndexKeys);
+            i.next().keyPattern().getFieldNames(_indexKeys);
     }
-    
-    void NamespaceDetailsTransient::startLog( int logSizeMb ) {
-        logNS_ = "local.temp.oplog." + ns;
-        logValid_ = true;
+
+    void NamespaceDetailsTransient::cllStart( int logSizeMb ) {
+        assertInWriteLock();
+        _cll_ns = "local.temp.oplog." + _ns;
+        _cll_enabled = true;
         stringstream spec;
         // 128MB
         spec << "{size:" << logSizeMb * 1024 * 1024 << ",capped:true,autoIndexId:false}";
-        setClient( logNS_.c_str() );
+        Client::Context ct( _cll_ns );
         string err;
-        massert( "Could not create log ns", userCreateNS( logNS_.c_str(), fromjson( spec.str() ), err, false ) );
-        NamespaceDetails *d = nsdetails( logNS_.c_str() );
+        massert( 10347 ,  "Could not create log ns", userCreateNS( _cll_ns.c_str(), fromjson( spec.str() ), err, false ) );
+        NamespaceDetails *d = nsdetails( _cll_ns.c_str() );
         d->cappedDisallowDelete();
     }
 
-    void NamespaceDetailsTransient::invalidateLog() {
-        dropLog();
-        logValid_ = false;
+    void NamespaceDetailsTransient::cllInvalidate() {
+        assertInWriteLock();
+        cllDrop();
+        _cll_enabled = false;
     }
     
-    bool NamespaceDetailsTransient::validateCompleteLog() {
-        dropLog();
-        bool ret = logValid_;
-        logValid_ = false;
-        logNS_ = "";
+    bool NamespaceDetailsTransient::cllValidateComplete() {
+        assertInWriteLock();
+        cllDrop();
+        bool ret = _cll_enabled;
+        _cll_enabled = false;
+        _cll_ns = "";
         return ret;
     }
     
-    void NamespaceDetailsTransient::dropLog() {
-        if ( !logValid_ )
+    void NamespaceDetailsTransient::cllDrop() {
+        assertInWriteLock();
+        if ( !_cll_enabled )
             return;
-        setClient( logNS_.c_str() );
-        dropNS( logNS_ );
+        Client::Context ctx( _cll_ns );
+        dropNS( _cll_ns );
     }
 
     /* ------------------------------------------------------------------------- */
@@ -648,7 +708,7 @@ namespace mongo {
                 b.append("options", *options);
             BSONObj j = b.done();
             char database[256];
-            nsToClient(ns, database);
+            nsToDatabase(ns, database);
             string s = database;
             s += ".system.namespaces";
             theDataFileMgr.insert(s.c_str(), j.objdata(), j.objsize(), true);
@@ -664,7 +724,7 @@ namespace mongo {
 		// index details across commands are in cursors and nsd
 		// transient (including query cache) so clear these.
 		ClientCursor::invalidate( from );
-		NamespaceDetailsTransient::drop( from );
+		NamespaceDetailsTransient::clearForPrefix( from );
 
 		NamespaceDetails *details = ni->details( from );
 		ni->add_ns( to, *details );
@@ -681,8 +741,8 @@ namespace mongo {
 		details = todetails;
 		
 		BSONObj oldSpec;
-		char database[MaxClientLen];
-		nsToClient(from, database);
+		char database[MaxDatabaseLen];
+		nsToDatabase(from, database);
 		string s = database;
 		s += ".system.namespaces";
 		assert( Helpers::findOne( s.c_str(), BSON( "name" << from ), oldSpec ) );
