@@ -21,7 +21,7 @@
 #include "../stdafx.h"
 #include "jsobj.h"
 #include "queryutil.h"
-#include "storage.h"
+#include "diskloc.h"
 #include "../util/hashtab.h"
 #include "../util/mmap.h"
 
@@ -33,10 +33,10 @@ namespace mongo {
 
 	/* in the mongo source code, "client" means "database". */
 
-    const int MaxClientLen = 256; // max str len for the db name, including null char
+    const int MaxDatabaseLen = 256; // max str len for the db name, including null char
 
 	// "database.a.b.c" -> "database"
-    inline void nsToClient(const char *ns, char *database) {
+    inline void nsToDatabase(const char *ns, char *database) {
         const char *p = ns;
         char *q = database;
         while ( *p != '.' ) {
@@ -45,14 +45,14 @@ namespace mongo {
             *q++ = *p++;
         }
         *q = 0;
-        if (q-database>=MaxClientLen) {
-            log() << "nsToClient: ns too long. terminating, buf overrun condition" << endl;
+        if (q-database>=MaxDatabaseLen) {
+            log() << "nsToDatabase: ns too long. terminating, buf overrun condition" << endl;
             dbexit( EXIT_POSSIBLE_CORRUPTION );
         }
     }
-    inline string nsToClient(const char *ns) {
-        char buf[MaxClientLen];
-        nsToClient(ns, buf);
+    inline string nsToDatabase(const char *ns) {
+        char buf[MaxDatabaseLen];
+        nsToDatabase(ns, buf);
         return buf;
     }
 
@@ -75,6 +75,10 @@ namespace mongo {
         NamespaceString( const char * ns ) { init(ns); }
         NamespaceString( const string& ns ) { init(ns.c_str()); }
 
+        string ns() const { 
+            return db + '.' + coll;
+        }
+
         bool isSystem() { 
             return strncmp(coll.c_str(), "system.", 7) == 0;
         }
@@ -88,7 +92,7 @@ namespace mongo {
             *this = ns;
         }
         Namespace& operator=(const char *ns) {
-            uassert("ns name too long, max size is 128", strlen(ns) < MaxNsLen);
+            uassert( 10080 , "ns name too long, max size is 128", strlen(ns) < MaxNsLen);
             //memset(buf, 0, MaxNsLen); /* this is just to keep stuff clean in the files for easy dumping and reading */
             strcpy_s(buf, MaxNsLen, ns);
             return *this;
@@ -97,9 +101,13 @@ namespace mongo {
         /* for more than 10 indexes -- see NamespaceDetails::Extra */
         string extraName() { 
             string s = string(buf) + "$extra";
-            massert("ns name too long", s.size() < MaxNsLen);
+            massert( 10348 , "ns name too long", s.size() < MaxNsLen);
             return s;
         }
+        bool isExtra() const { 
+            const char *p = strstr(buf, "$extra");
+            return p && p[6] == 0; //==0 important in case an index uses name "$extra_1" for example
+        }
 
         void kill() {
             buf[0] = 0x7f;
@@ -133,9 +141,19 @@ namespace mongo {
             return old + "." + local;
         }
 
+        operator string() const {
+            return (string)buf;
+        }
+
         char buf[MaxNsLen];
     };
 
+}
+
+#include "index.h"
+
+namespace mongo {
+
     /**
        @return true if a client can modify this namespace
        things like *.system.users
@@ -149,115 +167,6 @@ namespace mongo {
     const int Buckets = 19;
     const int MaxBucket = 18;
 
-	/* Maximum # of indexes per collection.  We need to raise this limit at some point.  
-	   (Backward datafile compatibility is main issue with changing.)
-	*/
-//    const int MaxIndexes = 10;
-
-	/* Details about a particular index. There is one of these effectively for each object in 
-	   system.namespaces (although this also includes the head pointer, which is not in that 
-	   collection).
-	 */
-    class IndexDetails {
-    public:
-        DiskLoc head; /* btree head disk location */
-
-        /* Location of index info object. Format:
-
-             { name:"nameofindex", ns:"parentnsname", key: {keypattobject}[, unique: <bool>] }
-
-           This object is in the system.indexes collection.  Note that since we
-           have a pointer to the object here, the object in system.indexes MUST NEVER MOVE.
-        */
-        DiskLoc info;
-
-        /* extract key value from the query object
-           e.g., if key() == { x : 1 },
-                 { x : 70, y : 3 } -> { x : 70 }
-        */
-        BSONObj getKeyFromQuery(const BSONObj& query) const {
-            BSONObj k = keyPattern();
-            BSONObj res = query.extractFieldsUnDotted(k);
-            return res;
-        }
-
-        /* pull out the relevant key objects from obj, so we
-           can index them.  Note that the set is multiple elements
-           only when it's a "multikey" array.
-           keys will be left empty if key not found in the object.
-        */
-        void getKeysFromObject( const BSONObj& obj, BSONObjSetDefaultOrder& keys) const;
-
-        /* get the key pattern for this object.
-           e.g., { lastname:1, firstname:1 }
-        */
-        BSONObj keyPattern() const {
-            return info.obj().getObjectField("key");
-        }
-
-        /* true if the specified key is in the index */
-        bool hasKey(const BSONObj& key);
-
-        // returns name of this index's storage area
-        // database.table.$index
-        string indexNamespace() const {
-            BSONObj io = info.obj();
-            string s;
-            s.reserve(Namespace::MaxNsLen);
-            s = io.getStringField("ns");
-            assert( !s.empty() );
-            s += ".$";
-            s += io.getStringField("name");
-            return s;
-        }
-
-        string indexName() const { // e.g. "ts_1"
-            BSONObj io = info.obj();
-            return io.getStringField("name");
-        }
-
-        static bool isIdIndexPattern( const BSONObj &pattern ) {
-            BSONObjIterator i(pattern);
-            BSONElement e = i.next();
-            if( strcmp(e.fieldName(), "_id") != 0 ) return false;
-            return i.next().eoo();            
-        }
-        
-        /* returns true if this is the _id index. */
-        bool isIdIndex() const { 
-            return isIdIndexPattern( keyPattern() );
-        }
-
-        /* gets not our namespace name (indexNamespace for that),
-           but the collection we index, its name.
-           */
-        string parentNS() const {
-            BSONObj io = info.obj();
-            return io.getStringField("ns");
-        }
-
-        bool unique() const { 
-            BSONObj io = info.obj();
-            return io["unique"].trueValue() || 
-                /* temp: can we juse make unique:true always be there for _id and get rid of this? */
-                isIdIndex();
-        }
-
-        /* if set, when building index, if any duplicates, drop the duplicating object */
-        bool dropDups() const {
-            return info.obj().getBoolField( "dropDups" );
-        }
-
-        /* delete this index.  does NOT clean up the system catalog
-           (system.indexes or system.namespaces) -- only NamespaceIndex.
-        */
-        void kill_idx();
-
-        operator string() const {
-            return info.obj().toString();
-        }
-    };
-
     extern int bucketSizes[];
 
     /* this is the "header" for a collection that has all its details.  in the .ns file.
@@ -285,6 +194,9 @@ namespace mongo {
 
         BOOST_STATIC_ASSERT( NIndexesMax == NIndexesBase + NIndexesExtra );
 
+        /* called when loaded from disk */
+        void onLoad(const Namespace& k);
+
         NamespaceDetails( const DiskLoc &loc, bool _capped ) {
             /* be sure to initialize new fields here -- doesn't default to zeroes the way we use it */
             firstExtent = lastExtent = capExtent = loc;
@@ -307,11 +219,21 @@ namespace mongo {
             multiKeyIndexBits = 0;
             reservedA = 0;
             extraOffset = 0;
+            backgroundIndexBuildInProgress = 0;
             memset(reserved, 0, sizeof(reserved));
         }
         DiskLoc firstExtent;
         DiskLoc lastExtent;
+
+        /* NOTE: capped collections override the meaning of deleted list.  
+                 deletedList[0] points to a list of free records (DeletedRecord's) for all extents in
+                 the namespace.
+                 deletedList[1] points to the last record in the prev extent.  When the "current extent" 
+                 changes, this value is updated.  !deletedList[1].isValid() when this value is not 
+                 yet computed.
+        */
         DiskLoc deletedList[Buckets];
+
         long long datasize;
         long long nrecords;
         int lastExtentSize;
@@ -337,8 +259,19 @@ namespace mongo {
         unsigned long long reservedA;
         long long extraOffset; // where the $extra info is located (bytes relative to this)
     public:
-        char reserved[80];
+        int backgroundIndexBuildInProgress; // 1 if in prog
+        char reserved[76];
 
+        /* when a background index build is in progress, we don't count the index in nIndexes until 
+           complete, yet need to still use it in _indexRecord() - thus we use this function for that.
+        */
+        int nIndexesBeingBuilt() const {
+            return nIndexes + backgroundIndexBuildInProgress;
+        }
+
+        /* NOTE: be careful with flags.  are we manipulating them in read locks?  if so, 
+                 this isn't thread safe.  TODO
+        */
         enum NamespaceFlags {
             Flag_HaveIdIndex = 1 << 0, // set when we have _id index (ONLY if ensureIdIndex was called -- 0 if that has never been called)
             Flag_CappedDisallowDelete = 1 << 1 // set when deletes not allowed during capped table allocation.
@@ -349,6 +282,10 @@ namespace mongo {
                 return _indexes[idxNo];
             return extra()->details[idxNo-NIndexesBase];
         }
+        IndexDetails& backgroundIdx() { 
+            DEV assert(backgroundIndexBuildInProgress);
+            return idx(nIndexes);
+        }
 
         class IndexIterator { 
             friend class NamespaceDetails;
@@ -386,13 +323,13 @@ namespace mongo {
                 if( &i.next() == &idx )
                     return i.pos()-1;
             }
-            massert("E12000 idxNo fails", false);
+            massert( 10349 , "E12000 idxNo fails", false);
             return -1;
         }
 
         /* multikey indexes are indexes where there are more than one key in the index
              for a single document. see multikey in wiki.
-           for these, we have to do some dedup object on queries.
+           for these, we have to do some dedup work on queries.
         */
         bool isMultikey(int i) {
             return (multiKeyIndexBits & (((unsigned long long) 1) << i)) != 0;
@@ -409,7 +346,7 @@ namespace mongo {
         /* add a new index.  does not add to system.indexes etc. - just to NamespaceDetails.
            caller must populate returned object. 
          */
-        IndexDetails& addIndex(const char *thisns);
+        IndexDetails& addIndex(const char *thisns, bool resetTransient=true);
 
         void aboutToDeleteAnIndex() {
             flags &= ~Flag_HaveIdIndex;
@@ -443,6 +380,16 @@ namespace mongo {
             return -1;
         }
 
+        //returns offset in indexes[]
+        int findIndexByKeyPattern(const BSONObj& keyPattern) {
+            IndexIterator i = ii();
+            while( i.more() ) {
+                if( i.next().keyPattern() == keyPattern ) 
+                    return i.pos()-1;
+            }
+            return -1;
+        }
+
         /* @return -1 = not found 
            generally id is first index, so not that expensive an operation (assuming present).
         */
@@ -485,7 +432,7 @@ namespace mongo {
 
         void checkMigrate();
 
-        long long storageSize();
+        long long storageSize( int * numExtents = 0 );
 
     private:
         bool cappedMayDelete() const {
@@ -499,7 +446,7 @@ namespace mongo {
         DiskLoc __stdAlloc(int len);
         DiskLoc __capAlloc(int len);
         DiskLoc _alloc(const char *ns, int len);
-        void compact();
+        void compact(); // combine adjacent deleted records
 
         DiskLoc &firstDeletedInCapExtent();
         bool nextIsInCapExtent( const DiskLoc &dl ) const;
@@ -514,69 +461,117 @@ namespace mongo {
        CAUTION: Are you maintaining this properly on a collection drop()?  A dropdatabase()?  Be careful.
                 The current field "allIndexKeys" may have too many keys in it on such an occurrence;
                 as currently used that does not cause anything terrible to happen.
+
+       todo: cleanup code, need abstractions and separation
     */
     class NamespaceDetailsTransient : boost::noncopyable {
-        string ns;
-        bool haveIndexKeys;
-        set<string> allIndexKeys;
-        void computeIndexKeys();
-        int writeCount_;
-        map< QueryPattern, pair< BSONObj, long long > > queryCache_;
-        string logNS_;
-        bool logValid_;
+        /* general ------------------------------------------------------------- */
+    private:
+        string _ns;
+        void reset();
+        static std::map< string, shared_ptr< NamespaceDetailsTransient > > _map;
     public:
-        NamespaceDetailsTransient(const char *_ns) : ns(_ns), haveIndexKeys(), writeCount_(), logValid_() {
-            haveIndexKeys=false; /*lazy load them*/
+        NamespaceDetailsTransient(const char *ns) : _ns(ns), _keysComputed(false), _qcWriteCount(), _cll_enabled() { }
+        /* _get() is not threadsafe -- see get_inlock() comments */
+        static NamespaceDetailsTransient& _get(const char *ns);
+        /* use get_w() when doing write operations */
+        static NamespaceDetailsTransient& get_w(const char *ns) { 
+            DEV assertInWriteLock();
+            return _get(ns);
         }
-
-        /* get set of index keys for this namespace.  handy to quickly check if a given
-           field is indexed (Note it might be a seconary component of a compound index.)
-        */
-        set<string>& indexKeys() {
-            if ( !haveIndexKeys ) {
-                haveIndexKeys=true;
-                computeIndexKeys();
-            }
-            return allIndexKeys;
-        }
-
         void addedIndex() { reset(); }
         void deletedIndex() { reset(); }
-        void registerWriteOp() {
-            if ( queryCache_.empty() )
+        /* Drop cached information on all namespaces beginning with the specified prefix.
+           Can be useful as index namespaces share the same start as the regular collection. 
+           SLOW - sequential scan of all NamespaceDetailsTransient objects */
+        static void clearForPrefix(const char *prefix);
+
+        /* indexKeys() cache ---------------------------------------------------- */
+        /* assumed to be in write lock for this */
+    private:
+        bool _keysComputed;
+        set<string> _indexKeys;
+        void computeIndexKeys();
+    public:
+        /* get set of index keys for this namespace.  handy to quickly check if a given
+           field is indexed (Note it might be a secondary component of a compound index.)
+        */
+        set<string>& indexKeys() {
+            DEV assertInWriteLock();
+            if ( !_keysComputed )
+                computeIndexKeys();
+            return _indexKeys;
+        }
+
+        /* IndexSpec caching */
+    private:
+        map<const IndexDetails*,IndexSpec> _indexSpecs;
+        static mongo::mutex _isMutex;
+    public:
+        const IndexSpec& getIndexSpec( const IndexDetails * details ){
+            IndexSpec& spec = _indexSpecs[details];
+            if ( ! spec._finishedInit ){
+                scoped_lock lk(_isMutex);
+                if ( ! spec._finishedInit ){
+                    spec.reset( details );
+                    assert( spec._finishedInit );
+                }
+            }
+            return spec;
+        }
+
+        /* query cache (for query optimizer) ------------------------------------- */
+    private:
+        int _qcWriteCount;
+        map< QueryPattern, pair< BSONObj, long long > > _qcCache;
+    public:
+        static mongo::mutex _qcMutex;
+        /* you must be in the qcMutex when calling this (and using the returned val): */
+        static NamespaceDetailsTransient& get_inlock(const char *ns) {
+            return _get(ns);
+        }
+        void clearQueryCache() { // public for unit tests
+            _qcCache.clear();
+            _qcWriteCount = 0;
+        }
+        /* you must notify the cache if you are doing writes, as query plan optimality will change */
+        void notifyOfWriteOp() {
+            if ( _qcCache.empty() )
                 return;
-            if ( ++writeCount_ >= 100 )
+            if ( ++_qcWriteCount >= 100 )
                 clearQueryCache();
         }
-        void clearQueryCache() {
-            queryCache_.clear();
-            writeCount_ = 0;
-        }
         BSONObj indexForPattern( const QueryPattern &pattern ) {
-            return queryCache_[ pattern ].first;
+            return _qcCache[ pattern ].first;
         }
         long long nScannedForPattern( const QueryPattern &pattern ) {
-            return queryCache_[ pattern ].second;
+            return _qcCache[ pattern ].second;
         }
         void registerIndexForPattern( const QueryPattern &pattern, const BSONObj &indexKey, long long nScanned ) {
-            queryCache_[ pattern ] = make_pair( indexKey, nScanned );
+            _qcCache[ pattern ] = make_pair( indexKey, nScanned );
         }
-        
-        void startLog( int logSizeMb = 128 );
-        void invalidateLog();
-        bool validateCompleteLog();
-        string logNS() const { return logNS_; }
-        bool logValid() const { return logValid_; }
-        
+
+        /* for collection-level logging -- see CmdLogCollection ----------------- */ 
+        /* assumed to be in write lock for this */
     private:
-        void reset();
-        void dropLog();
-        static std::map< string, shared_ptr< NamespaceDetailsTransient > > map_;
+        string _cll_ns; // "local.temp.oplog." + _ns;
+        bool _cll_enabled;
+        void cllDrop(); // drop _cll_ns
     public:
-        static NamespaceDetailsTransient& get(const char *ns);
-        // Drop cached information on all namespaces beginning with the specified prefix.
-        static void drop(const char *prefix);
-    };
+        string cllNS() const { return _cll_ns; }
+        bool cllEnabled() const { return _cll_enabled; }
+        void cllStart( int logSizeMb = 256 ); // begin collection level logging
+        void cllInvalidate();
+        bool cllValidateComplete();
+
+    }; /* NamespaceDetailsTransient */
+
+    inline NamespaceDetailsTransient& NamespaceDetailsTransient::_get(const char *ns) {
+        shared_ptr< NamespaceDetailsTransient > &t = _map[ ns ];
+        if ( t.get() == 0 )
+            t.reset( new NamespaceDetailsTransient(ns) );
+        return *t;
+    }
 
     /* NamespaceIndex is the ".ns" file you see in the data directory.  It is the "system catalog"
        if you will: at least the core parts.  (Additional info in system.* collections.)
@@ -586,9 +581,9 @@ namespace mongo {
         BOOST_STATIC_ASSERT( sizeof(NamespaceDetails::Extra) <= sizeof(NamespaceDetails) );
     public:
         NamespaceIndex(const string &dir, const string &database) :
-        ht( 0 ),
-        dir_( dir ),
-        database_( database ) {}
+          ht( 0 ),
+          dir_( dir ),
+          database_( database ) {}
 
         /* returns true if new db will be created if we init lazily */
         bool exists() const;
@@ -603,7 +598,7 @@ namespace mongo {
 		void add_ns( const char *ns, const NamespaceDetails &details ) {
             init();
             Namespace n(ns);
-            uassert("too many namespaces/collections", ht->put(n, details));
+            uassert( 10081 , "too many namespaces/collections", ht->put(n, details));
 		}
 
         /* just for diagnostics */
@@ -618,12 +613,12 @@ namespace mongo {
             Namespace n(ns);
             Namespace extra(n.extraName().c_str()); // throws userexception if ns name too long
             NamespaceDetails *d = details(ns);
-            massert( "allocExtra: base ns missing?", d );
+            massert( 10350 ,  "allocExtra: base ns missing?", d );
             assert( d->extraOffset == 0 );
-            massert( "allocExtra: extra already exists", ht->get(extra) == 0 );
+            massert( 10351 ,  "allocExtra: extra already exists", ht->get(extra) == 0 );
             NamespaceDetails::Extra temp;
             memset(&temp, 0, sizeof(temp));
-            uassert( "allocExtra: too many namespaces/collections", ht->put(extra, (NamespaceDetails&) temp));
+            uassert( 10082 ,  "allocExtra: too many namespaces/collections", ht->put(extra, (NamespaceDetails&) temp));
             NamespaceDetails::Extra *e = (NamespaceDetails::Extra *) ht->get(extra);
             d->extraOffset = ((char *) e) - ((char *) d);
             assert( d->extra() == e );
@@ -668,6 +663,7 @@ namespace mongo {
 
     private:
         boost::filesystem::path path() const;
+        void maybeMkdir() const;
         
         MemoryMappedFile f;
         HashTable<Namespace,NamespaceDetails> *ht;
@@ -675,7 +671,8 @@ namespace mongo {
         string database_;
     };
 
-    extern string dbpath; // --dbpath parm 
+    extern string dbpath; // --dbpath parm
+    extern bool directoryperdb;
 
     // Rename a namespace within current 'client' db.
     // (Arguments should include db name)
