@@ -29,9 +29,10 @@
 namespace NamespaceTests {
     namespace IndexDetailsTests {
         class Base {
+            dblock lk;
+            Client::Context _context;
         public:
-            Base() {
-                setClient( ns() );
+            Base() : _context(ns()){
             }
             virtual ~Base() {
                 if ( id_.info.isNull() )
@@ -41,6 +42,7 @@ namespace NamespaceTests {
             }
         protected:
             void create() {
+                NamespaceDetailsTransient::get_w( ns() ).deletedIndex();
                 BSONObjBuilder builder;
                 builder.append( "ns", ns() );
                 builder.append( "name", "testIndex" );
@@ -142,7 +144,7 @@ namespace NamespaceTests {
                 BSONObjSetDefaultOrder keys;
                 id().getKeysFromObject( a.done(), keys );
                 checkSize( 1, keys );
-                assertEquals( e.obj(), *keys.begin() );
+                ASSERT_EQUALS( e.obj(), *keys.begin() );
             }
         private:
             virtual BSONObj key() const {
@@ -569,8 +571,11 @@ namespace NamespaceTests {
     namespace NamespaceDetailsTests {
 
         class Base {
+            const char *ns_;
+            dblock lk;
+            Client::Context _context;
         public:
-            Base( const char *ns = "unittests.NamespaceDetailsTests" ) : ns_( ns ) {}
+            Base( const char *ns = "unittests.NamespaceDetailsTests" ) : ns_( ns ) , _context( ns ) {}
             virtual ~Base() {
                 if ( !nsd() )
                     return;
@@ -582,7 +587,6 @@ namespace NamespaceTests {
         protected:
             void create() {
                 dblock lk;
-                setClient( ns() );
                 string err;
                 ASSERT( userCreateNS( ns(), fromjson( spec() ), err, false ) );
             }
@@ -624,8 +628,6 @@ namespace NamespaceTests {
                 b.append( "a", as );
                 return b.obj();
             }
-        private:
-            const char *ns_;
         };
 
         class Create : public Base {
