@@ -29,6 +29,11 @@
 
 namespace mongo {
     extern BSONObj id_obj;
+    auto_ptr< QueryResult > runQuery(Message& m, QueryMessage& q ){
+        CurOp op( &(cc()) );
+        op.ensureStarted();
+        return runQuery( m , q , op );
+    }
 } // namespace mongo
 
 namespace QueryOptimizerTests {
@@ -50,7 +55,6 @@ namespace QueryOptimizerTests {
             virtual bool lowerInclusive() { return true; }
             virtual BSONElement upper() { return maxKey.firstElement(); }
             virtual bool upperInclusive() { return true; }
-        private:
             static void checkElt( BSONElement expected, BSONElement actual ) {
                 if ( expected.woCompare( actual, false ) ) {
                     stringstream ss;
@@ -139,7 +143,17 @@ namespace QueryOptimizerTests {
             }
         };        
 
-        class Regex : public Base {
+        struct RegexBase : Base {
+            void run() { //need to only look at first interval
+                FieldRangeSet s( "ns", query() );
+                checkElt( lower(), s.range( "a" ).intervals()[0].lower_.bound_ );
+                checkElt( upper(), s.range( "a" ).intervals()[0].upper_.bound_ );
+                ASSERT_EQUALS( lowerInclusive(), s.range( "a" ).intervals()[0].lower_.inclusive_ );
+                ASSERT_EQUALS( upperInclusive(), s.range( "a" ).intervals()[0].upper_.inclusive_ );
+            }
+        };
+
+        class Regex : public RegexBase {
         public:
             Regex() : o1_( BSON( "" << "abc" ) ), o2_( BSON( "" << "abd" ) ) {}
             virtual BSONObj query() {
@@ -152,13 +166,35 @@ namespace QueryOptimizerTests {
             virtual bool upperInclusive() { return false; }
             BSONObj o1_, o2_;
         };        
+
+        class RegexObj : public RegexBase {
+        public:
+            RegexObj() : o1_( BSON( "" << "abc" ) ), o2_( BSON( "" << "abd" ) ) {}
+            virtual BSONObj query() { return BSON("a" << BSON("$regex" << "^abc")); }
+            virtual BSONElement lower() { return o1_.firstElement(); }
+            virtual BSONElement upper() { return o2_.firstElement(); }
+            virtual bool upperInclusive() { return false; }
+            BSONObj o1_, o2_;
+        };
         
-        class UnhelpfulRegex : public Base {
+        class UnhelpfulRegex : public RegexBase {
+        public:
+            UnhelpfulRegex() {
+                BSONObjBuilder b;
+                b.appendMinForType("lower", String);
+                b.appendMaxForType("upper", String);
+                limits = b.obj();
+            }
+
             virtual BSONObj query() {
                 BSONObjBuilder b;
                 b.appendRegex( "a", "abc" );
                 return b.obj();
             }            
+            virtual BSONElement lower() { return limits["lower"]; }
+            virtual BSONElement upper() { return limits["upper"]; }
+            virtual bool upperInclusive() { return false; }
+            BSONObj limits;
         };
         
         class In : public Base {
@@ -302,8 +338,7 @@ namespace QueryOptimizerTests {
     namespace QueryPlanTests {
         class Base {
         public:
-            Base() : indexNum_( 0 ) {
-                setClient( ns() );
+            Base() : _ctx( ns() ) , indexNum_( 0 ) {
                 string err;
                 userCreateNS( ns(), BSONObj(), err, false );
             }
@@ -324,7 +359,7 @@ namespace QueryOptimizerTests {
                 client_.ensureIndex( ns(), key, false, name.c_str() );
                 NamespaceDetails *d = nsd();
                 for( int i = 0; i < d->nIndexes; ++i ) {
-                    if ( d->idx(i).indexName() == name || ( d->idx(i).isIdIndex() && IndexDetails::isIdIndexPattern( key ) ) )
+                    if ( d->idx(i).keyPattern() == key /*indexName() == name*/ || ( d->idx(i).isIdIndex() && IndexDetails::isIdIndexPattern( key ) ) )
                         return &d->idx(i);
                 }
                 assert( false );
@@ -343,6 +378,7 @@ namespace QueryOptimizerTests {
             }
         private:
             dblock lk_;
+            Client::Context _ctx;
             int indexNum_;
             static DBDirectClient client_;
         };
@@ -581,15 +617,14 @@ namespace QueryOptimizerTests {
     namespace QueryPlanSetTests {
         class Base {
         public:
-            Base() {
-                setClient( ns() );
+            Base() : _context( ns() ){
                 string err;
                 userCreateNS( ns(), BSONObj(), err, false );
             }
             ~Base() {
                 if ( !nsd() )
                     return;
-                NamespaceDetailsTransient::get( ns() ).clearQueryCache();
+                NamespaceDetailsTransient::_get( ns() ).clearQueryCache();
                 string s( ns() );
                 dropNS( s );
             }
@@ -611,6 +646,7 @@ namespace QueryOptimizerTests {
             static NamespaceDetails *nsd() { return nsdetails( ns() ); }
         private:
             dblock lk_;
+            Client::Context _context;
         };
         
         class NoIndexes : public Base {
@@ -738,7 +774,10 @@ namespace QueryOptimizerTests {
                 Message m;
                 assembleRequest( "unittests.missingNS", BSONObj(), 0, 0, 0, 0, m );
                 stringstream ss;
-                ASSERT_EQUALS( 0, runQuery( m, ss )->nReturned );
+
+                DbMessage d(m);
+                QueryMessage q(d);
+                ASSERT_EQUALS( 0, runQuery( m, q)->nReturned );
             }
         };
         
@@ -775,7 +814,7 @@ namespace QueryOptimizerTests {
                 virtual void next() {
                     if ( iThrow_ )
                         threw_ = true;
-                    massert( "throw", !iThrow_ );
+                    massert( 10408 ,  "throw", !iThrow_ );
                     if ( ++i_ > 10 )
                         setComplete();
                 }
@@ -811,7 +850,7 @@ namespace QueryOptimizerTests {
             public:
                 virtual void init() {}
                 virtual void next() {
-                    massert( "throw", false );
+                    massert( 10409 ,  "throw", false );
                 }
                 virtual QueryOp *clone() const {
                     return new TestOp();
@@ -899,8 +938,8 @@ namespace QueryOptimizerTests {
                 QueryPlanSet s( ns(), BSON( "a" << 4 ), BSON( "b" << 1 ) );
                 ScanOnlyTestOp op;
                 s.runOp( op );
-                ASSERT( fromjson( "{$natural:1}" ).woCompare( NamespaceDetailsTransient::get( ns() ).indexForPattern( s.fbs().pattern( BSON( "b" << 1 ) ) ) ) == 0 );
-                ASSERT_EQUALS( 1, NamespaceDetailsTransient::get( ns() ).nScannedForPattern( s.fbs().pattern( BSON( "b" << 1 ) ) ) );
+                ASSERT( fromjson( "{$natural:1}" ).woCompare( NamespaceDetailsTransient::_get( ns() ).indexForPattern( s.fbs().pattern( BSON( "b" << 1 ) ) ) ) == 0 );
+                ASSERT_EQUALS( 1, NamespaceDetailsTransient::_get( ns() ).nScannedForPattern( s.fbs().pattern( BSON( "b" << 1 ) ) ) );
                 
                 QueryPlanSet s2( ns(), BSON( "a" << 4 ), BSON( "b" << 1 ) );
                 TestOp op2;
@@ -912,7 +951,7 @@ namespace QueryOptimizerTests {
                 virtual void init() {}
                 virtual void next() {
                     if ( qp().indexKey().firstElement().fieldName() == string( "$natural" ) )
-                        massert( "throw", false );
+                        massert( 10410 ,  "throw", false );
                     setComplete();
                 }
                 virtual QueryOp *clone() const {
@@ -924,7 +963,7 @@ namespace QueryOptimizerTests {
                 virtual void next() {
                     if ( qp().indexKey().firstElement().fieldName() == string( "$natural" ) )
                         setComplete();
-                    massert( "throw", false );
+                    massert( 10411 ,  "throw", false );
                 }
                 virtual QueryOp *clone() const {
                     return new ScanOnlyTestOp();
@@ -956,8 +995,8 @@ namespace QueryOptimizerTests {
                 BSONObj one = BSON( "a" << 1 );
                 theDataFileMgr.insert( ns(), one );
                 deleteObjects( ns(), BSON( "a" << 1 ), false );
-                ASSERT( BSON( "a" << 1 ).woCompare( NamespaceDetailsTransient::get( ns() ).indexForPattern( FieldRangeSet( ns(), BSON( "a" << 1 ) ).pattern() ) ) == 0 );
-                ASSERT_EQUALS( 2, NamespaceDetailsTransient::get( ns() ).nScannedForPattern( FieldRangeSet( ns(), BSON( "a" << 1 ) ).pattern() ) );
+                ASSERT( BSON( "a" << 1 ).woCompare( NamespaceDetailsTransient::_get( ns() ).indexForPattern( FieldRangeSet( ns(), BSON( "a" << 1 ) ).pattern() ) ) == 0 );
+                ASSERT_EQUALS( 2, NamespaceDetailsTransient::_get( ns() ).nScannedForPattern( FieldRangeSet( ns(), BSON( "a" << 1 ) ).pattern() ) );
             }
         };
         
@@ -1007,14 +1046,22 @@ namespace QueryOptimizerTests {
                 // Need to return at least 2 records to cause plan to be recorded.
                 assembleRequest( ns(), QUERY( "b" << 0 << "a" << GTE << 0 ).obj, 2, 0, 0, 0, m );
                 stringstream ss;
-                runQuery( m, ss );
-                ASSERT( BSON( "$natural" << 1 ).woCompare( NamespaceDetailsTransient::get( ns() ).indexForPattern( FieldRangeSet( ns(), BSON( "b" << 0 << "a" << GTE << 0 ) ).pattern() ) ) == 0 );
+                {
+                    DbMessage d(m);
+                    QueryMessage q(d);
+                    runQuery( m, q);
+                }
+                ASSERT( BSON( "$natural" << 1 ).woCompare( NamespaceDetailsTransient::_get( ns() ).indexForPattern( FieldRangeSet( ns(), BSON( "b" << 0 << "a" << GTE << 0 ) ).pattern() ) ) == 0 );
                 
                 Message m2;
                 assembleRequest( ns(), QUERY( "b" << 99 << "a" << GTE << 0 ).obj, 2, 0, 0, 0, m2 );
-                runQuery( m2, ss );
-                ASSERT( BSON( "a" << 1 ).woCompare( NamespaceDetailsTransient::get( ns() ).indexForPattern( FieldRangeSet( ns(), BSON( "b" << 0 << "a" << GTE << 0 ) ).pattern() ) ) == 0 );                
-                ASSERT_EQUALS( 2, NamespaceDetailsTransient::get( ns() ).nScannedForPattern( FieldRangeSet( ns(), BSON( "b" << 0 << "a" << GTE << 0 ) ).pattern() ) );
+                {
+                    DbMessage d(m2);
+                    QueryMessage q(d);
+                    runQuery( m2, q);
+                }
+                ASSERT( BSON( "a" << 1 ).woCompare( NamespaceDetailsTransient::_get( ns() ).indexForPattern( FieldRangeSet( ns(), BSON( "b" << 0 << "a" << GTE << 0 ) ).pattern() ) ) == 0 );                
+                ASSERT_EQUALS( 2, NamespaceDetailsTransient::_get( ns() ).nScannedForPattern( FieldRangeSet( ns(), BSON( "b" << 0 << "a" << GTE << 0 ) ).pattern() ) );
             }
         };
         
@@ -1111,6 +1158,7 @@ namespace QueryOptimizerTests {
             add< FieldRangeTests::EqGte >();
             add< FieldRangeTests::EqGteInvalid >();
             add< FieldRangeTests::Regex >();
+            add< FieldRangeTests::RegexObj >();
             add< FieldRangeTests::UnhelpfulRegex >();
             add< FieldRangeTests::In >();
             add< FieldRangeTests::Equality >();
