@@ -27,13 +27,17 @@
 
 #include "dbtests.h"
 
+namespace mongo {
+    extern int __findingStartInitialTimeout;
+}
+
 namespace QueryTests {
 
     class Base {
+        dblock lk;
+        Client::Context _context;
     public:
-        Base() {
-            dblock lk;
-            setClient( ns() );
+        Base() : _context( ns() ){
             addIndex( fromjson( "{\"a\":1}" ) );
         }
         ~Base() {
@@ -185,6 +189,31 @@ namespace QueryTests {
         }
     };
 
+    class PositiveLimit : public ClientBase {
+    public:
+        const char* ns;
+        PositiveLimit() : ns("unittests.querytests.PositiveLimit") {}
+        ~PositiveLimit() {
+            client().dropCollection( ns );
+        }
+
+        void testLimit(int limit){
+            ASSERT_EQUALS(client().query( ns, BSONObj(), limit )->itcount(), limit);
+        }
+        void run() {
+            for(int i=0; i<1000; i++)
+                insert( ns, BSON( GENOID << "i" << i ) );
+
+            ASSERT_EQUALS( client().query(ns, BSONObj(),    1 )->itcount(), 1);
+            ASSERT_EQUALS( client().query(ns, BSONObj(),   10 )->itcount(), 10);
+            ASSERT_EQUALS( client().query(ns, BSONObj(),  101 )->itcount(), 101);
+            ASSERT_EQUALS( client().query(ns, BSONObj(),  999 )->itcount(), 999);
+            ASSERT_EQUALS( client().query(ns, BSONObj(), 1000 )->itcount(), 1000);
+            ASSERT_EQUALS( client().query(ns, BSONObj(), 1001 )->itcount(), 1000);
+            ASSERT_EQUALS( client().query(ns, BSONObj(),    0 )->itcount(), 1000);
+        }
+    };
+
     class ReturnOneOfManyAndTail : public ClientBase {
     public:
         ~ReturnOneOfManyAndTail() {
@@ -192,10 +221,11 @@ namespace QueryTests {
         }
         void run() {
             const char *ns = "unittests.querytests.ReturnOneOfManyAndTail";
+            client().createCollection( ns, 0, true );
             insert( ns, BSON( "a" << 0 ) );
             insert( ns, BSON( "a" << 1 ) );
             insert( ns, BSON( "a" << 2 ) );
-            auto_ptr< DBClientCursor > c = client().query( ns, QUERY( "a" << GT << 0 ).hint( BSON( "$natural" << 1 ) ), 1, 0, 0, Option_CursorTailable );
+            auto_ptr< DBClientCursor > c = client().query( ns, QUERY( "a" << GT << 0 ).hint( BSON( "$natural" << 1 ) ), 1, 0, 0, QueryOption_CursorTailable );
             // If only one result requested, a cursor is not saved.
             ASSERT_EQUALS( 0, c->getCursorId() );
             ASSERT( c->more() );
@@ -210,10 +240,11 @@ namespace QueryTests {
         }
         void run() {
             const char *ns = "unittests.querytests.TailNotAtEnd";
+            client().createCollection( ns, 0, true );
             insert( ns, BSON( "a" << 0 ) );
             insert( ns, BSON( "a" << 1 ) );
             insert( ns, BSON( "a" << 2 ) );
-            auto_ptr< DBClientCursor > c = client().query( ns, Query().hint( BSON( "$natural" << 1 ) ), 2, 0, 0, Option_CursorTailable );
+            auto_ptr< DBClientCursor > c = client().query( ns, Query().hint( BSON( "$natural" << 1 ) ), 2, 0, 0, QueryOption_CursorTailable );
             ASSERT( 0 != c->getCursorId() );
             while( c->more() )
                 c->next();
@@ -234,9 +265,14 @@ namespace QueryTests {
         }
         void run() {
             const char *ns = "unittests.querytests.EmptyTail";
-            ASSERT_EQUALS( 0, client().query( ns, Query().hint( BSON( "$natural" << 1 ) ), 2, 0, 0, Option_CursorTailable )->getCursorId() );
+            client().createCollection( ns, 0, true );
+            auto_ptr< DBClientCursor > c = client().query( ns, Query().hint( BSON( "$natural" << 1 ) ), 2, 0, 0, QueryOption_CursorTailable );
+            ASSERT_EQUALS( 0, c->getCursorId() );
+            ASSERT( c->isDead() );
             insert( ns, BSON( "a" << 0 ) );
-            ASSERT( 0 != client().query( ns, QUERY( "a" << 1 ).hint( BSON( "$natural" << 1 ) ), 2, 0, 0, Option_CursorTailable )->getCursorId() );
+            c = client().query( ns, QUERY( "a" << 1 ).hint( BSON( "$natural" << 1 ) ), 2, 0, 0, QueryOption_CursorTailable );
+            ASSERT( 0 != c->getCursorId() );
+            ASSERT( !c->isDead() );            
         }
     };
 
@@ -247,14 +283,15 @@ namespace QueryTests {
         }
         void run() {
             const char *ns = "unittests.querytests.TailableDelete";
+            client().createCollection( ns, 0, true, 2 );
             insert( ns, BSON( "a" << 0 ) );
             insert( ns, BSON( "a" << 1 ) );
-            auto_ptr< DBClientCursor > c = client().query( ns, Query().hint( BSON( "$natural" << 1 ) ), 2, 0, 0, Option_CursorTailable );
+            auto_ptr< DBClientCursor > c = client().query( ns, Query().hint( BSON( "$natural" << 1 ) ), 2, 0, 0, QueryOption_CursorTailable );
             c->next();
             c->next();
             ASSERT( !c->more() );
-            client().remove( ns, QUERY( "a" << 1 ) );
             insert( ns, BSON( "a" << 2 ) );
+            insert( ns, BSON( "a" << 3 ) );
             ASSERT( !c->more() );
             ASSERT_EQUALS( 0, c->getCursorId() );
         }
@@ -267,9 +304,10 @@ namespace QueryTests {
         }
         void run() {
             const char *ns = "unittests.querytests.TailableInsertDelete";
+            client().createCollection( ns, 0, true );
             insert( ns, BSON( "a" << 0 ) );
             insert( ns, BSON( "a" << 1 ) );
-            auto_ptr< DBClientCursor > c = client().query( ns, Query().hint( BSON( "$natural" << 1 ) ), 2, 0, 0, Option_CursorTailable );
+            auto_ptr< DBClientCursor > c = client().query( ns, Query().hint( BSON( "$natural" << 1 ) ), 2, 0, 0, QueryOption_CursorTailable );
             c->next();
             c->next();
             ASSERT( !c->more() );
@@ -281,6 +319,52 @@ namespace QueryTests {
         }
     };
 
+    class TailCappedOnly : public ClientBase {
+    public:
+        ~TailCappedOnly() {
+            client().dropCollection( "unittest.querytests.TailCappedOnly" );
+        }
+        void run() {
+            const char *ns = "unittests.querytests.TailCappedOnly";
+            client().insert( ns, BSONObj() );
+            auto_ptr< DBClientCursor > c = client().query( ns, BSONObj(), 0, 0, 0, QueryOption_CursorTailable );
+            ASSERT( c->isDead() );
+            ASSERT( !client().getLastError().empty() );
+        }
+    };
+    
+    class TailableQueryOnId : public ClientBase {
+    public:
+        ~TailableQueryOnId() {
+            client().dropCollection( "unittests.querytests.TailableQueryOnId" );
+        }
+        void run() {
+            const char *ns = "unittests.querytests.TailableQueryOnId";
+            BSONObj info;
+            client().runCommand( "unittests", BSON( "create" << "querytests.TailableQueryOnId" << "capped" << true << "autoIndexId" << true ), info );
+            insert( ns, BSON( "a" << 0 ) );
+            insert( ns, BSON( "a" << 1 ) );
+            auto_ptr< DBClientCursor > c1 = client().query( ns, QUERY( "a" << GT << -1 ), 0, 0, 0, QueryOption_CursorTailable );
+            OID id;
+            id.init("000000000000000000000000");
+            auto_ptr< DBClientCursor > c2 = client().query( ns, QUERY( "_id" << GT << id ), 0, 0, 0, QueryOption_CursorTailable );
+            c1->next();
+            c1->next();
+            ASSERT( !c1->more() );
+            c2->next();
+            c2->next();
+            ASSERT( !c2->more() );
+            insert( ns, BSON( "a" << 2 ) );
+            ASSERT( c1->more() );
+            ASSERT_EQUALS( 2, c1->next().getIntField( "a" ) );
+            ASSERT( !c1->more() );
+            ASSERT( c2->more() );
+            ASSERT_EQUALS( 2, c2->next().getIntField( "a" ) );  // SERVER-645
+            ASSERT( !c2->more() );
+            ASSERT( !c2->isDead() );
+        }
+    };
+
     class OplogReplayMode : public ClientBase {
     public:
         ~OplogReplayMode() {
@@ -288,13 +372,13 @@ namespace QueryTests {
         }
         void run() {
             const char *ns = "unittests.querytests.OplogReplayMode";
-            insert( ns, BSON( "a" << 3 ) );
-            insert( ns, BSON( "a" << 0 ) );
-            insert( ns, BSON( "a" << 1 ) );
-            insert( ns, BSON( "a" << 2 ) );
-            auto_ptr< DBClientCursor > c = client().query( ns, QUERY( "a" << GT << 1 ).hint( BSON( "$natural" << 1 ) ), 0, 0, 0, Option_OplogReplay );
+            insert( ns, BSON( "ts" << 3 ) );
+            insert( ns, BSON( "ts" << 0 ) );
+            insert( ns, BSON( "ts" << 1 ) );
+            insert( ns, BSON( "ts" << 2 ) );
+            auto_ptr< DBClientCursor > c = client().query( ns, QUERY( "ts" << GT << 1 ).hint( BSON( "$natural" << 1 ) ), 0, 0, 0, QueryOption_OplogReplay );
             ASSERT( c->more() );
-            ASSERT_EQUALS( 2, c->next().getIntField( "a" ) );
+            ASSERT_EQUALS( 2, c->next().getIntField( "ts" ) );
             ASSERT( !c->more() );
         }
     };
@@ -343,6 +427,7 @@ namespace QueryTests {
             client().dropCollection( "unittests.querytests._UnderscoreNs" );
         }
         void run() {
+            ASSERT( !error() );
             const char *ns = "unittests.querytests._UnderscoreNs";
             ASSERT( client().findOne( ns, "{}" ).isEmpty() );
             client().insert( ns, BSON( "a" << 1 ) );
@@ -543,7 +628,7 @@ namespace QueryTests {
         void check( const string &hintField ) {
             const char *ns = "unittests.querytests.SubobjArr";
             ASSERT( client().query( ns, Query( "{'a.b':1}" ).hint( BSON( hintField << 1 ) ) )->more() );
-            ASSERT( !client().query( ns, Query( "{'a.b':[1]}" ).hint( BSON( hintField << 1 ) ) )->more() );
+            ASSERT( client().query( ns, Query( "{'a.b':[1]}" ).hint( BSON( hintField << 1 ) ) )->more() );
         }
     };
 
@@ -606,7 +691,7 @@ namespace QueryTests {
     public:
         void run() {
             dblock lk;
-            setClient( "unittests.DirectLocking" );
+            Client::Context ctx( "unittests.DirectLocking" );
             client().remove( "a.b", BSONObj() );
             ASSERT_EQUALS( "unittests", cc().database()->name );
         }
@@ -676,6 +761,7 @@ namespace QueryTests {
         CollectionBase( string leaf ){
             _ns = "unittests.querytests.";
             _ns += leaf;
+            client().dropCollection( ns() );
         }
         
         virtual ~CollectionBase(){
@@ -721,6 +807,10 @@ namespace QueryTests {
         }
         void run(){
             string err;
+
+            writelock lk("");            
+            Client::Context ctx( "unittests" );
+
             ASSERT( userCreateNS( ns() , fromjson( "{ capped : true , size : 2000 }" ) , err , false ) );
             for ( int i=0; i<100; i++ ){
                 insertNext();
@@ -729,7 +819,7 @@ namespace QueryTests {
             
             int a = count();
             
-            auto_ptr< DBClientCursor > c = client().query( ns() , QUERY( "i" << GT << 0 ).hint( BSON( "$natural" << 1 ) ), 0, 0, 0, Option_CursorTailable );
+            auto_ptr< DBClientCursor > c = client().query( ns() , QUERY( "i" << GT << 0 ).hint( BSON( "$natural" << 1 ) ), 0, 0, 0, QueryOption_CursorTailable );
             int n=0;
             while ( c->more() ){
                 BSONObj z = c->next();
@@ -763,6 +853,9 @@ namespace QueryTests {
         }
 
         void run(){
+            writelock lk("");
+            Client::Context ctx( "unittests" );
+            
             for ( int i=0; i<50; i++ ){
                 insert( ns() , BSON( "_id" << i << "x" << i * 2 ) );
             }
@@ -773,10 +866,10 @@ namespace QueryTests {
             ASSERT( Helpers::findOne( ns() , BSON( "_id" << 20 ) , res , true ) );
             ASSERT_EQUALS( 40 , res["x"].numberInt() );
             
-            ASSERT( Helpers::findById( ns() , BSON( "_id" << 20 ) , res ) );
+            ASSERT( Helpers::findById( cc(), ns() , BSON( "_id" << 20 ) , res ) );
             ASSERT_EQUALS( 40 , res["x"].numberInt() );
 
-            ASSERT( ! Helpers::findById( ns() , BSON( "_id" << 200 ) , res ) );
+            ASSERT( ! Helpers::findById( cc(), ns() , BSON( "_id" << 200 ) , res ) );
 
             unsigned long long slow , fast;
             
@@ -791,13 +884,31 @@ namespace QueryTests {
             {
                 Timer t;
                 for ( int i=0; i<n; i++ ){
-                    ASSERT( Helpers::findById( ns() , BSON( "_id" << 20 ) , res ) );
+                    ASSERT( Helpers::findById( cc(), ns() , BSON( "_id" << 20 ) , res ) );
                 }
                 fast = t.micros();
             }
             
             cout << "HelperTest  slow:" << slow << " fast:" << fast << endl;
+            
+            {
+                auto_ptr<CursorIterator> i = Helpers::find( ns() );
+                int n = 0;
+                while ( i->hasNext() ){
+                    BSONObj o = i->next();
+                    n++;
+                }
+                ASSERT_EQUALS( 50 , n );
 
+                i = Helpers::find( ns() , BSON( "_id" << 20 ) );
+                n = 0;
+                while ( i->hasNext() ){
+                    BSONObj o = i->next();
+                    n++;
+                }
+                ASSERT_EQUALS( 1 , n );
+            }
+            
         }
     };
 
@@ -808,6 +919,9 @@ namespace QueryTests {
         }
 
         void run(){
+            writelock lk("");
+            Client::Context ctx( "unittests" );
+
             for ( int i=0; i<1000; i++ ){
                 insert( ns() , BSON( "_id" << i << "x" << i * 2 ) );
             }
@@ -817,13 +931,133 @@ namespace QueryTests {
 
             BSONObj res;            
             for ( int i=0; i<1000; i++ ){
-                bool found = Helpers::findById( ns() , BSON( "_id" << i ) , res );
-                ASSERT_EQUALS( i % 2 , found );
+                bool found = Helpers::findById( cc(), ns() , BSON( "_id" << i ) , res );
+                ASSERT_EQUALS( i % 2 , int(found) );
             }
 
         }
     };
 
+    class ClientCursorTest : public CollectionBase{
+        ClientCursorTest() : CollectionBase( "clientcursortest" ){
+        }
+
+        void run(){
+            writelock lk("");
+            Client::Context ctx( "unittests" );
+            
+            for ( int i=0; i<1000; i++ ){
+                insert( ns() , BSON( "_id" << i << "x" << i * 2 ) );
+            }
+
+            
+        }
+    };
+
+    class FindingStart : public CollectionBase {
+    public:
+        FindingStart() : CollectionBase( "findingstart" ), _old( __findingStartInitialTimeout ) {
+            __findingStartInitialTimeout = 0;
+        }
+        ~FindingStart() {
+            __findingStartInitialTimeout = _old;
+        }
+        
+        void run() {
+            BSONObj info;
+            ASSERT( client().runCommand( "unittests", BSON( "create" << "querytests.findingstart" << "capped" << true << "size" << 1000 << "$nExtents" << 5 << "autoIndexId" << false ), info ) );
+            
+            int i = 0;
+            for( int oldCount = -1;
+                count() != oldCount;
+                oldCount = count(), client().insert( ns(), BSON( "ts" << i++ ) ) );
+
+            for( int k = 0; k < 5; ++k ) {
+                client().insert( ns(), BSON( "ts" << i++ ) );
+                int min = client().query( ns(), Query().sort( BSON( "$natural" << 1 ) ) )->next()[ "ts" ].numberInt();            
+                for( int j = -1; j < i; ++j ) {
+                    auto_ptr< DBClientCursor > c = client().query( ns(), QUERY( "ts" << GTE << j ), 0, 0, 0, QueryOption_OplogReplay );
+                    ASSERT( c->more() );
+                    BSONObj next = c->next();
+                    ASSERT( !next[ "ts" ].eoo() );
+                    ASSERT_EQUALS( ( j > min ? j : min ), next[ "ts" ].numberInt() );
+                }
+            }
+        }
+        
+    private:
+        int _old;
+    };
+
+    class FindingStartPartiallyFull : public CollectionBase {
+    public:
+        FindingStartPartiallyFull() : CollectionBase( "findingstart" ), _old( __findingStartInitialTimeout ) {
+            __findingStartInitialTimeout = 0;
+        }
+        ~FindingStartPartiallyFull() {
+            __findingStartInitialTimeout = _old;
+        }
+        
+        void run() {
+            BSONObj info;
+            ASSERT( client().runCommand( "unittests", BSON( "create" << "querytests.findingstart" << "capped" << true << "size" << 10000 << "$nExtents" << 5 << "autoIndexId" << false ), info ) );
+            
+            int i = 0;
+            for( ; i < 150; client().insert( ns(), BSON( "ts" << i++ ) ) );
+            
+            for( int k = 0; k < 5; ++k ) {
+                client().insert( ns(), BSON( "ts" << i++ ) );
+                int min = client().query( ns(), Query().sort( BSON( "$natural" << 1 ) ) )->next()[ "ts" ].numberInt();            
+                for( int j = -1; j < i; ++j ) {
+                    auto_ptr< DBClientCursor > c = client().query( ns(), QUERY( "ts" << GTE << j ), 0, 0, 0, QueryOption_OplogReplay );
+                    ASSERT( c->more() );
+                    BSONObj next = c->next();
+                    ASSERT( !next[ "ts" ].eoo() );
+                    ASSERT_EQUALS( ( j > min ? j : min ), next[ "ts" ].numberInt() );
+                }
+            }
+        }
+        
+    private:
+        int _old;
+    };
+        
+    
+    class WhatsMyUri : public CollectionBase {
+    public:
+        WhatsMyUri() : CollectionBase( "whatsmyuri" ) {}
+        void run() {
+            BSONObj result;
+            client().runCommand( "admin", BSON( "whatsmyuri" << 1 ), result );
+            ASSERT_EQUALS( unknownAddress.toString(), result[ "you" ].str() );
+        }
+    };
+    
+    namespace parsedtests {
+        class basic1 {
+        public:
+            void _test( const BSONObj& in ){
+                ParsedQuery q( "a.b" , 5 , 6 , 9 , in , BSONObj() );
+                ASSERT_EQUALS( BSON( "x" << 5 ) , q.getFilter() );
+            }
+            void run(){
+                _test( BSON( "x" << 5 ) );
+                _test( BSON( "query" << BSON( "x" << 5 ) ) );
+                _test( BSON( "$query" << BSON( "x" << 5 ) ) );
+
+                {
+                    ParsedQuery q( "a.b" , 5 , 6 , 9 , BSON( "x" << 5 ) , BSONObj() );
+                    ASSERT_EQUALS( 6 , q.getNumToReturn() );
+                    ASSERT( q.wantMore() );
+                }
+                {
+                    ParsedQuery q( "a.b" , 5 , -6 , 9 , BSON( "x" << 5 ) , BSONObj() );
+                    ASSERT_EQUALS( 6 , q.getNumToReturn() );
+                    ASSERT( ! q.wantMore() );
+                }
+            }
+        };
+    };
 
     class All : public Suite {
     public:
@@ -838,11 +1072,14 @@ namespace QueryTests {
             add< CountIndexedRegex >();
             add< BoundedKey >();
             add< GetMore >();
+            add< PositiveLimit >();
             add< ReturnOneOfManyAndTail >();
             add< TailNotAtEnd >();
             add< EmptyTail >();
             add< TailableDelete >();
             add< TailableInsertDelete >();
+            add< TailCappedOnly >();
+            add< TailableQueryOnId >();
             add< OplogReplayMode >();
             add< ArrayId >();
             add< UnderscoreNs >();
@@ -867,6 +1104,11 @@ namespace QueryTests {
             add< TailableCappedRaceCondition >();
             add< HelperTest >();
             add< HelperByIdTest >();
+            add< FindingStart >();
+            add< FindingStartPartiallyFull >();
+            add< WhatsMyUri >();
+
+            add< parsedtests::basic1 >();
         }
     } myall;
     
