@@ -37,19 +37,19 @@ namespace ReplTests {
     }    
     
     class Base {
+        dblock lk;
+        Client::Context _context;
     public:
-        Base() {
-            master = true;
+        Base() : _context( ns() ){
+            replSettings.master = true;
             createOplog();
-            dblock lk;
-            setClient( ns() );
             ensureHaveIdIndex( ns() );
         }
         ~Base() {
             try {
-                master = false;
+                replSettings.master = false;
                 deleteAll( ns() );
-                deleteAll( logNs() );
+                deleteAll( cllNS() );
             } catch ( ... ) {
                 FAIL( "Exception while cleaning up test" );
             }
@@ -58,7 +58,7 @@ namespace ReplTests {
         static const char *ns() {
             return "unittests.repltests";
         }
-        static const char *logNs() {
+        static const char *cllNS() {
             return "local.oplog.$main";
         }
         DBDirectClient *client() const { return &client_; }
@@ -80,15 +80,15 @@ namespace ReplTests {
                 out() << "expected: " << expected.toString()
                     << ", got: " << got.toString() << endl;
             }
-            ASSERT( !expected.woCompare( got ) );
+            ASSERT_EQUALS( expected , got );
         }
         BSONObj oneOp() const { 
-            return client()->findOne( logNs(), BSONObj() );
+            return client()->findOne( cllNS(), BSONObj() );
         }
         int count() const {
             int count = 0;
             dblock lk;
-            setClient( ns() );
+            Client::Context ctx( ns() );
             auto_ptr< Cursor > c = theDataFileMgr.findAll( ns() );
             for(; c->ok(); c->advance(), ++count ) {
 //                cout << "obj: " << c->current().toString() << endl;
@@ -97,9 +97,9 @@ namespace ReplTests {
         }
         static int opCount() {
             dblock lk;
-            setClient( logNs() );
+            Client::Context ctx( cllNS() );
             int count = 0;
-            for( auto_ptr< Cursor > c = theDataFileMgr.findAll( logNs() ); c->ok(); c->advance() )
+            for( auto_ptr< Cursor > c = theDataFileMgr.findAll( cllNS() ); c->ok(); c->advance() )
                 ++count;
             return count;
         }
@@ -111,17 +111,21 @@ namespace ReplTests {
                 }
             };
             dblock lk;
-            setClient( logNs() );
             vector< BSONObj > ops;
-            for( auto_ptr< Cursor > c = theDataFileMgr.findAll( logNs() ); c->ok(); c->advance() )
-                ops.push_back( c->current() );
-            setClient( ns() );
-            for( vector< BSONObj >::iterator i = ops.begin(); i != ops.end(); ++i )
-                Applier::apply( *i );
+            {
+                Client::Context ctx( cllNS() );
+                for( auto_ptr< Cursor > c = theDataFileMgr.findAll( cllNS() ); c->ok(); c->advance() )
+                    ops.push_back( c->current() );
+            }
+            {
+                Client::Context ctx( ns() );
+                for( vector< BSONObj >::iterator i = ops.begin(); i != ops.end(); ++i )
+                    Applier::apply( *i );
+            }
         }
         static void printAll( const char *ns ) {
             dblock lk;
-            setClient( ns );
+            Client::Context ctx( ns );
             auto_ptr< Cursor > c = theDataFileMgr.findAll( ns );
             vector< DiskLoc > toDelete;
             out() << "all for " << ns << endl;
@@ -132,7 +136,7 @@ namespace ReplTests {
         // These deletes don't get logged.
         static void deleteAll( const char *ns ) {
             dblock lk;
-            setClient( ns );
+            Client::Context ctx( ns );
             auto_ptr< Cursor > c = theDataFileMgr.findAll( ns );
             vector< DiskLoc > toDelete;
             for(; c->ok(); c->advance() ) {
@@ -144,7 +148,7 @@ namespace ReplTests {
         }
         static void insert( const BSONObj &o, bool god = false ) {
             dblock lk;
-            setClient( ns() );
+            Client::Context ctx( ns() );
             theDataFileMgr.insert( ns(), o.objdata(), o.objsize(), god );
         }
         static BSONObj wid( const char *json ) {
@@ -215,7 +219,7 @@ namespace ReplTests {
                 deleteAll( ns() );
             }
         private:
-            mutable unsigned long long date_;
+            mutable Date_t date_;
         };
         
         class InsertAutoId : public Base {
@@ -305,7 +309,7 @@ namespace ReplTests {
                 insert( BSON( "_id" << 1 ) );
             }
         private:
-            mutable unsigned long long date_;
+            mutable Date_t date_;
         };
         
         class UpdateSameField : public Base {
@@ -460,6 +464,75 @@ namespace ReplTests {
             BSONObj o_, q_, u_, ou_;            
         };
 
+        class UpdateInc2 : public Base {
+        public:
+            UpdateInc2() :
+            o_( fromjson( "{'_id':1,a:5}" ) ),
+            q_( fromjson( "{a:5}" ) ),
+            u_( fromjson( "{$inc:{a:3},$set:{x:5}}" ) ),
+            ou_( fromjson( "{'_id':1,a:8,x:5}" ) ) {}
+            void doIt() const {
+                client()->update( ns(), q_, u_ );
+            }
+            void check() const {
+                ASSERT_EQUALS( 1, count() );
+                checkOne( ou_ );
+            }
+            void reset() const {
+                deleteAll( ns() );
+                insert( o_ );
+            }
+        protected:
+            BSONObj o_, q_, u_, ou_;            
+        };
+        
+        class IncEmbedded : public Base {
+        public:
+            IncEmbedded() :
+            o_( fromjson( "{'_id':1,a:{b:3},b:{b:1}}" ) ),
+            q_( fromjson( "{'_id':1}" ) ),
+            u_( fromjson( "{$inc:{'a.b':1,'b.b':1}}" ) ),
+            ou_( fromjson( "{'_id':1,a:{b:4},b:{b:2}}" ) )
+            {}
+            void doIt() const {
+                client()->update( ns(), q_, u_ );
+            }
+            void check() const {
+                ASSERT_EQUALS( 1, count() );
+                checkOne( ou_ );
+            }
+            void reset() const {
+                deleteAll( ns() );
+                insert( o_ );
+            }
+        protected:
+            BSONObj o_, q_, u_, ou_;            
+        };
+
+        class IncCreates : public Base {
+        public:
+            IncCreates() :
+            o_( fromjson( "{'_id':1}" ) ),
+            q_( fromjson( "{'_id':1}" ) ),
+            u_( fromjson( "{$inc:{'a':1}}" ) ),
+            ou_( fromjson( "{'_id':1,a:1}") )
+            {}
+            void doIt() const {
+                client()->update( ns(), q_, u_ );
+            }
+            void check() const {
+                ASSERT_EQUALS( 1, count() );
+                checkOne( ou_ );
+            }
+            void reset() const {
+                deleteAll( ns() );
+                insert( o_ );
+            }
+        protected:
+            BSONObj o_, q_, u_, ou_;            
+        };
+
+
         class UpsertInsertIdMod : public Base {
         public:
             UpsertInsertIdMod() :
@@ -843,6 +916,24 @@ namespace ReplTests {
             }            
         };
 
+        class BitOp : public Base {
+        public:
+            void doIt() const {
+                client()->update( ns(), BSON( "_id" << 0 ), fromjson( "{$bit:{a:{and:2,or:8}}}" ) );
+            }
+            using ReplTests::Base::check;
+            void check() const {
+                ASSERT_EQUALS( 1, count() );
+                check( BSON( "_id" << 0 << "a" << ( ( 3 & 2 ) | 8 ) ) , one( fromjson( "{'_id':0}" ) ) );
+            }
+            void reset() const {
+                deleteAll( ns() );
+                insert( fromjson( "{'_id':0,a:3}" ) );
+            }            
+        };
+
+        
+
     } // namespace Idempotence
     
     class DeleteOpIsIdBased : public Base {
@@ -867,7 +958,7 @@ namespace ReplTests {
     class DbIdsTest {
     public:
         void run() {
-            setClient( "unittests.repltest.DbIdsTest" );
+            Client::Context ctx( "unittests.repltest.DbIdsTest" );
             
             s_.reset( new DbIds( "local.temp.DbIdsTest" ) );
             s_->reset();
@@ -942,7 +1033,7 @@ namespace ReplTests {
     class IdTrackerTest {
     public:
         void run() {
-            setClient( "unittests.repltests.IdTrackerTest" );
+            Client::Context ctx( "unittests.repltests.IdTrackerTest" );
             
             ASSERT( s_.inMem() );
             s_.reset( 4 * sizeof( BSONObj ) - 1 );
@@ -962,7 +1053,8 @@ namespace ReplTests {
             check();
             ASSERT( !s_.inMem() );            
             
-            s_.reset();
+            s_.reset( 4 * sizeof( BSONObj ) - 1 );
+            s_.mayUpgradeStorage();
             ASSERT( s_.inMem() );                        
         }
     private:
@@ -998,6 +1090,9 @@ namespace ReplTests {
             add< Idempotence::UpsertInsertNoMods >();
             add< Idempotence::UpdateSet >();
             add< Idempotence::UpdateInc >();
+            add< Idempotence::UpdateInc2 >();
+            add< Idempotence::IncEmbedded >(); // SERVER-716
+            add< Idempotence::IncCreates >(); // SERVER-717
             add< Idempotence::UpsertInsertIdMod >();
             add< Idempotence::UpsertInsertSet >();
             add< Idempotence::UpsertInsertInc >();
@@ -1020,6 +1115,7 @@ namespace ReplTests {
             add< Idempotence::PullAll >();
             add< Idempotence::Pop >();
             add< Idempotence::PopReverse >();
+            add< Idempotence::BitOp >();
             add< DeleteOpIsIdBased >();
             add< DbIdsTest >();
             add< MemIdsTest >();
