@@ -12,11 +12,10 @@ soonCount = function( count ) {
 
 doTest = function( signal ) {
 
-    ports = allocatePorts( 2 );
+    rt = new ReplTest( "repl1tests" );
     
-    // spec small oplog for fast startup on 64bit machines
-    m = startMongod( "--port", ports[ 0 ], "--dbpath", "/data/db/" + baseName + "-master", "--master", "--oplogSize", "1", "--nohttpinterface", "--noprealloc", "--bind_ip", "127.0.0.1" );
-    s = startMongod( "--port", ports[ 1 ], "--dbpath", "/data/db/" + baseName + "-slave", "--slave", "--source", "127.0.0.1:" + ports[ 0 ], "--nohttpinterface", "--noprealloc", "--bind_ip", "127.0.0.1" );
+    m = rt.start( true );
+    s = rt.start( false );
     
     am = m.getDB( baseName ).a
     
@@ -28,19 +27,19 @@ doTest = function( signal ) {
     assert.eq( 1, as.find( { i: 0 } ).count() );
     assert.eq( 1, as.find( { i: 999 } ).count() );
 
-    stopMongod( ports[ 1 ], signal );
+    rt.stop( false, signal );
     
     for( i = 1000; i < 1010; ++i )
         am.save( { _id: new ObjectId(), i: i } );
 
-    s = startMongoProgram( "mongod", "--port", ports[ 1 ], "--dbpath", "/data/db/" + baseName + "-slave", "--slave", "--source", "127.0.0.1:" + ports[ 0 ], "--nohttpinterface", "--noprealloc", "--bind_ip", "127.0.0.1" );
+    s = rt.start( false, null, true );
     soonCount( 1010 );
     as = s.getDB( baseName ).a
     assert.eq( 1, as.find( { i: 1009 } ).count() );
 
-    stopMongod( ports[ 0 ], signal );
+    rt.stop( true, signal );
     
-    m = startMongoProgram( "mongod", "--port", ports[ 0 ], "--dbpath", "/data/db/" + baseName + "-master", "--master", "--oplogSize", "1", "--nohttpinterface", "--noprealloc", "--bind_ip", "127.0.0.1" );
+    m = rt.start( true, null, true );
     am = m.getDB( baseName ).a
 
     for( i = 1010; i < 1020; ++i )
@@ -49,7 +48,7 @@ doTest = function( signal ) {
     assert.soon( function() { return as.find().count() == 1020; } );
     assert.eq( 1, as.find( { i: 1019 } ).count() );
 
-    ports.forEach( function( x ) { stopMongod( x ); } );
+    rt.stop();
 }
 
 doTest( 15 ); // SIGTERM
