@@ -1,5 +1,5 @@
 
-t = db.foo;
+t = db.jstests_uniqueness;
 
 t.drop();
 
@@ -22,11 +22,11 @@ assert( db.getLastError() , 4);
 assert( t.findOne( {_id:4} ), 5 );
 
 // Check for an error message when we index and there are dups 
-db.bar.drop();
-db.bar.insert({a:3});
-db.bar.insert({a:3});
-assert( db.bar.count() == 2 , 6) ;
-db.bar.ensureIndex({a:1}, true);
+db.jstests_uniqueness2.drop();
+db.jstests_uniqueness2.insert({a:3});
+db.jstests_uniqueness2.insert({a:3});
+assert( db.jstests_uniqueness2.count() == 2 , 6) ;
+db.jstests_uniqueness2.ensureIndex({a:1}, true);
 assert( db.getLastError() , 7);
 
 /* Check that if we update and remove _id, it gets added back by the DB */
