@@ -28,7 +28,7 @@ namespace mongo {
 
     // -------  Shard --------
 
-    long Chunk::MaxChunkSize = 1024 * 1204 * 50;
+    int Chunk::MaxChunkSize = 1024 * 1204 * 200;
     
     Chunk::Chunk( ChunkManager * manager ) : _manager( manager ){
         _modified = false;
@@ -41,13 +41,13 @@ namespace mongo {
         _markModified();
     }
     
-    bool Chunk::contains( const BSONObj& obj ){
+    bool Chunk::contains( const BSONObj& obj ) const{
         return
             _manager->getShardKey().compare( getMin() , obj ) <= 0 &&
             _manager->getShardKey().compare( obj , getMax() ) < 0;
     }
 
-    BSONObj Chunk::pickSplitPoint(){
+    BSONObj Chunk::pickSplitPoint() const{
         int sort = 0;
         
         if ( _manager->getShardKey().globalMin().woCompare( getMin() ) == 0 ){
@@ -63,13 +63,13 @@ namespace mongo {
             if ( sort == 1 )
                 q.sort( _manager->getShardKey().key() );
             else {
-                BSONObj k = _manager->getShardKey().keyDotted();
+                BSONObj k = _manager->getShardKey().key();
                 BSONObjBuilder r;
                 
                 BSONObjIterator i(k);
                 while( i.more() ) {
                     BSONElement e = i.next();
-                    uassert( "can only handle numbers here - which i think is correct" , e.isNumber() );
+                    uassert( 10163 ,  "can only handle numbers here - which i think is correct" , e.isNumber() );
                     r.append( e.fieldName() , -1 * e.number() );
                 }
                 
@@ -77,7 +77,7 @@ namespace mongo {
             }
             BSONObj end = conn->findOne( _ns , q );
             conn.done();
-            
+
             if ( ! end.isEmpty() )
                 return _manager->getShardKey().extractKey( end );
         }
@@ -85,17 +85,33 @@ namespace mongo {
         ScopedDbConnection conn( getShard() );
         BSONObj result;
         if ( ! conn->runCommand( "admin" , BSON( "medianKey" << _ns
-                                                 << "keyPattern" << _manager->getShardKey().keyDotted() 
-                                                 << "min" << getMinDotted() 
-                                                 << "max" << getMaxDotted() 
+                                                 << "keyPattern" << _manager->getShardKey().key()
+                                                 << "min" << getMin()
+                                                 << "max" << getMax()
                                                  ) , result ) ){
             stringstream ss;
             ss << "medianKey command failed: " << result;
-            uassert( ss.str() , 0 );
+            uassert( 10164 ,  ss.str() , 0 );
         }
+
+        BSONObj median = result.getObjectField( "median" );
+        if (median == getMin()){
+            //TODO compound support
+            BSONElement key = getMin().firstElement();
+            BSONObjBuilder b;
+            b.appendAs("$gt", key);
+
+            Query q = QUERY(key.fieldName() << b.obj());
+            q.sort(_manager->getShardKey().key());
+
+            median = conn->findOne(_ns, q);
+            median = _manager->getShardKey().extractKey( median );
+            PRINT(median);
+        }
+
         conn.done();
         
-        return result.getObjectField( "median" ).getOwned();
+        return median.getOwned();
     }
 
     Chunk * Chunk::split(){
@@ -103,19 +119,19 @@ namespace mongo {
     }
     
     Chunk * Chunk::split( const BSONObj& m ){
-        uassert( "can't split as shard that doesn't have a manager" , _manager );
+        uassert( 10165 ,  "can't split as shard that doesn't have a manager" , _manager );
         
         log(1) << " before split on: "  << m << "\n"
                << "\t self  : " << toString() << endl;
 
-        uassert( "locking namespace on server failed" , lockNamespaceOnServer( getShard() , _ns ) );
-
-        BSONObj nested = dotted2nested(m);
+        uassert( 10166 ,  "locking namespace on server failed" , lockNamespaceOnServer( getShard() , _ns ) );
+        uassert( 13003 ,  "can't split chunk. does it have only one distinct value?" ,
+                          !m.isEmpty() && _min.woCompare(m) && _max.woCompare(m)); 
 
         Chunk * s = new Chunk( _manager );
         s->_ns = _ns;
         s->_shard = _shard;
-        s->setMin(nested.getOwned());
+        s->setMin(m.getOwned());
         s->setMax(_max);
         
         s->_markModified();
@@ -123,7 +139,7 @@ namespace mongo {
         
         _manager->_chunks.push_back( s );
         
-        setMax(nested.getOwned());
+        setMax(m.getOwned());
         
         log(1) << " after split:\n" 
                << "\t left : " << toString() << "\n" 
@@ -136,7 +152,7 @@ namespace mongo {
     }
 
     bool Chunk::moveAndCommit( const string& to , string& errmsg ){
-        uassert( "can't move shard to its current location!" , to != getShard() );
+        uassert( 10167 ,  "can't move shard to its current location!" , to != getShard() );
 
         log() << "moving chunk ns: " << _ns << " moving chunk: " << toString() << " " << _shard << " -> " << to << endl;
         
@@ -189,7 +205,7 @@ namespace mongo {
             }
             else if ( newVersion <= oldVersion ){
                 log() << "newVersion: " << newVersion << " oldVersion: " << oldVersion << endl;
-                uassert( "version has to be higher" , newVersion > oldVersion );
+                uassert( 10168 ,  "version has to be higher" , newVersion > oldVersion );
             }
             
             BSONObjBuilder b;
@@ -218,10 +234,13 @@ namespace mongo {
         
         if ( _dataWritten < MaxChunkSize / 5 )
             return false;
+        
+        log(1) << "\t want to split chunk : " << this << endl;
 
         _dataWritten = 0;
         
-        if ( _min.woCompare( _max ) == 0 ){
+        BSONObj split_point = pickSplitPoint();
+        if ( split_point.isEmpty() || _min == split_point || _max == split_point) {
             log() << "SHARD PROBLEM** shard is too big, but can't split: " << toString() << endl;
             return false;
         }
@@ -231,7 +250,7 @@ namespace mongo {
             return false;
         
         log() << "autosplitting " << _ns << " size: " << size << " shard: " << toString() << endl;
-        Chunk * newShard = split();
+        Chunk * newShard = split(split_point);
 
         moveIfShould( newShard );
         
@@ -264,17 +283,17 @@ namespace mongo {
         log() << "moving chunk (auto): " << toMove->toString() << " to: " << newLocation << " #objcets: " << toMove->countObjects() << endl;
 
         string errmsg;
-        massert( (string)"moveAndCommit failed: " + errmsg , 
+        massert( 10412 ,  (string)"moveAndCommit failed: " + errmsg , 
                  toMove->moveAndCommit( newLocation , errmsg ) );
         
         return true;
     }
 
-    long Chunk::getPhysicalSize(){
+    long Chunk::getPhysicalSize() const{
         ScopedDbConnection conn( getShard() );
         
         BSONObj result;
-        uassert( "datasize failed!" , conn->runCommand( "admin" , BSON( "datasize" << _ns
+        uassert( 10169 ,  "datasize failed!" , conn->runCommand( "admin" , BSON( "datasize" << _ns
                                                                         << "keyPattern" << _manager->getShardKey().key() 
                                                                         << "min" << getMin() 
                                                                         << "max" << getMax() 
@@ -285,7 +304,7 @@ namespace mongo {
     }
 
     
-    long Chunk::countObjects( const BSONObj& filter ){
+    long Chunk::countObjects( const BSONObj& filter ) const{
         ScopedDbConnection conn( getShard() );
         
         BSONObj f = getFilter();
@@ -299,14 +318,14 @@ namespace mongo {
         return (long)n;
     }
     
-    bool Chunk::operator==( const Chunk& s ){
+    bool Chunk::operator==( const Chunk& s ) const{
         return 
             _manager->getShardKey().compare( _min , s._min ) == 0 &&
             _manager->getShardKey().compare( _max , s._max ) == 0
             ;
     }
 
-    void Chunk::getFilter( BSONObjBuilder& b ){
+    void Chunk::getFilter( BSONObjBuilder& b ) const{
         _manager->_key.getFilter( b , _min , _max );
     }
     
@@ -318,26 +337,30 @@ namespace mongo {
 
         to << "ns" << _ns;
         to << "min" << _min;
-        to << "minDotted" << _minDotted;
         to << "max" << _max;
-        to << "maxDotted" << _maxDotted;
         to << "shard" << _shard;
     }
     
     void Chunk::unserialize(const BSONObj& from){
         _ns = from.getStringField( "ns" );
-        _min = from.getObjectField( "min" ).getOwned();
-        _minDotted = from.getObjectField( "minDotted" ).getOwned();
-        _max = from.getObjectField( "max" ).getOwned();
-        _maxDotted = from.getObjectField( "maxDotted" ).getOwned();
         _shard = from.getStringField( "shard" );
-        _lastmod = from.hasField( "lastmod" ) ? from["lastmod"].date() : 0;
-        
-        uassert( "Chunk needs a ns" , ! _ns.empty() );
-        uassert( "Chunk needs a server" , ! _ns.empty() );
+        _lastmod = from.hasField( "lastmod" ) ? from["lastmod"]._numberLong() : 0;
 
-        uassert( "Chunk needs a min" , ! _min.isEmpty() );
-        uassert( "Chunk needs a max" , ! _max.isEmpty() );
+        BSONElement e = from["minDotted"];
+        cout << from << endl;
+        if (e.eoo()){
+            _min = from.getObjectField( "min" ).getOwned();
+            _max = from.getObjectField( "max" ).getOwned();
+        } else { // TODO delete this case after giving people a chance to migrate
+            _min = e.embeddedObject().getOwned();
+            _max = from.getObjectField( "maxDotted" ).getOwned();
+        }
+        
+        uassert( 10170 ,  "Chunk needs a ns" , ! _ns.empty() );
+        uassert( 10171 ,  "Chunk needs a server" , ! _ns.empty() );
+
+        uassert( 10172 ,  "Chunk needs a min" , ! _min.isEmpty() );
+        uassert( 10173 ,  "Chunk needs a max" , ! _max.isEmpty() );
     }
 
     string Chunk::modelServer() {
@@ -347,9 +370,7 @@ namespace mongo {
     
     void Chunk::_markModified(){
         _modified = true;
-
-        unsigned long long t = time(0);
-        t *= 1000;
+        // set to 0 so that the config server sets it
         _lastmod = 0;
     }
 
@@ -358,32 +379,32 @@ namespace mongo {
         Model::save( check );
         if ( reload ){
             // need to do this so that we get the new _lastMod and therefore version number
-            massert( "_id has to be filled in already" , ! _id.isEmpty() );
+            massert( 10413 ,  "_id has to be filled in already" , ! _id.isEmpty() );
             
             string b = toString();
             BSONObj q = _id.copy();
-            massert( "how could load fail?" , load( q ) );
+            massert( 10414 ,  "how could load fail?" , load( q ) );
             log(2) << "before: " << q << "\t" << b << endl;
             log(2) << "after : " << _id << "\t" << toString() << endl;
-            massert( "chunk reload changed content!" , b == toString() );
-            massert( "id changed!" , q["_id"] == _id["_id"] );
+            massert( 10415 ,  "chunk reload changed content!" , b == toString() );
+            massert( 10416 ,  "id changed!" , q["_id"] == _id["_id"] );
         }
     }
     
     void Chunk::ensureIndex(){
         ScopedDbConnection conn( getShard() );
-        conn->ensureIndex( _ns , _manager->getShardKey().keyDotted() , _manager->_unique );
+        conn->ensureIndex( _ns , _manager->getShardKey().key() , _manager->_unique );
         conn.done();
     }
 
     string Chunk::toString() const {
         stringstream ss;
-        ss << "shard  ns:" << _ns << " shard: " << _shard << " min: " << _minDotted << " max: " << _maxDotted;
+        ss << "shard  ns:" << _ns << " shard: " << _shard << " min: " << _min << " max: " << _max;
         return ss.str();
     }
     
     
-    ShardKeyPattern Chunk::skey(){
+    ShardKeyPattern Chunk::skey() const{
         return _manager->getShardKey();
     }
 
@@ -446,7 +467,7 @@ namespace mongo {
         }
         stringstream ss;
         ss << "couldn't find a chunk which should be impossible  extracted: " << _key.extractKey( obj );
-        throw UserException( ss.str() );
+        throw UserException( 8070 , ss.str() );
     }
 
     Chunk* ChunkManager::findChunkOnServer( const string& server ) const {
@@ -492,7 +513,7 @@ namespace mongo {
     }
     
     void ChunkManager::drop(){
-        uassert( "config servers not all up" , configServer.allUp() );
+        uassert( 10174 ,  "config servers not all up" , configServer.allUp() );
         
         map<string,ShardChunkVersion> seen;
         
@@ -509,7 +530,7 @@ namespace mongo {
                 continue;
 
             // rollback
-            uassert( "don't know how to rollback locks b/c drop can't lock all shards" , 0 );
+            uassert( 10175 ,  "don't know how to rollback locks b/c drop can't lock all shards" , 0 );
         }
         
         log(1) << "ChunkManager::drop : " << _ns << "\t all locked" << endl;        
@@ -529,7 +550,7 @@ namespace mongo {
         log(1) << "ChunkManager::drop : " << _ns << "\t removed shard data" << endl;        
 
         // clean up database meta-data
-        uassert( "no sharding data?" , _config->removeSharding( _ns ) );
+        uassert( 10176 ,  "no sharding data?" , _config->removeSharding( _ns ) );
         _config->save();
         
         
@@ -544,7 +565,7 @@ namespace mongo {
             ScopedDbConnection conn( i->first );
             BSONObj res;
             if ( ! setShardVersion( conn.conn() , _ns , 0 , true , res ) )
-                throw UserException( (string)"OH KNOW, cleaning up after drop failed: " + res.toString() );
+                throw UserException( 8071 , (string)"OH KNOW, cleaning up after drop failed: " + res.toString() );
             conn.done();
         }
 
@@ -567,7 +588,7 @@ namespace mongo {
             withRealChunks.insert( c->getShard() );
         }
         
-        massert( "how did version get smalled" , getVersion() >= a );
+        massert( 10417 ,  "how did version get smalled" , getVersion() >= a );
 
         ensureIndex(); // TODO: this is too aggressive - but not really sooo bad
     }
