@@ -54,6 +54,9 @@ namespace mongo {
             virtual bool adminOnly() {
                 return true;
             }
+
+            // all grid commands are designed not to lock
+            virtual LockType locktype(){ return NONE; } 
         };
 
         // --------------- misc commands ----------------------
@@ -213,6 +216,8 @@ namespace mongo {
                     errmsg = "already enabled";
                     return false;
                 }
+                
+                log() << "enabling sharding on: " << dbname << endl;
 
                 config->enableSharding();
                 config->save( true );
@@ -283,6 +288,8 @@ namespace mongo {
 
                     conn.done();
                 }
+                
+                log() << "CMD: shardcollection: " << cmdObj << endl;
 
                 config->shardCollection( ns , key , cmdObj["unique"].trueValue() );
                 config->save( true );
@@ -359,9 +366,9 @@ namespace mongo {
                         return false;
                     }
                 }
-
+                
                 ChunkManager * info = config->getChunkManager( ns );
-                Chunk& old = info->findChunk( dotted2nested(find) );
+                Chunk& old = info->findChunk( find );
 
                 return _split( result , errmsg , ns , info , old , cmdObj.getObjectField( "middle" ) );
             }
@@ -438,9 +445,11 @@ namespace mongo {
                     errmsg = "you have to specify where you want to move the chunk";
                     return false;
                 }
+                
+                log() << "CMD: movechunk: " << cmdObj << endl;
 
                 ChunkManager * info = config->getChunkManager( ns );
-                Chunk& c = info->findChunk( dotted2nested(find) );
+                Chunk& c = info->findChunk( find );
                 string from = c.getShard();
 
                 if ( from == to ){
@@ -500,8 +509,7 @@ namespace mongo {
                 
                 if ( host == "localhost" || host.find( "localhost:" ) == 0 ||
                      host == "127.0.0.1" || host.find( "127.0.0.1:" ) == 0 ){
-                    if ( cmdObj["allowLocal"].type() != Bool || 
-                         ! cmdObj["allowLocal"].boolean() ){
+                    if ( ! cmdObj["allowLocal"].trueValue() ){
                         errmsg = 
                             "can't use localhost as a shard since all shards need to communicate.  "
                             "allowLocal to override for testing";
@@ -580,6 +588,7 @@ namespace mongo {
 
         class IsDbGridCmd : public Command {
         public:
+            virtual LockType locktype(){ return NONE; } 
             virtual bool slaveOk() {
                 return true;
             }
@@ -593,6 +602,7 @@ namespace mongo {
 
         class CmdIsMaster : public Command {
         public:
+            virtual LockType locktype(){ return NONE; } 
             virtual bool requiresAuth() { return false; }
             virtual bool slaveOk() {
                 return true;
@@ -610,6 +620,7 @@ namespace mongo {
 
         class CmdShardingGetPrevError : public Command {
         public:
+            virtual LockType locktype(){ return NONE; } 
             virtual bool requiresAuth() { return false; }
             virtual bool slaveOk() {
                 return true;
@@ -626,6 +637,7 @@ namespace mongo {
 
         class CmdShardingGetLastError : public Command {
         public:
+            virtual LockType locktype(){ return NONE; } 
             virtual bool requiresAuth() { return false; }
             virtual bool slaveOk() {
                 return true;
