@@ -48,7 +48,7 @@ namespace mongo {
         if ( isSharded( ns ) )
             return "";
         
-        uassert( "no primary!" , _primary.size() );
+        uassert( 10178 ,  "no primary!" , _primary.size() );
         return _primary;
     }
     
@@ -58,15 +58,16 @@ namespace mongo {
     
     ChunkManager* DBConfig::shardCollection( const string& ns , ShardKeyPattern fieldsAndOrder , bool unique ){
         if ( ! _shardingEnabled )
-            throw UserException( "db doesn't have sharding enabled" );
+            throw UserException( 8042 , "db doesn't have sharding enabled" );
         
         ChunkManager * info = _shards[ns];
         if ( info )
             return info;
         
         if ( isSharded( ns ) )
-            throw UserException( "already sharded" );
+            throw UserException( 8043 , "already sharded" );
 
+        log() << "enable sharding on: " << ns << " with shard key: " << fieldsAndOrder << endl;
         _sharded[ns] = CollectionInfo( fieldsAndOrder , unique );
 
         info = new ChunkManager( this , ns , fieldsAndOrder , unique );
@@ -88,8 +89,8 @@ namespace mongo {
             cout << "BBBB" << endl;
             return false;
         }
-        uassert( "_sharded but no info" , info );
-        uassert( "info but no sharded" , i != _sharded.end() );
+        uassert( 10179 ,  "_sharded but no info" , info );
+        uassert( 10180 ,  "info but no sharded" , i != _sharded.end() );
         
         _sharded.erase( i );
         _shards.erase( ns ); // TODO: clean this up, maybe switch to shared_ptr
@@ -101,7 +102,7 @@ namespace mongo {
         if ( m && ! reload )
             return m;
 
-        uassert( (string)"not sharded:" + ns , isSharded( ns ) );
+        uassert( 10181 ,  (string)"not sharded:" + ns , isSharded( ns ) );
         if ( m && reload )
             log() << "reloading shard info for: " << ns << endl;
         m = new ChunkManager( this , ns , _sharded[ ns ].key , _sharded[ns].unique );
@@ -128,6 +129,8 @@ namespace mongo {
     
     void DBConfig::unserialize(const BSONObj& from){
         _name = from.getStringField("name");
+        log(1) << "DBConfig unserialize: " << _name << " " << from << endl;
+
         _shardingEnabled = from.getBoolField("partitioned");
         _primary = from.getStringField("primary");
         
@@ -137,9 +140,9 @@ namespace mongo {
             BSONObjIterator i(sharded);
             while ( i.more() ){
                 BSONElement e = i.next();
-                uassert( "sharded things have to be objects" , e.type() == Object );
+                uassert( 10182 ,  "sharded things have to be objects" , e.type() == Object );
                 BSONObj c = e.embeddedObject();
-                uassert( "key has to be an object" , c["key"].type() == Object );
+                uassert( 10183 ,  "key has to be an object" , c["key"].type() == Object );
                 _sharded[e.fieldName()] = CollectionInfo( c["key"].embeddedObject() , 
                                                           c["unique"].trueValue() );
             }
@@ -173,7 +176,7 @@ namespace mongo {
          * 5) drop everywhere to clean up loose ends
          */
 
-        log(1) << "DBConfig::dropDatabase: " << _name << endl;
+        log() << "DBConfig::dropDatabase: " << _name << endl;
         
         // 1
         if ( ! configServer.allUp( errmsg ) ){
@@ -251,7 +254,7 @@ namespace mongo {
             i->second->drop();
             
             num++;
-            uassert( "_dropShardedCollections too many collections - bailing" , num < 100000 );
+            uassert( 10184 ,  "_dropShardedCollections too many collections - bailing" , num < 100000 );
             log(2) << "\t\t dropped " << num << " so far" << endl;
         }
         return true;
@@ -296,7 +299,7 @@ namespace mongo {
         if ( database == "config" )
             return &configServer;
 
-        boostlock l( _lock );
+        scoped_lock l( _lock );
 
         DBConfig*& cc = _databases[database];
         if ( cc == 0 ){
@@ -317,7 +320,7 @@ namespace mongo {
                     }
                     else {
                         log() << "\t can't find a shard to put new db on" << endl;
-                        uassert( "can't find a shard to put new db on" , 0 );
+                        uassert( 10185 ,  "can't find a shard to put new db on" , 0 );
                     }
                 }
                 else {
@@ -331,8 +334,8 @@ namespace mongo {
     }
 
     void Grid::removeDB( string database ){
-        uassert( "removeDB expects db name" , database.find( '.' ) == string::npos );
-        boostlock l( _lock );
+        uassert( 10186 ,  "removeDB expects db name" , database.find( '.' ) == string::npos );
+        scoped_lock l( _lock );
         _databases.erase( database );
         
     }
@@ -341,10 +344,10 @@ namespace mongo {
         ScopedDbConnection conn( configServer.getPrimary() );
         
         BSONObj result;
-        massert( "getoptime failed" , conn->simpleCommand( "admin" , &result , "getoptime" ) );
+        massert( 10421 ,  "getoptime failed" , conn->simpleCommand( "admin" , &result , "getoptime" ) );
         conn.done();
 
-        return result["optime"].date();
+        return result["optime"]._numberLong();
     }
 
     /* --- ConfigServer ---- */
@@ -359,7 +362,7 @@ namespace mongo {
     }
 
     bool ConfigServer::init( vector<string> configHosts ){
-        uassert( "need configdbs" , configHosts.size() );
+        uassert( 10187 ,  "need configdbs" , configHosts.size() );
 
         string hn = getHostName();
         if ( hn.empty() ) {
@@ -368,30 +371,35 @@ namespace mongo {
         }
         ourHostname = hn;
         
+        stringstream fullString;
+
         set<string> hosts;
         for ( size_t i=0; i<configHosts.size(); i++ ){
             string host = configHosts[i];
             hosts.insert( getHost( host , false ) );
             configHosts[i] = getHost( host , true );
+            if ( i > 0 )
+                fullString << ",";
+            fullString << configHosts[i];
         }
-
+        
         for ( set<string>::iterator i=hosts.begin(); i!=hosts.end(); i++ ){
             string host = *i;
             bool ok = false;
-            for ( int x=0; x<10; x++ ){
+            for ( int x=10; x>0; x-- ){
                 if ( ! hostbyname( host.c_str() ).empty() ){
                     ok = true;
                     break;
                 }
-                log() << "can't resolve DNS for [" << host << "]  sleeping and trying " << (10-x) << " more times" << endl;
+                log() << "can't resolve DNS for [" << host << "]  sleeping and trying " << x << " more times" << endl;
                 sleepsecs( 10 );
             }
             if ( ! ok )
                 return false;
         }
         
-        uassert( "can only hand 1 config db right now" , configHosts.size() == 1 );
-        _primary = configHosts[0];
+        _primary = fullString.str();
+        log(1) << " config string : " << fullString.str() << endl;
         
         return true;
     }
@@ -429,7 +437,7 @@ namespace mongo {
         if ( c->more() ){
             BSONObj o = c->next();
             version = o["version"].numberInt();
-            uassert( "should only have 1 thing in config.version" , ! c->more() );
+            uassert( 10189 ,  "should only have 1 thing in config.version" , ! c->more() );
         }
         else {
             if ( conn.count( "config.shard" ) || conn.count( "config.databases" ) ){
@@ -447,7 +455,7 @@ namespace mongo {
         
         if ( cur == 0 ){
             ScopedDbConnection conn( _primary );
-            conn->insert( "config.version" , BSON( "version" << VERSION ) );
+            conn->insert( "config.version" , BSON( "_id" << 1 << "version" << VERSION ) );
             pool.flush();
             assert( VERSION == dbConfigVersion( conn.conn() ) );
             conn.done();
@@ -458,6 +466,32 @@ namespace mongo {
         return -8;
     }
 
+    void ConfigServer::reloadSettings(){
+        set<string> got;
+        
+        ScopedDbConnection conn( _primary );
+        auto_ptr<DBClientCursor> c = conn->query( "config.settings" , BSONObj() );
+        while ( c->more() ){
+            BSONObj o = c->next();
+            string name = o["_id"].valuestrsafe();
+            got.insert( name );
+            if ( name == "chunksize" ){
+                log(1) << "MaxChunkSize: " << o["value"] << endl;
+                Chunk::MaxChunkSize = o["value"].numberInt() * 1024 * 1024;
+            }
+            else {
+                log() << "warning: unknown setting [" << name << "]" << endl;
+            }
+        }
+
+        if ( ! got.count( "chunksize" ) ){
+            conn->insert( "config.settings" , BSON( "_id" << "chunksize"  <<
+                                                    "value" << (Chunk::MaxChunkSize / ( 1024 * 1024 ) ) ) );
+        }
+
+        conn.done();
+    }
+
     string ConfigServer::getHost( string name , bool withPort ){
         if ( name.find( ":" ) ){
             if ( withPort )
