@@ -58,7 +58,7 @@ namespace mongo {
             return (unsigned long long)e.number();
         
         if ( e.type() == Date || e.type() == Timestamp )
-            return e.date();
+            return e._numberLong();
 
         
         errmsg = "version is not a numberic type";
@@ -79,6 +79,7 @@ namespace mongo {
     
     class WriteBackCommand : public MongodShardCommand {
     public:
+        virtual LockType locktype(){ return NONE; } 
         WriteBackCommand() : MongodShardCommand( "writebacklisten" ){}
         bool run(const char *cmdns, BSONObj& cmdObj, string& errmsg, BSONObjBuilder& result, bool){
 
@@ -90,8 +91,6 @@ namespace mongo {
             
             const OID id = e.__oid();
             
-            dbtemprelease unlock;
-            
             if ( ! clientQueues[id.str()] )
                 clientQueues[id.str()] = new BlockingQueue<BSONObj>();
 
@@ -114,6 +113,8 @@ namespace mongo {
             help << " example: { setShardVersion : 'alleyinsider.foo' , version : 1 , configdb : '' } ";
         }
         
+        virtual LockType locktype(){ return WRITE; } // TODO: figure out how to make this not need to lock
+ 
         bool run(const char *cmdns, BSONObj& cmdObj, string& errmsg, BSONObjBuilder& result, bool){
             
             bool authoritative = cmdObj.getBoolField( "authoritative" );
@@ -247,6 +248,8 @@ namespace mongo {
             help << " example: { getShardVersion : 'alleyinsider.foo'  } ";
         }
         
+        virtual LockType locktype(){ return WRITE; } // TODO: figure out how to make this not need to lock
+
         bool run(const char *cmdns, BSONObj& cmdObj, string& errmsg, BSONObjBuilder& result, bool){
             string ns = cmdObj["getShardVersion"].valuestrsafe();
             if ( ns.size() == 0 ){
@@ -273,6 +276,8 @@ namespace mongo {
         virtual void help( stringstream& help ) const {
             help << "should not be calling this directly" << endl;
         }
+
+        virtual LockType locktype(){ return WRITE; } 
         
         bool run(const char *cmdns, BSONObj& cmdObj, string& errmsg, BSONObjBuilder& result, bool){
             // so i have to start clone, tell caller its ok to make change
@@ -342,6 +347,8 @@ namespace mongo {
         virtual void help( stringstream& help ) const {
             help << "should not be calling this directly" << endl;
         }
+
+        virtual LockType locktype(){ return WRITE; } 
         
         bool run(const char *cmdns, BSONObj& cmdObj, string& errmsg, BSONObjBuilder& result, bool){
             // see MoveShardStartCommand::run
@@ -481,6 +488,11 @@ namespace mongo {
 
 
     bool handlePossibleShardedMessage( Message &m, DbResponse &dbresponse ){
+
+        if ( shardConfigServer.empty() ){
+            return false;
+        }
+
         int op = m.data->operation();
         if ( op < 2000 || op >= 3000 )
             return false;
@@ -503,7 +515,7 @@ namespace mongo {
             }
             
             QueryResult *qr = (QueryResult*)b.buf();
-            qr->resultFlags() = QueryResult::ResultFlag_ErrSet | QueryResult::ResultFlag_ShardConfigStale;
+            qr->_resultFlags() = QueryResult::ResultFlag_ErrSet | QueryResult::ResultFlag_ShardConfigStale;
             qr->len = b.len();
             qr->setOperation( opReply );
             qr->cursorId = 0;
@@ -520,7 +532,7 @@ namespace mongo {
         }
         
         OID * clientID = clientServerIds.get();
-        massert( "write with bad shard config and no server id!" , clientID );
+        massert( 10422 ,  "write with bad shard config and no server id!" , clientID );
         
         log() << "got write with an old config - writing back" << endl;
 
