@@ -26,6 +26,17 @@
 #define assert xassert
 #endif
 
+#define smuassert( cx , msg , val ) \
+  if ( ! ( val ) ){ \
+    JS_ReportError( cx , msg ); \
+    return JS_FALSE; \
+  }
+
+#define CHECKNEWOBJECT(xx,ctx,w)                                   \
+    if ( ! xx ){                                                   \
+        massert(13072,(string)"JS_NewObject failed: " + w ,xx);    \
+    }
+
 namespace mongo {
 
     string trim( string s ){
@@ -39,8 +50,8 @@ namespace mongo {
     }
 
     boost::thread_specific_ptr<SMScope> currentScope( dontDeleteScope );
-    boost::recursive_mutex smmutex;
-#define smlock recursive_boostlock ___lk( smmutex );
+    boost::recursive_mutex &smmutex = *( new boost::recursive_mutex );
+#define smlock recursive_scoped_lock ___lk( smmutex );
 
 #define GETHOLDER(x,o) ((BSONHolder*)JS_GetPrivate( x , o ))
 
@@ -61,7 +72,7 @@ namespace mongo {
         }
 
         void check(){
-            uassert( "holder magic value is wrong" , _magic == 17 && _obj.isValid() );
+            uassert( 10212 ,  "holder magic value is wrong" , _magic == 17 && _obj.isValid() );
         }
 
         BSONFieldIterator * it();
@@ -144,7 +155,7 @@ namespace mongo {
             free( dst );
             if ( !JS_CStringsAreUTF8() )
                 for( string::const_iterator i = ss.begin(); i != ss.end(); ++i )
-                    uassert( "non ascii character detected", (unsigned char)(*i) <= 127 );
+                    uassert( 10213 ,  "non ascii character detected", (unsigned char)(*i) <= 127 );
             return ss;
         }
 
@@ -152,9 +163,22 @@ namespace mongo {
             return toString( JS_ValueToString( _context , v ) );
         }
 
+        // NOTE No validation of passed in object
+        long long toNumberLongUnsafe( JSObject *o ) {
+            boost::uint64_t val;
+            if ( hasProperty( o, "top" ) ) {
+                val =
+                ( (boost::uint64_t)(boost::uint32_t)getNumber( o , "top" ) << 32 ) +
+                ( boost::uint32_t)( getNumber( o , "bottom" ) );
+            } else {
+                val = (boost::uint64_t) getNumber( o, "floatApprox" );
+            }
+            return val;
+        }
+        
         double toNumber( jsval v ){
             double d;
-            uassert( "not a number" , JS_ValueToNumber( _context , v , &d ) );
+            uassert( 10214 ,  "not a number" , JS_ValueToNumber( _context , v , &d ) );
             return d;
         }
 
@@ -174,7 +198,7 @@ namespace mongo {
             return oid;
         }
 
-        BSONObj toObject( JSObject * o ){
+        BSONObj toObject( JSObject * o , int depth = 0){
             if ( ! o )
                 return BSONObj();
 
@@ -198,9 +222,11 @@ namespace mongo {
 
             if ( ! appendSpecialDBObject( this , b , "value" , OBJECT_TO_JSVAL( o ) , o ) ){
 
-                jsval theid = getProperty( o , "_id" );
-                if ( ! JSVAL_IS_VOID( theid ) ){
-                    append( b , "_id" , theid );
+                if ( depth == 0 ){
+                    jsval theid = getProperty( o , "_id" );
+                    if ( ! JSVAL_IS_VOID( theid ) ){
+                        append( b , "_id" , theid , EOO , depth + 1 );
+                    }
                 }
                 
                 JSIdArray * properties = JS_Enumerate( _context , o );
@@ -211,10 +237,10 @@ namespace mongo {
                     jsval nameval;
                     assert( JS_IdToValue( _context ,id , &nameval ) );
                     string name = toString( nameval );
-                    if ( name == "_id" )
+                    if ( depth == 0 && name == "_id" )
                         continue;
                     
-                    append( b , name , getProperty( o , name.c_str() ) , orig[name].type() );
+                    append( b , name , getProperty( o , name.c_str() ) , orig[name].type() , depth + 1 );
                 }
 
                 JS_DestroyIdArray( _context , properties );
@@ -228,7 +254,7 @@ namespace mongo {
                  JSVAL_IS_VOID( v ) )
                 return BSONObj();
 
-            uassert( "not an object" , JSVAL_IS_OBJECT( v ) );
+            uassert( 10215 ,  "not an object" , JSVAL_IS_OBJECT( v ) );
             return toObject( JSVAL_TO_OBJECT( v ) );
         }
 
@@ -237,7 +263,7 @@ namespace mongo {
         }
 
         string getFunctionCode( jsval v ){
-            uassert( "not a function" , JS_TypeOfValue( _context , v ) == JSTYPE_FUNCTION );
+            uassert( 10216 ,  "not a function" , JS_TypeOfValue( _context , v ) == JSTYPE_FUNCTION );
             return getFunctionCode( JS_ValueToFunction( _context , v ) );
         }
         
@@ -248,7 +274,7 @@ namespace mongo {
             b.appendRegex( name.c_str() , s.substr( 0 , end ).c_str() , s.substr( end + 1 ).c_str() );
         }
 
-        void append( BSONObjBuilder& b , string name , jsval val , BSONType oldType = EOO  ){
+        void append( BSONObjBuilder& b , string name , jsval val , BSONType oldType = EOO , int depth=0 ){
             //cout << "name: " << name << "\t" << typeString( val ) << " oldType: " << oldType << endl;
             switch ( JS_TypeOfValue( _context , val ) ){
 
@@ -272,7 +298,7 @@ namespace mongo {
                     b.appendNull( name.c_str() );
                 }
                 else if ( ! appendSpecialDBObject( this , b , name , val , o ) ){
-                    BSONObj sub = toObject( o );
+                    BSONObj sub = toObject( o , depth );
                     if ( JS_IsArrayObject( _context , o ) ){
                         b.appendArray( name.c_str() , sub );
                     }
@@ -294,7 +320,7 @@ namespace mongo {
                 break;
             }
 
-            default: uassert( (string)"can't append field.  name:" + name + " type: " + typeString( val ) , 0 );
+            default: uassert( 10217 ,  (string)"can't append field.  name:" + name + " type: " + typeString( val ) , 0 );
             }
         }
 
@@ -383,12 +409,12 @@ namespace mongo {
                 paramString = trim( paramString );
             }
             
-            const char ** paramArray = new const char*[params.size()];
+            boost::scoped_array<const char *> paramArray (new const char*[params.size()]);
             for ( size_t i=0; i<params.size(); i++ )
                 paramArray[i] = params[i].c_str();
             
-            JSFunction * func = JS_CompileFunction( _context , assoc , fname.str().c_str() , params.size() , paramArray , code.c_str() , strlen( code.c_str() ) , "nofile_b" , 0 );
-            delete paramArray;
+            JSFunction * func = JS_CompileFunction( _context , assoc , fname.str().c_str() , params.size() , paramArray.get() , code.c_str() , strlen( code.c_str() ) , "nofile_b" , 0 );
+
             if ( ! func ){
                 cout << "compile failed for: " << raw << endl;
                 return 0;
@@ -427,7 +453,7 @@ namespace mongo {
                 jsval v;
                 if ( JS_GetPendingException( _context , &v ) )
                     cout << "\t why: " << toString( v ) << endl;
-                throw UserException( "invalid utf8" );
+                throw UserException( 9006 , "invalid utf8" );
             }
 
             assert( s );
@@ -438,13 +464,12 @@ namespace mongo {
             static string ref = "$ref";
             if ( ref == obj->firstElement().fieldName() ){
                 JSObject * o = JS_NewObject( _context , &dbref_class , NULL, NULL);
-                assert( o );
-                setProperty( o , "$ref" , toval( obj->firstElement() ) );
-                setProperty( o , "$id" , toval( (*obj)["$id"] ) );
+                CHECKNEWOBJECT(o,_context,"toJSObject1");
+                assert( JS_SetPrivate( _context , o , (void*)(new BSONHolder( obj->getOwned() ) ) ) );
                 return o;
             }
             JSObject * o = JS_NewObject( _context , readOnly ? &bson_ro_class : &bson_class , NULL, NULL);
-            assert( o );
+            CHECKNEWOBJECT(o,_context,"toJSObject2");
             assert( JS_SetPrivate( _context , o , (void*)(new BSONHolder( obj->getOwned() ) ) ) );
             return o;
         }
@@ -463,7 +488,6 @@ namespace mongo {
                 return JSVAL_NULL;
             case NumberDouble:
             case NumberInt:
-            case NumberLong:
                 return toval( e.number() );
             case Symbol: // TODO: should we make a special class for this
             case String:
@@ -499,6 +523,7 @@ namespace mongo {
             case jstOID:{
                 OID oid = e.__oid();
                 JSObject * o = JS_NewObject( _context , &object_id_class , 0 , 0 );
+                CHECKNEWOBJECT(o,_context,"jstOID");
                 setProperty( o , "str" , toval( oid.str().c_str() ) );
                 return OBJECT_TO_JSVAL( o );
             }
@@ -537,7 +562,7 @@ namespace mongo {
                 return OBJECT_TO_JSVAL( JS_GetFunctionObject( func ) );
             }
             case Date:
-                return OBJECT_TO_JSVAL( js_NewDateObjectMsec( _context , (jsdouble) e.date() ) );
+                return OBJECT_TO_JSVAL( js_NewDateObjectMsec( _context , (jsdouble) e.date().millis ) );
 
             case MinKey:
                 return OBJECT_TO_JSVAL( JS_NewObject( _context , &minkey_class , 0 , 0 ) );
@@ -547,16 +572,31 @@ namespace mongo {
 
             case Timestamp: {
                 JSObject * o = JS_NewObject( _context , &timestamp_class , 0 , 0 );
+                CHECKNEWOBJECT(o,_context,"Timestamp1");
                 setProperty( o , "t" , toval( (double)(e.timestampTime()) ) );
                 setProperty( o , "i" , toval( (double)(e.timestampInc()) ) );
                 return OBJECT_TO_JSVAL( o );
             }
-
+            case NumberLong: {
+                boost::uint64_t val = (boost::uint64_t)e.numberLong();
+                JSObject * o = JS_NewObject( _context , &numberlong_class , 0 , 0 );
+                CHECKNEWOBJECT(o,_context,"NumberLong1");
+                setProperty( o , "floatApprox" , toval( (double)(boost::int64_t)( val ) ) );                    
+                if ( (boost::int64_t)val != (boost::int64_t)(double)(boost::int64_t)( val ) ) {
+                    // using 2 doubles here instead of a single double because certain double
+                    // bit patterns represent undefined values and sm might trash them
+                    setProperty( o , "top" , toval( (double)(boost::uint32_t)( val >> 32 ) ) );
+                    setProperty( o , "bottom" , toval( (double)(boost::uint32_t)( val & 0x00000000ffffffff ) ) );
+                }
+                return OBJECT_TO_JSVAL( o );                
+            }
             case DBRef: {
                 JSObject * o = JS_NewObject( _context , &dbpointer_class , 0 , 0 );
+                CHECKNEWOBJECT(o,_context,"DBRef1");
                 setProperty( o , "ns" , toval( e.dbrefNS() ) );
 
                 JSObject * oid = JS_NewObject( _context , &object_id_class , 0 , 0 );
+                CHECKNEWOBJECT(oid,_context,"DBRef2");
                 setProperty( oid , "str" , toval( e.dbrefOID().str().c_str() ) );
 
                 setProperty( o , "id" , OBJECT_TO_JSVAL( oid ) );
@@ -564,9 +604,10 @@ namespace mongo {
             }
             case BinData:{
                 JSObject * o = JS_NewObject( _context , &bindata_class , 0 , 0 );
+                CHECKNEWOBJECT(o,_context,"Bindata_BinData1");
                 int len;
-                void * data = (void*)e.binData( len );
-                assert( JS_SetPrivate( _context , o , data ) );
+                const char * data = e.binData( len );
+                assert( JS_SetPrivate( _context , o , new BinDataHolder( data ) ) );
 
                 setProperty( o , "len" , toval( len ) );
                 setProperty( o , "type" , toval( (int)e.binDataType() ) );
@@ -575,7 +616,7 @@ namespace mongo {
             }
 
             cout << "toval: unknown type: " << e.type() << endl;
-            uassert( "not done: toval" , 0 );
+            uassert( 10218 ,  "not done: toval" , 0 );
             return 0;
         }
 
@@ -602,7 +643,7 @@ namespace mongo {
         }
 
         jsval getProperty( JSObject * o , const char * field ){
-            uassert( "object passed to getPropery is null" , o );
+            uassert( 10219 ,  "object passed to getPropery is null" , o );
             jsval v;
             assert( JS_GetProperty( _context , o , field , &v ) );
             return v;
@@ -693,7 +734,7 @@ namespace mongo {
             return JS_TRUE;
         }
 
-        uassert( "don't know what to do with this op" , 0 );
+        uassert( 10220 ,  "don't know what to do with this op" , 0 );
         return JS_FALSE;
     }
 
@@ -716,17 +757,6 @@ namespace mongo {
         JSCLASS_NO_OPTIONAL_MEMBERS
     };
 
-    JSBool bson_get_size(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval){
-        BSONHolder * o = GETHOLDER( cx , obj );
-        double size = 0;
-        if ( o ){
-            size = o->_obj.objsize();
-        }
-        Convertor c(cx);
-        *rval = c.toval( size );
-        return JS_TRUE;
-    }
-    
     JSBool bson_cons( JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval ){
         cerr << "bson_cons : shouldn't be here!" << endl;
         JS_ReportError( cx , "can't construct bson object" );
@@ -734,7 +764,6 @@ namespace mongo {
     }
 
     JSFunctionSpec bson_functions[] = {
-        { "bsonsize" , bson_get_size , 0 , 0 , JSPROP_READONLY | JSPROP_PERMANENT } , 
         { 0 }
     };
     
@@ -759,6 +788,8 @@ namespace mongo {
     JSBool mark_modified( JSContext *cx, JSObject *obj, jsval idval, jsval *vp){
         Convertor c(cx);
         BSONHolder * holder = GETHOLDER( cx , obj );
+        if ( !holder ) // needed when we're messing with DBRef.prototype
+            return JS_TRUE;
         if ( holder->_inResolve )
             return JS_TRUE;
         holder->_modified = true;
@@ -850,12 +881,35 @@ namespace mongo {
         { "nativeHelper" , &native_helper , 1 , 0 , 0 } ,
         { "load" , &native_load , 1 , 0 , 0 } ,
         { "gc" , &native_gc , 1 , 0 , 0 } ,
-
         { 0 , 0 , 0 , 0 , 0 }
     };
 
     // ----END global helpers ----
 
+    // Object helpers
+    
+    JSBool bson_get_size(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval){
+        if ( argc != 1 || !JSVAL_IS_OBJECT( argv[ 0 ] ) ) {
+            JS_ReportError( cx , "bsonsize requires one valid object" );
+            return JS_FALSE;
+        }
+        
+        BSONHolder * o = GETHOLDER( cx , JSVAL_TO_OBJECT( argv[ 0 ] ) );
+        double size = 0;
+        if ( o ){
+            size = o->_obj.objsize();
+        }
+        Convertor c(cx);
+        *rval = c.toval( size );
+        return JS_TRUE;
+    }
+    
+    JSFunctionSpec objectHelpers[] = {
+    { "bsonsize" , &bson_get_size , 1 , 0 , 0 } ,
+    { 0 , 0 , 0 , 0 , 0 }
+    };
+    
+    // end Object helpers
 
     JSBool resolveBSONField( JSContext *cx, JSObject *obj, jsval id, uintN flags, JSObject **objp ){
         assert( JS_EnterLocalRootScope( cx ) );
@@ -913,7 +967,7 @@ namespace mongo {
 #endif
 
             _runtime = JS_NewRuntime(8L * 1024L * 1024L);
-            uassert( "JS_NewRuntime failed" , _runtime );
+            uassert( 10221 ,  "JS_NewRuntime failed" , _runtime );
             
             if ( ! utf8Ok() ){
                 log() << "*** warning: spider monkey build without utf8 support.  consider rebuilding with utf8 support" << endl;
@@ -921,7 +975,7 @@ namespace mongo {
 
             int x = 0;
             assert( x = 1 );
-            uassert( "assert not being executed" , x == 1 );
+            uassert( 10222 ,  "assert not being executed" , x == 1 );
         }
 
         ~SMEngine(){
@@ -955,29 +1009,6 @@ namespace mongo {
     }
 
 
-    // ------ special helpers -------
-
-    JSBool object_keyset(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval){
-
-        JSIdArray * properties = JS_Enumerate( cx , obj );
-        assert( properties );
-
-        JSObject * array = JS_NewArrayObject( cx , properties->length , 0 );
-        assert( array );
-
-        for ( jsint i=0; i<properties->length; i++ ){
-            jsid id = properties->vector[i];
-            jsval idval;
-            assert( JS_IdToValue( cx , id , &idval ) );
-            assert( JS_SetElement( cx , array , i ,  &idval ) );
-        }
-
-        JS_DestroyIdArray( cx , properties );
-
-        *rval = OBJECT_TO_JSVAL( array );
-        return JS_TRUE;
-    }
-
     // ------ scope ------
 
 
@@ -995,25 +1026,22 @@ namespace mongo {
             smlock;
             _context = JS_NewContext( globalSMEngine->_runtime , 8192 );
             _convertor = new Convertor( _context );
-            massert( "JS_NewContext failed" , _context );
+            massert( 10431 ,  "JS_NewContext failed" , _context );
 
             JS_SetOptions( _context , JSOPTION_VAROBJFIX);
             //JS_SetVersion( _context , JSVERSION_LATEST); TODO
             JS_SetErrorReporter( _context , errorReporter );
 
             _global = JS_NewObject( _context , &global_class, NULL, NULL);
-            massert( "JS_NewObject failed for global" , _global );
+            massert( 10432 ,  "JS_NewObject failed for global" , _global );
             JS_SetGlobalObject( _context , _global );
-            massert( "js init failed" , JS_InitStandardClasses( _context , _global ) );
+            massert( 10433 ,  "js init failed" , JS_InitStandardClasses( _context , _global ) );
 
             JS_SetOptions( _context , JS_GetOptions( _context ) | JSOPTION_VAROBJFIX );
 
             JS_DefineFunctions( _context , _global , globalHelpers );
-
-            // install my special helpers
-
-            assert( JS_DefineFunction( _context , _convertor->getGlobalPrototype( "Object" ) ,
-                                       "keySet" , object_keyset , 0 , JSPROP_READONLY ) );
+            
+            JS_DefineFunctions( _context , _convertor->getGlobalObject( "Object" ), objectHelpers );
 
             //JS_SetGCCallback( _context , no_gc ); // this is useful for seeing if something is a gc problem
 
@@ -1022,7 +1050,7 @@ namespace mongo {
         
         ~SMScope(){
             smlock;
-            uassert( "deleted SMScope twice?" , _convertor );
+            uassert( 10223 ,  "deleted SMScope twice?" , _convertor );
 
             for ( list<void*>::iterator i=_roots.begin(); i != _roots.end(); i++ ){
                 JS_RemoveRoot( _context , *i );
@@ -1079,7 +1107,7 @@ namespace mongo {
 
         void externalSetup(){
             smlock;
-            uassert( "already local connected" , ! _localConnect );
+            uassert( 10224 ,  "already local connected" , ! _localConnect );
             if ( _externalSetup )
                 return;
             initMongoJS( this , _context , _global , false );
@@ -1088,9 +1116,9 @@ namespace mongo {
 
         void localConnect( const char * dbName ){
             smlock;
-            uassert( "already setup for external db" , ! _externalSetup );
+            uassert( 10225 ,  "already setup for external db" , ! _externalSetup );
             if ( _localConnect ){
-                uassert( "connected to different db" , _localDBName == dbName );
+                uassert( 10226 ,  "connected to different db" , _localDBName == dbName );
                 return;
             }
             
@@ -1158,7 +1186,7 @@ namespace mongo {
             case JSTYPE_NUMBER: return NumberDouble;
             case JSTYPE_BOOLEAN: return Bool;
             default:
-                uassert( "unknown type" , 0 );
+                uassert( 10227 ,  "unknown type" , 0 );
             }
             return 0;
         }
@@ -1282,7 +1310,7 @@ namespace mongo {
             uninstallCheckTimeout( timeoutMs );
 
             if ( assertOnError )
-                uassert( name + " exec failed" , worked );
+                uassert( 10228 ,  name + " exec failed" , worked );
 
             if ( reportError && ! _error.empty() ){
                 // cout << "exec error: " << _error << endl;
@@ -1305,12 +1333,11 @@ namespace mongo {
             assert( JS_EnterLocalRootScope( _context ) );
                 
             int nargs = args.nFields();
-            auto_ptr<jsval> smargsPtr( new jsval[nargs] );
-            jsval* smargs = smargsPtr.get();
+            scoped_array<jsval> smargsPtr( new jsval[nargs] );
             if ( nargs ){
                 BSONObjIterator it( args );
                 for ( int i=0; i<nargs; i++ ){
-                    smargs[i] = _convertor->toval( it.next() );
+                    smargsPtr[i] = _convertor->toval( it.next() );
                 }
             }
 
@@ -1325,7 +1352,7 @@ namespace mongo {
 
             installCheckTimeout( timeoutMs );
             jsval rval;
-            JSBool ret = JS_CallFunction( _context , _this ? _this : _global , func , nargs , smargs , &rval );
+            JSBool ret = JS_CallFunction( _context , _this ? _this : _global , func , nargs , smargsPtr.get() , &rval );
             uninstallCheckTimeout( timeoutMs );
 
             if ( !ret ) {
@@ -1364,6 +1391,7 @@ namespace mongo {
         }
 
         virtual void gc(){
+            smlock;
             JS_GC( _context );
         }
 
@@ -1400,7 +1428,7 @@ namespace mongo {
         stringstream ss;
         ss << "JS Error: " << message;
 
-        if ( report ){
+        if ( report && report->filename ){
             ss << " " << report->filename << ":" << report->lineno;
         }
 
@@ -1470,7 +1498,7 @@ namespace mongo {
             return;
 
         SMScope * scope = currentScope.get();
-        uassert( "need a scope" , scope );
+        uassert( 10229 ,  "need a scope" , scope );
         
         JSObject * o = JS_GetFunctionObject( f );
         assert( o );
