@@ -17,6 +17,9 @@
 
 // hacked in right now from engine_spidermonkey.cpp
 
+#include "../client/syncclusterconnection.h"
+#include "../util/base64.h"
+
 namespace mongo {
 
     bool haveLocalShardingInfo( const string& ns );
@@ -50,25 +53,25 @@ namespace mongo {
 
     class CursorHolder {
     public:
-        CursorHolder( auto_ptr< DBClientCursor > &cursor, const shared_ptr< DBClientBase > &connection ) :
+        CursorHolder( auto_ptr< DBClientCursor > &cursor, const shared_ptr< DBClientWithCommands > &connection ) :
         connection_( connection ),
         cursor_( cursor ) {
             assert( cursor_.get() );
         }
         DBClientCursor *get() const { return cursor_.get(); }
     private:
-        shared_ptr< DBClientBase > connection_;
+        shared_ptr< DBClientWithCommands > connection_;
         auto_ptr< DBClientCursor > cursor_;
     };
     
     DBClientCursor *getCursor( JSContext *cx, JSObject *obj ) {
         CursorHolder * holder = (CursorHolder*)JS_GetPrivate( cx , obj );
-        uassert( "no cursor!" , holder );
+        uassert( 10235 ,  "no cursor!" , holder );
         return holder->get();
     }
     
     JSBool internal_cursor_constructor( JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval ){
-        uassert( "no args to internal_cursor_constructor" , argc == 0 );
+        uassert( 10236 ,  "no args to internal_cursor_constructor" , argc == 0 );
         assert( JS_SetPrivate( cx , obj , 0 ) ); // just for safety
         return JS_TRUE;
     }
@@ -99,11 +102,10 @@ namespace mongo {
         *rval = c.toval( &n );
         return JS_TRUE;
     }
-    
 
     JSFunctionSpec internal_cursor_functions[] = {
-        { "hasNext" , internal_cursor_hasNext , 0 , 0 , JSPROP_READONLY | JSPROP_PERMANENT } ,
-        { "next" , internal_cursor_next , 0 , 0 , JSPROP_READONLY | JSPROP_PERMANENT } ,
+        { "hasNext" , internal_cursor_hasNext , 0 , JSPROP_READONLY | JSPROP_PERMANENT, 0 } ,
+        { "next" , internal_cursor_next , 0 , JSPROP_READONLY | JSPROP_PERMANENT, 0 } ,
         { 0 }
     };
 
@@ -118,15 +120,15 @@ namespace mongo {
     // ------ mongo stuff ------
 
     JSBool mongo_constructor( JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval ){
-        uassert( "mongo_constructor not implemented yet" , 0 );
+        uassert( 10237 ,  "mongo_constructor not implemented yet" , 0 );
         throw -1;
     }
     
     JSBool mongo_local_constructor( JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval ){
         Convertor c( cx );
 
-        shared_ptr< DBClientBase > client( createDirectClient() );
-        assert( JS_SetPrivate( cx , obj , (void*)( new shared_ptr< DBClientBase >( client ) ) ) );
+        shared_ptr< DBClientWithCommands > client( createDirectClient() );
+        assert( JS_SetPrivate( cx , obj , (void*)( new shared_ptr< DBClientWithCommands >( client ) ) ) );
 
         jsval host = c.toval( "EMBEDDED" );
         assert( JS_SetProperty( cx , obj , "host" , &host ) );
@@ -137,48 +139,58 @@ namespace mongo {
     JSBool mongo_external_constructor( JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval ){
         Convertor c( cx );
         
-        uassert( "0 or 1 args to Mongo" , argc <= 1 );
+        uassert( 10238 ,  "0 or 1 args to Mongo" , argc <= 1 );
         
         string host = "127.0.0.1";
         if ( argc > 0 )
             host = c.toString( argv[0] );
+        
+        int numCommas = DBClientBase::countCommas( host );
 
-        shared_ptr< DBClientBase > conn;
+        shared_ptr< DBClientWithCommands > conn;
         
         string errmsg;
-        if ( host.find( "," ) == string::npos ){
+        if ( numCommas == 0 ){
             DBClientConnection * c = new DBClientConnection( true );
             conn.reset( c );
             if ( ! c->connect( host , errmsg ) ){
                 JS_ReportError( cx , ((string)"couldn't connect: " + errmsg).c_str() );
                 return JS_FALSE;
             }
+            ScriptEngine::runConnectCallback( *c );
         }
-        else { // paired
+        else if ( numCommas == 1 ){ // paired
             DBClientPaired * c = new DBClientPaired();
             conn.reset( c );
             if ( ! c->connect( host ) ){
                 JS_ReportError( cx , "couldn't connect to pair" );
-                return JS_FALSE;
+                    return JS_FALSE;
             }
         }
+        else if ( numCommas == 2 ){
+            conn.reset( new SyncClusterConnection( host ) );
+        }
+        else {
+            JS_ReportError( cx , "1 (paired) or 2(quorum) commas are allowed" );
+            return JS_FALSE;
+        }
         
         
-        assert( JS_SetPrivate( cx , obj , (void*)( new shared_ptr< DBClientBase >( conn ) ) ) );
+        assert( JS_SetPrivate( cx , obj , (void*)( new shared_ptr< DBClientWithCommands >( conn ) ) ) );
         jsval host_val = c.toval( host.c_str() );
         assert( JS_SetProperty( cx , obj , "host" , &host_val ) );
         return JS_TRUE;
 
     }
 
-    DBClientBase *getConnection( JSContext *cx, JSObject *obj ) {
-        shared_ptr< DBClientBase > * connHolder = (shared_ptr< DBClientBase >*)JS_GetPrivate( cx , obj );
-        uassert( "no connection!" , connHolder && connHolder->get() );
+    DBClientWithCommands *getConnection( JSContext *cx, JSObject *obj ) {
+        shared_ptr< DBClientWithCommands > * connHolder = (shared_ptr< DBClientWithCommands >*)JS_GetPrivate( cx , obj );
+        uassert( 10239 ,  "no connection!" , connHolder && connHolder->get() );
         return connHolder->get();
     }
     
     void mongo_finalize( JSContext * cx , JSObject * obj ){
-        shared_ptr< DBClientBase > * connHolder = (shared_ptr< DBClientBase >*)JS_GetPrivate( cx , obj );
+        shared_ptr< DBClientWithCommands > * connHolder = (shared_ptr< DBClientWithCommands >*)JS_GetPrivate( cx , obj );
         if ( connHolder ){
             delete connHolder;
             assert( JS_SetPrivate( cx , obj , 0 ) );
@@ -193,10 +205,10 @@ namespace mongo {
      };
 
     JSBool mongo_find(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval){
-        uassert( "mongo_find neesd 5 args" , argc == 5 );
-        shared_ptr< DBClientBase > * connHolder = (shared_ptr< DBClientBase >*)JS_GetPrivate( cx , obj );
-        uassert( "no connection!" , connHolder && connHolder->get() );
-        DBClientBase *conn = connHolder->get();
+        uassert( 10240 ,  "mongo_find neesd 6 args" , argc == 6 );
+        shared_ptr< DBClientWithCommands > * connHolder = (shared_ptr< DBClientWithCommands >*)JS_GetPrivate( cx , obj );
+        uassert( 10241 ,  "no connection!" , connHolder && connHolder->get() );
+        DBClientWithCommands *conn = connHolder->get();
                       
         Convertor c( cx );
 
@@ -208,15 +220,18 @@ namespace mongo {
         int nToReturn = (int) c.toNumber( argv[3] );
         int nToSkip = (int) c.toNumber( argv[4] );
         bool slaveOk = c.getBoolean( obj , "slaveOk" );
+        int batchSize = (int) c.toNumber( argv[5] );
 
         try {
 
-            auto_ptr<DBClientCursor> cursor = conn->query( ns , q , nToReturn , nToSkip , f.nFields() ? &f : 0  , slaveOk ? Option_SlaveOk : 0 );
+            auto_ptr<DBClientCursor> cursor = conn->query( ns , q , nToReturn , nToSkip , f.nFields() ? &f : 0  , slaveOk ? QueryOption_SlaveOk : 0 , batchSize );
             if ( ! cursor.get() ){
+                log() << "query failed : " << ns << " " << q << " to: " << conn->toString() << endl;
                 JS_ReportError( cx , "error doing query: failed" );
                 return JS_FALSE;
             }
             JSObject * mycursor = JS_NewObject( cx , &internal_cursor_class , 0 , 0 );
+            CHECKNEWOBJECT( mycursor, cx, "internal_cursor_class" );
             assert( JS_SetPrivate( cx , mycursor , new CursorHolder( cursor, *connHolder ) ) );
             *rval = OBJECT_TO_JSVAL( mycursor );
             return JS_TRUE;
@@ -228,14 +243,18 @@ namespace mongo {
     }
 
     JSBool mongo_update(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval){
-        uassert( "mongo_find needs at elast 3 args" , argc >= 3 );
-        uassert( "2nd param to update has to be an object" , JSVAL_IS_OBJECT( argv[1] ) );
-        uassert( "3rd param to update has to be an object" , JSVAL_IS_OBJECT( argv[2] ) );
-        
-        DBClientBase * conn = getConnection( cx, obj );
-        uassert( "no connection!" , conn );
+        smuassert( cx ,  "mongo_find needs at elast 3 args" , argc >= 3 );
+        smuassert( cx ,  "2nd param to update has to be an object" , JSVAL_IS_OBJECT( argv[1] ) );
+        smuassert( cx ,  "3rd param to update has to be an object" , JSVAL_IS_OBJECT( argv[2] ) );
 
         Convertor c( cx );
+        if ( c.getBoolean( obj , "readOnly" ) ){
+            JS_ReportError( cx , "js db in read only mode - mongo_update" );
+            return JS_FALSE;
+        }
+
+        DBClientWithCommands * conn = getConnection( cx, obj );
+        uassert( 10245 ,  "no connection!" , conn );
 
         string ns = c.toString( argv[0] );
 
@@ -253,13 +272,18 @@ namespace mongo {
     }
 
     JSBool mongo_insert(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval){    
-        uassert( "mongo_insert needs 2 args" , argc == 2 );
-        uassert( "2nd param to insert has to be an object" , JSVAL_IS_OBJECT( argv[1] ) );
-        
-        DBClientBase * conn = getConnection( cx, obj );
-        uassert( "no connection!" , conn );
-        
+        smuassert( cx ,  "mongo_insert needs 2 args" , argc == 2 );
+        smuassert( cx ,  "2nd param to insert has to be an object" , JSVAL_IS_OBJECT( argv[1] ) );
+
         Convertor c( cx );
+        if ( c.getBoolean( obj , "readOnly" ) ){
+            JS_ReportError( cx , "js db in read only mode - mongo_insert" );
+            return JS_FALSE;
+        }
+        
+        DBClientWithCommands * conn = getConnection( cx, obj );
+        uassert( 10248 ,  "no connection!" , conn );
+        
         
         string ns = c.toString( argv[0] );
         BSONObj o = c.toObject( argv[1] );
@@ -270,6 +294,13 @@ namespace mongo {
             conn->insert( ns , o );
             return JS_TRUE;
         }
+        catch ( std::exception& e ){
+            stringstream ss;
+            ss << "error doing insert:" << e.what();
+            string s = ss.str();
+            JS_ReportError( cx , s.c_str() );
+            return JS_FALSE;
+        }
         catch ( ... ){
             JS_ReportError( cx , "error doing insert" );
             return JS_FALSE;
@@ -277,13 +308,17 @@ namespace mongo {
     }
 
     JSBool mongo_remove(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval){    
-        uassert( "mongo_remove needs 2 arguments" , argc == 2 );
-        uassert( "2nd param to insert has to be an object" , JSVAL_IS_OBJECT( argv[1] ) );
-
-        DBClientBase * conn = getConnection( cx, obj );
-        uassert( "no connection!" , conn );
+        smuassert( cx ,  "mongo_remove needs 2 arguments" , argc == 2 );
+        smuassert( cx ,  "2nd param to insert has to be an object" , JSVAL_IS_OBJECT( argv[1] ) );
 
         Convertor c( cx );
+        if ( c.getBoolean( obj , "readOnly" ) ){
+            JS_ReportError( cx , "js db in read only mode - mongo_remove" );
+            return JS_FALSE;
+        }
+
+        DBClientWithCommands * conn = getConnection( cx, obj );
+        uassert( 10251 ,  "no connection!" , conn );
         
         string ns = c.toString( argv[0] );
         BSONObj o = c.toObject( argv[1] );
@@ -300,10 +335,10 @@ namespace mongo {
     }
 
     JSFunctionSpec mongo_functions[] = {
-        { "find" , mongo_find , 0 , 0 , JSPROP_READONLY | JSPROP_PERMANENT } ,
-        { "update" , mongo_update , 0 , 0 , JSPROP_READONLY | JSPROP_PERMANENT } ,
-        { "insert" , mongo_insert , 0 , 0 , JSPROP_READONLY | JSPROP_PERMANENT } ,
-        { "remove" , mongo_remove , 0 , 0 , JSPROP_READONLY | JSPROP_PERMANENT } ,
+        { "find" , mongo_find , 0 , JSPROP_READONLY | JSPROP_PERMANENT, 0 } ,
+        { "update" , mongo_update , 0 , JSPROP_READONLY | JSPROP_PERMANENT, 0 } ,
+        { "insert" , mongo_insert , 0 , JSPROP_READONLY | JSPROP_PERMANENT, 0 } ,
+        { "remove" , mongo_remove , 0 , JSPROP_READONLY | JSPROP_PERMANENT, 0 } ,
         { 0 }
     };
 
@@ -311,7 +346,7 @@ namespace mongo {
      // -------------  db_collection -------------
 
      JSBool db_collection_constructor( JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval ){    
-         uassert( "db_collection_constructor wrong args" , argc == 4 );
+         smuassert( cx ,  "db_collection_constructor wrong args" , argc == 4 );
          assert( JS_SetProperty( cx , obj , "_mongo" , &(argv[0]) ) );
          assert( JS_SetProperty( cx , obj , "_db" , &(argv[1]) ) );
          assert( JS_SetProperty( cx , obj , "_shortName" , &(argv[2]) ) );
@@ -374,6 +409,7 @@ namespace mongo {
          assert( c.hasProperty( db , "_name" ) );
          
          JSObject * coll = JS_NewObject( cx , &db_collection_class , 0 , 0 );
+         CHECKNEWOBJECT( coll, cx, "doCreateCollection" );
          c.setProperty( coll , "_mongo" , c.getProperty( db , "_mongo" ) );
          c.setProperty( coll , "_db" , OBJECT_TO_JSVAL( db ) );
          c.setProperty( coll , "_shortName" , c.toval( shortName.c_str() ) );
@@ -394,7 +430,7 @@ namespace mongo {
     
     
     JSBool db_constructor( JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval ){
-        uassert( "wrong number of arguments to DB" , argc == 2 );
+        smuassert( cx,  "wrong number of arguments to DB" , argc == 2 );
         assert( JS_SetProperty( cx , obj , "_mongo" , &(argv[0]) ) );
         assert( JS_SetProperty( cx , obj , "_name" , &(argv[1]) ) );
 
@@ -446,22 +482,14 @@ namespace mongo {
             oid.init();
         }
         else {
-            uassert( "object_id_constructor can't take more than 1 param" , argc == 1 );
+            smuassert( cx ,  "object_id_constructor can't take more than 1 param" , argc == 1 );
             string s = c.toString( argv[0] );
 
-            if ( s.size() != 24 ){
-                JS_ReportError( cx , "invalid object id: length" );
-                return JS_FALSE;
-            }
-
-            for ( string::size_type i=0; i<s.size(); i++ ){
-                char c = s[i];
-                if ( ( c >= '0' && c <= '9' ) ||
-                     ( c >= 'a' && c <= 'f' ) ||
-                     ( c >= 'A' && c <= 'F' ) ){
-                    continue;
-                }
-                JS_ReportError( cx , "invalid object id: not hex" );
+            try {
+                Scope::validateObjectIdString( s );
+            } catch ( const MsgAssertionException &m ) {
+                static string error = m.toString();
+                JS_ReportError( cx, error.c_str() );
                 return JS_FALSE;
             }
             oid.init( s );
@@ -469,7 +497,7 @@ namespace mongo {
         
         if ( ! JS_InstanceOf( cx , obj , &object_id_class , 0 ) ){
             obj = JS_NewObject( cx , &object_id_class , 0 , 0 );
-            assert( obj );
+            CHECKNEWOBJECT( obj, cx, "object_id_constructor" );
             *rval = OBJECT_TO_JSVAL( obj );
         }
         
@@ -492,10 +520,11 @@ namespace mongo {
     }
 
     JSFunctionSpec object_id_functions[] = {
-        { "toString" , object_id_tostring , 0 , 0 , JSPROP_READONLY | JSPROP_PERMANENT } ,
+        { "toString" , object_id_tostring , 0 , JSPROP_READONLY | JSPROP_PERMANENT, 0 } ,
         { 0 }
     };
 
+
     // dbpointer
 
     JSBool dbpointer_constructor( JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval ){
@@ -532,46 +561,82 @@ namespace mongo {
 
     JSBool dbref_constructor( JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval ){
         Convertor c( cx );
-                
+
         if ( argc == 2 ){
-            assert( JS_SetProperty( cx , obj , "$ref" , &(argv[0]) ) );
-            assert( JS_SetProperty( cx , obj , "$id" , &(argv[1]) ) );
+            JSObject * o = JS_NewObject( cx , NULL , NULL, NULL );
+            CHECKNEWOBJECT( o, cx, "dbref_constructor" );
+            assert( JS_SetProperty( cx, o , "$ref" , &argv[ 0 ] ) );
+            assert( JS_SetProperty( cx, o , "$id" , &argv[ 1 ] ) );
+            BSONObj bo = c.toObject( o );
+            assert( JS_SetPrivate( cx , obj , (void*)(new BSONHolder( bo.getOwned() ) ) ) );
             return JS_TRUE;
         }
         else {
             JS_ReportError( cx , "DBRef needs 2 arguments" );
+            assert( JS_SetPrivate( cx , obj , (void*)(new BSONHolder( BSONObj().getOwned() ) ) ) );
             return JS_FALSE;            
         }
     }
  
-    JSClass dbref_class = {
-        "DBRef" , JSCLASS_HAS_PRIVATE ,
-        JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
-        JS_EnumerateStub, JS_ResolveStub , JS_ConvertStub, JS_FinalizeStub,
-        JSCLASS_NO_OPTIONAL_MEMBERS
-    };
-
-    JSFunctionSpec dbref_functions[] = {
-        { 0 }
-    };
-
+    JSClass dbref_class = bson_class; // name will be fixed later
 
     // BinData
 
 
     JSBool bindata_constructor( JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval ){
-        JS_ReportError( cx , "can't create a BinData yet" );
-        return JS_FALSE;            
+        Convertor c( cx );
+        
+        if ( argc == 2 ){
+
+            int type = (int)c.toNumber( argv[ 0 ] );
+            string encoded = c.toString( argv[ 1 ] );
+            string decoded = base64::decode( encoded );
+
+            assert( JS_SetPrivate( cx, obj, new BinDataHolder( decoded.data(), decoded.length() ) ) );
+            c.setProperty( obj, "len", c.toval( decoded.length() ) );
+            c.setProperty( obj, "type", c.toval( type ) );
+
+            return JS_TRUE;
+        }
+        else {
+            JS_ReportError( cx , "BinData needs 2 arguments" );
+            return JS_FALSE;            
+        }
     }
  
+    JSBool bindata_tostring(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval){    
+        Convertor c(cx);
+        int type = (int)c.getNumber( obj , "type" );
+        int len = (int)c.getNumber( obj, "len" );
+        void *holder = JS_GetPrivate( cx, obj );
+        assert( holder );
+        const char *data = ( ( BinDataHolder* )( holder ) )->c_;
+        stringstream ss;
+        ss << "BinData( type: " << type << ", base64: \"";
+        base64::encode( ss, (const char *)data, len );
+        ss << "\" )";
+        string ret = ss.str();
+        return *rval = c.toval( ret.c_str() );
+    }
+
+    void bindata_finalize( JSContext * cx , JSObject * obj ){
+        Convertor c(cx);
+        void *holder = JS_GetPrivate( cx, obj );
+        if ( holder ){
+            delete ( BinDataHolder* )holder;
+            assert( JS_SetPrivate( cx , obj , 0 ) );
+        }
+    }    
+    
     JSClass bindata_class = {
         "BinData" , JSCLASS_HAS_PRIVATE ,
         JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
-        JS_EnumerateStub, JS_ResolveStub , JS_ConvertStub, JS_FinalizeStub,
+        JS_EnumerateStub, JS_ResolveStub , JS_ConvertStub, bindata_finalize,
         JSCLASS_NO_OPTIONAL_MEMBERS
     };
 
     JSFunctionSpec bindata_functions[] = {
+        { "toString" , bindata_tostring , 0 , JSPROP_READONLY | JSPROP_PERMANENT, 0 } ,
         { 0 }
     };
     
@@ -588,7 +653,7 @@ namespace mongo {
         }
 
         JSObject * array = JS_NewObject( cx , 0 , 0 , 0 );
-        assert( array );
+        CHECKNEWOBJECT( array, cx, "map_constructor" );
 
         jsval a = OBJECT_TO_JSVAL( array );
         JS_SetProperty( cx , obj , "_data" , &a );
@@ -626,7 +691,38 @@ namespace mongo {
         JS_EnumerateStub, JS_ResolveStub , JS_ConvertStub, JS_FinalizeStub,
         JSCLASS_NO_OPTIONAL_MEMBERS
     };
+    
+    JSClass numberlong_class = {
+        "NumberLong" , JSCLASS_HAS_PRIVATE ,
+        JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+        JS_EnumerateStub, JS_ResolveStub , JS_ConvertStub, JS_FinalizeStub,
+        JSCLASS_NO_OPTIONAL_MEMBERS
+    };
+    
+    JSBool numberlong_valueof(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval){    
+        Convertor c(cx);
+        return *rval = c.toval( double( c.toNumberLongUnsafe( obj ) ) );        
+    }
+    
+    JSBool numberlong_tonumber(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval){    
+        return numberlong_valueof( cx, obj, argc, argv, rval );
+    }
 
+    JSBool numberlong_tostring(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval){    
+        Convertor c(cx);
+        stringstream ss;
+        ss <<  c.toNumberLongUnsafe( obj );
+        string ret = ss.str();
+        return *rval = c.toval( ret.c_str() );
+    }
+
+    JSFunctionSpec numberlong_functions[] = {
+    { "valueOf" , numberlong_valueof , 0 , JSPROP_READONLY | JSPROP_PERMANENT, 0 } ,
+    { "toNumber" , numberlong_tonumber , 0 , JSPROP_READONLY | JSPROP_PERMANENT, 0 } ,
+    { "toString" , numberlong_tostring , 0 , JSPROP_READONLY | JSPROP_PERMANENT, 0 } ,
+    { 0 }
+    };    
+    
     JSClass minkey_class = {
         "MinKey" , JSCLASS_HAS_PRIVATE ,
         JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
@@ -644,7 +740,7 @@ namespace mongo {
     // dbquery
 
     JSBool dbquery_constructor( JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval ){
-        uassert( "DDQuery needs at least 4 args" , argc >= 4 );
+        smuassert( cx ,  "DDQuery needs at least 4 args" , argc >= 4 );
         
         Convertor c(cx);
         c.setProperty( obj , "_mongo" , argv[0] );
@@ -654,8 +750,11 @@ namespace mongo {
 
         if ( argc > 4 && JSVAL_IS_OBJECT( argv[4] ) )
             c.setProperty( obj , "_query" , argv[4] );
-        else 
-            c.setProperty( obj , "_query" , OBJECT_TO_JSVAL( JS_NewObject( cx , 0 , 0 , 0 ) ) );
+        else {
+            JSObject * temp = JS_NewObject( cx , 0 , 0 , 0 );
+            CHECKNEWOBJECT( temp, cx, "dbquery_constructor" );
+            c.setProperty( obj , "_query" , OBJECT_TO_JSVAL( temp ) );
+        }
         
         if ( argc > 5 && JSVAL_IS_OBJECT( argv[5] ) )
             c.setProperty( obj , "_fields" , argv[5] );
@@ -672,6 +771,11 @@ namespace mongo {
             c.setProperty( obj , "_skip" , argv[7] );
         else 
             c.setProperty( obj , "_skip" , JSVAL_ZERO );
+
+        if ( argc > 8 && JSVAL_IS_NUMBER( argv[8] ) )
+            c.setProperty( obj , "_batchSize" , argv[8] );
+        else 
+            c.setProperty( obj , "_batchSize" , JSVAL_ZERO );
         
         c.setProperty( obj , "_cursor" , JSVAL_NULL );
         c.setProperty( obj , "_numReturned" , JSVAL_ZERO );
@@ -714,10 +818,10 @@ namespace mongo {
         assert( JS_InitClass( cx , global , 0 , &internal_cursor_class , internal_cursor_constructor , 0 , 0 , internal_cursor_functions , 0 , 0 ) );
         assert( JS_InitClass( cx , global , 0 , &dbquery_class , dbquery_constructor , 0 , 0 , 0 , 0 , 0 ) );
         assert( JS_InitClass( cx , global , 0 , &dbpointer_class , dbpointer_constructor , 0 , 0 , dbpointer_functions , 0 , 0 ) );
-        assert( JS_InitClass( cx , global , 0 , &dbref_class , dbref_constructor , 0 , 0 , dbref_functions , 0 , 0 ) );
         assert( JS_InitClass( cx , global , 0 , &bindata_class , bindata_constructor , 0 , 0 , bindata_functions , 0 , 0 ) );
 
         assert( JS_InitClass( cx , global , 0 , &timestamp_class , 0 , 0 , 0 , 0 , 0 , 0 ) );
+        assert( JS_InitClass( cx , global , 0 , &numberlong_class , 0 , 0 , 0 , numberlong_functions , 0 , 0 ) );
         assert( JS_InitClass( cx , global , 0 , &minkey_class , 0 , 0 , 0 , 0 , 0 , 0 ) );
         assert( JS_InitClass( cx , global , 0 , &maxkey_class , 0 , 0 , 0 , 0 , 0 , 0 ) );
 
@@ -726,6 +830,10 @@ namespace mongo {
         assert( JS_InitClass( cx , global , 0 , &bson_ro_class , bson_cons , 0 , 0 , bson_functions , 0 , 0 ) );
         assert( JS_InitClass( cx , global , 0 , &bson_class , bson_cons , 0 , 0 , bson_functions , 0 , 0 ) );
         
+        static const char *dbrefName = "DBRef";
+        dbref_class.name = dbrefName;
+        assert( JS_InitClass( cx , global , 0 , &dbref_class , dbref_constructor , 2 , 0 , bson_functions , 0 , 0 ) );
+        
         scope->exec( jsconcatcode );
     }
 
@@ -753,15 +861,22 @@ namespace mongo {
             return true;
         }
 
+        if ( JS_InstanceOf( c->_context , o , &numberlong_class , 0 ) ){
+            b.append( name.c_str() , c->toNumberLongUnsafe( o ) );
+            return true;
+        }
+        
         if ( JS_InstanceOf( c->_context , o , &dbpointer_class , 0 ) ){
             b.appendDBRef( name.c_str() , c->getString( o , "ns" ).c_str() , c->toOID( c->getProperty( o , "id" ) ) );
             return true;
         }
         
         if ( JS_InstanceOf( c->_context , o , &bindata_class , 0 ) ){
+            void *holder = JS_GetPrivate( c->_context , o );
+            const char *data = ( ( BinDataHolder * )( holder ) )->c_;
             b.appendBinData( name.c_str() , 
                              (int)(c->getNumber( o , "len" )) , (BinDataType)((char)(c->getNumber( o , "type" ) ) ) , 
-                             (char*)JS_GetPrivate( c->_context , o ) + 1
+                             data
                              );
             return true;
         }
@@ -771,20 +886,21 @@ namespace mongo {
         {
             jsdouble d = js_DateGetMsecSinceEpoch( c->_context , o );
             if ( d ){
-                b.appendDate( name.c_str() , (unsigned long long)d );
+                b.appendDate( name.c_str() , Date_t(d) );
                 return true;
             }
         }
 #elif defined( XULRUNNER )
         if ( JS_InstanceOf( c->_context , o, globalSMEngine->_dateClass , 0 ) ){
             jsdouble d = js_DateGetMsecSinceEpoch( c->_context , o );
-            b.appendDate( name.c_str() , (unsigned long long)d );
+            b.appendDate( name.c_str() , Date_t(d) );
             return true;
         }
 #else
         if ( JS_InstanceOf( c->_context , o, &js_DateClass , 0 ) ){
             jsdouble d = js_DateGetMsecSinceEpoch( c->_context , o );
-            b.appendDate( name.c_str() , (unsigned long long)d );
+            //TODO: make signed
+            b.appendDate( name.c_str() , Date_t((unsigned long long)d) );
             return true;
         }
 #endif
