@@ -1,4 +1,22 @@
 // utils.cpp
+/*
+ *    Copyright 2010 10gen Inc.
+ *
+ *    Licensed under the Apache License, Version 2.0 (the "License");
+ *    you may not use this file except in compliance with the License.
+ *    You may obtain a copy of the License at
+ *
+ *    http://www.apache.org/licenses/LICENSE-2.0
+ *
+ *    Unless required by applicable law or agreed to in writing, software
+ *    distributed under the License is distributed on an "AS IS" BASIS,
+ *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ *    See the License for the specific language governing permissions and
+ *    limitations under the License.
+ */
+
+
+#include "../stdafx.h"
 
 #include <boost/thread/xtime.hpp>
 
@@ -10,21 +28,38 @@
 #include <map>
 #include <sstream>
 #include <vector>
+#include <fcntl.h>
 
-#ifndef _WIN32
-#include <sys/socket.h>
-#include <netinet/in.h>
+#ifdef _WIN32
+# include <io.h>
+# define SIGKILL 9
+#else
+# include <sys/socket.h>
+# include <netinet/in.h>
+# include <signal.h>
+# include <sys/stat.h>
+# include <sys/wait.h>
 #endif
 
 #include "../client/dbclient.h"
 #include "../util/processinfo.h"
-#include "../util/md5.hpp"
 #include "utils.h"
 
+extern const char * jsconcatcode_server;
+
 namespace mongo {
+#ifdef _WIN32
+    inline int close(int fd) { return _close(fd); }
+    inline int read(int fd, void* buf, size_t size) { return _read(fd, buf, size); }
+
+    inline int pipe(int fds[2]) { return _pipe(fds, 1024, _O_TEXT | _O_NOINHERIT); }
+#endif
 
     namespace shellUtils {
         
+        std::string _dbConnect;
+        std::string _dbAuth;
+                
         const char *argv0 = 0;
         void RecordMyLocation( const char *_argv0 ) { argv0 = _argv0; }
         
@@ -61,17 +96,50 @@ namespace mongo {
             assert( args.nFields() == 1 );
             assert( args.firstElement().isNumber() );
             int ms = int( args.firstElement().number() );
-            sleepms( ms );
+            {
+                auto_ptr< ScriptEngine::Unlocker > u = globalScriptEngine->newThreadUnlocker();
+                sleepms( ms );
+            }
             return undefined_;
         }
 
+        
+        BSONObj Quit(const BSONObj& args) {
+            // If not arguments are given first element will be EOO, which
+            // converts to the integer value 0.
+            int exit_code = int( args.firstElement().number() );
+            ::exit(exit_code);
+            return undefined_;
+        }
+
+        BSONObj JSGetMemInfo( const BSONObj& args ){
+            ProcessInfo pi;
+            uassert( 10258 ,  "processinfo not supported" , pi.supported() );
+            
+            BSONObjBuilder e;
+            e.append( "virtual" , pi.getVirtualMemorySize() );
+            e.append( "resident" , pi.getResidentSize() );
+            
+            BSONObjBuilder b;
+            b.append( "ret" , e.obj() );
+            
+            return b.obj();
+        }
+
+
+#ifndef MONGO_SAFE_SHELL
+
         BSONObj listFiles(const BSONObj& args){
-            uassert( "need to specify 1 argument to listFiles" , args.nFields() == 1 );
+            uassert( 10257 ,  "need to specify 1 argument to listFiles" , args.nFields() == 1 );
             
             BSONObjBuilder lst;
             
             string rootname = args.firstElement().valuestrsafe();
             path root( rootname );
+            stringstream ss;
+            ss << "listFiles: no such directory: " << rootname;
+            string msg = ss.str();
+            uassert( 12581, msg.c_str(), boost::filesystem::exists( root ) );
             
             directory_iterator end;
             directory_iterator i( root);
@@ -104,92 +172,34 @@ namespace mongo {
             ret.appendArray( "", lst.done() );
             return ret.obj();
         }
-        
-        BSONObj Quit(const BSONObj& args) {
-            // If not arguments are given first element will be EOO, which
-            // converts to the integer value 0.
-            int exit_code = int( args.firstElement().number() );
-            ::exit(exit_code);
-            return undefined_;
-        }
 
-        BSONObj JSGetMemInfo( const BSONObj& args ){
-            ProcessInfo pi;
-            uassert( "processinfo not supported" , pi.supported() );
-            
-            BSONObjBuilder e;
-            e.append( "virtual" , pi.getVirtualMemorySize() );
-            e.append( "resident" , pi.getResidentSize() );
-            
-            BSONObjBuilder b;
-            b.append( "ret" , e.obj() );
-            
-            return b.obj();
-        }
 
-        BSONObj JSVersion( const BSONObj& args ){
-            cout << "version: " << versionString << endl;
-            if ( strstr( versionString , "+" ) )
-                printGitVersion();
-            return BSONObj();
-        }
-        
-#ifndef _WIN32
-#include <signal.h>
-#include <fcntl.h>
-#include <sys/stat.h>
-#include <sys/wait.h>
-        
-        BSONObj AllocatePorts( const BSONObj &args ) {
-            uassert( "allocatePorts takes exactly 1 argument", args.nFields() == 1 );
-            uassert( "allocatePorts needs to be passed an integer", args.firstElement().isNumber() );
+        BSONObj removeFile(const BSONObj& args){
+            uassert( 12597 ,  "need to specify 1 argument to listFiles" , args.nFields() == 1 );
             
-            int n = int( args.firstElement().number() );
+            bool found = false;
             
-            vector< int > ports;
-            vector< int > sockets;
-            for( int i = 0; i < n; ++i ) {
-                int s = socket( AF_INET, SOCK_STREAM, 0 );
-                assert( s );
-                
-                sockaddr_in address;
-                memset(address.sin_zero, 0, sizeof(address.sin_zero));
-                address.sin_family = AF_INET;
-                address.sin_port = 0;
-                address.sin_addr.s_addr = inet_addr( "127.0.0.1" );
-                assert( 0 == ::bind( s, (sockaddr*)&address, sizeof( address ) ) );
-                
-                sockaddr_in newAddress;
-                socklen_t len = sizeof( newAddress );
-                assert( 0 == getsockname( s, (sockaddr*)&newAddress, &len ) );
-                ports.push_back( ntohs( newAddress.sin_port ) );
-                sockets.push_back( s );
+            path root( args.firstElement().valuestrsafe() );
+            if ( boost::filesystem::exists( root ) ){
+                found = true;
+                boost::filesystem::remove_all( root );
             }
-            for( vector< int >::const_iterator i = sockets.begin(); i != sockets.end(); ++i )
-                assert( 0 == close( *i ) );
-            
-            sort( ports.begin(), ports.end() );
-            for( unsigned i = 1; i < ports.size(); ++i )
-                massert( "duplicate ports allocated", ports[ i - 1 ] != ports[ i ] );
+
             BSONObjBuilder b;
-            b.append( "", ports );
+            b.appendBool( "removed" , found );
             return b.obj();
         }
-
         map< int, pair< pid_t, int > > dbs;
         map< pid_t, int > shells;
+#ifdef _WIN32
+        map< pid_t, HANDLE > handles;
+#endif
         
-        char *copyString( const char *original ) {
-            char *ret = reinterpret_cast< char * >( malloc( strlen( original ) + 1 ) );
-            strcpy( ret, original );
-            return ret;
-        }
-        
-        boost::mutex &mongoProgramOutputMutex( *( new boost::mutex ) );
+        mongo::mutex mongoProgramOutputMutex;
         stringstream mongoProgramOutput_;
-        
+
         void writeMongoProgramOutputLine( int port, int pid, const char *line ) {
-            boost::mutex::scoped_lock lk( mongoProgramOutputMutex );
+            mongo::mutex::scoped_lock lk( mongoProgramOutputMutex );
             stringstream buf;
             if ( port > 0 )
                 buf << "m" << port << "| " << line;
@@ -199,38 +209,52 @@ namespace mongo {
             mongoProgramOutput_ << buf.str() << endl;
         }
         
+        // only returns last 100000 characters
         BSONObj RawMongoProgramOutput( const BSONObj &args ) {
-            boost::mutex::scoped_lock lk( mongoProgramOutputMutex );
-            return BSON( "" << mongoProgramOutput_.str() );
+            mongo::mutex::scoped_lock lk( mongoProgramOutputMutex );
+            string out = mongoProgramOutput_.str();
+            size_t len = out.length();
+            if ( len > 100000 )
+                out = out.substr( len - 100000, 100000 );
+            return BSON( "" << out );
         }
-                
-        class MongoProgramRunner {
-            char **argv_;
+
+        BSONObj ClearRawMongoProgramOutput( const BSONObj &args ) {
+            mongo::mutex::scoped_lock lk( mongoProgramOutputMutex );
+            mongoProgramOutput_.str( "" );
+            return undefined_;
+        }
+        
+        class ProgramRunner {
+            vector<string> argv_;
             int port_;
             int pipe_;
             pid_t pid_;
         public:
             pid_t pid() const { return pid_; }
-            MongoProgramRunner( const BSONObj &args ) {
-                assert( args.nFields() > 0 );
+            ProgramRunner( const BSONObj &args , bool isMongoProgram=true)
+            {
+                assert( !args.isEmpty() );
+
                 string program( args.firstElement().valuestrsafe() );
-                
                 assert( !program.empty() );
-                boost::filesystem::path programPath = ( boost::filesystem::path( argv0 ) ).branch_path() / program;
-                massert( "couldn't find " + programPath.native_file_string(), boost::filesystem::exists( programPath ) );
+                boost::filesystem::path programPath = program;
+
+                if (isMongoProgram){
+                    programPath = boost::filesystem::initial_path() / programPath;
+#ifdef _WIN32
+                    programPath = change_extension(programPath, ".exe");
+#endif
+                    massert( 10435 ,  "couldn't find " + programPath.native_file_string(), boost::filesystem::exists( programPath ) );
+                }
+
+                argv_.push_back( programPath.native_file_string() );
                 
                 port_ = -1;
-                argv_ = new char *[ args.nFields() + 1 ];
-                {
-                    string s = programPath.native_file_string();
-                    if ( s == program )
-                        s = "./" + s;
-                    argv_[ 0 ] = copyString( s.c_str() );
-                }
                 
                 BSONObjIterator j( args );
-                j.next();
-                for( int i = 1; i < args.nFields(); ++i ) {
+                j.next(); // skip program name (handled above)
+                while(j.more()) {
                     BSONElement e = j.next();
                     string str;
                     if ( e.isNumber() ) {
@@ -241,14 +265,12 @@ namespace mongo {
                         assert( e.type() == mongo::String );
                         str = e.valuestr();
                     }
-                    char *s = copyString( str.c_str() );
-                    if ( string( "--port" ) == s )
+                    if ( str == "--port" )
                         port_ = -2;
                     else if ( port_ == -2 )
-                        port_ = strtol( s, 0, 10 );
-                    argv_[ i ] = s;
+                        port_ = strtol( str.c_str(), 0, 10 );
+                    argv_.push_back(str);
                 }
-                argv_[ args.nFields() ] = 0;
                 
                 if ( program != "mongod" && program != "mongos" && program != "mongobridge" )
                     port_ = 0;
@@ -265,27 +287,12 @@ namespace mongo {
                 assert( pipe( pipeEnds ) != -1 );
                 
                 fflush( 0 );
-                pid_ = fork();
-                assert( pid_ != -1 );
-                
-                if ( pid_ == 0 ) {
-                    
-                    assert( dup2( pipeEnds[ 1 ], STDOUT_FILENO ) != -1 );
-                    assert( dup2( pipeEnds[ 1 ], STDERR_FILENO ) != -1 );
-                    execvp( argv_[ 0 ], argv_ );
-                    massert( "Unable to start program" , 0 );
-                }
+                launch_process(pipeEnds[1]); //sets pid_
                 
                 cout << "shell: started mongo program";
-                int i = 0;
-                while( argv_[ i ] )
-                    cout << " " << argv_[ i++ ];
+                for (unsigned i=0; i < argv_.size(); i++)
+                    cout << " " << argv_[i];
                 cout << endl;
-                
-                i = 0;
-                while( argv_[ i ] )
-                    free( argv_[ i++ ] );
-                free( argv_ );
 
                 if ( port_ > 0 )
                     dbs.insert( make_pair( port_, make_pair( pid_, pipeEnds[ 1 ] ) ) );
@@ -323,30 +330,136 @@ namespace mongo {
                         strcpy( temp, last );
                         strcpy( buf, temp );
                     } else {
-                        assert( strlen( buf ) < 1023 );
+                        assert( strlen( buf ) <= 1023 );
                     }
                     start = buf + strlen( buf );
                 }        
             }
+            void launch_process(int child_stdout){
+#ifdef _WIN32
+                stringstream ss;
+                for (int i=0; i < argv_.size(); i++){
+                    if (i) ss << ' ';
+                    if (argv_[i].find(' ') == string::npos)
+                        ss << argv_[i];
+                    else
+                        ss << '"' << argv_[i] << '"';
+                }
+
+                string args = ss.str();
+                
+                boost::scoped_array<TCHAR> args_tchar (new TCHAR[args.size() + 1]);
+                for (size_t i=0; i < args.size()+1; i++)
+                    args_tchar[i] = args[i];
+
+                HANDLE h = (HANDLE)_get_osfhandle(child_stdout);
+                assert(h != INVALID_HANDLE_VALUE);
+                assert(SetHandleInformation(h, HANDLE_FLAG_INHERIT, 1));
+
+                STARTUPINFO si;
+                ZeroMemory(&si, sizeof(si));
+                si.cb = sizeof(si);
+                si.hStdError = h;
+                si.hStdOutput = h;
+                si.dwFlags |= STARTF_USESTDHANDLES;
+
+                PROCESS_INFORMATION pi;
+                ZeroMemory(&pi, sizeof(pi));
+
+                bool success = CreateProcess( NULL, args_tchar.get(), NULL, NULL, true, 0, NULL, NULL, &si, &pi);
+                assert(success);
+
+                CloseHandle(pi.hThread);
+
+                pid_ = pi.dwProcessId;
+                handles.insert( make_pair( pid_, pi.hProcess ) );
+
+#else
+
+                pid_ = fork();
+                assert( pid_ != -1 );
+                
+                if ( pid_ == 0 ) {
+                    // DON'T ASSERT IN THIS BLOCK - very bad things will happen
+
+                    const char** argv = new const char* [argv_.size()+1]; // don't need to free - in child
+                    for (unsigned i=0; i < argv_.size(); i++){
+                        argv[i] = argv_[i].c_str();
+                    }
+                    argv[argv_.size()] = 0;
+                    
+                    if ( dup2( child_stdout, STDOUT_FILENO ) == -1 ||
+                         dup2( child_stdout, STDERR_FILENO ) == -1 )
+                    {
+                        cout << "Unable to dup2 child output: " << OUTPUT_ERRNO << endl;
+                        ::_Exit(-1); //do not pass go, do not call atexit handlers
+                    }
+
+                    execvp( argv[ 0 ], const_cast<char**>(argv) );
+
+                    cout << "Unable to start program: " << OUTPUT_ERRNO << endl;
+                    ::_Exit(-1);
+                }
+
+#endif
+            }
         };
         
+        //returns true if process exited
+        bool wait_for_pid(pid_t pid, bool block=true, int* exit_code=NULL){
+#ifdef _WIN32
+            assert(handles.count(pid));
+            HANDLE h = handles[pid];
+
+            if (block)
+                WaitForSingleObject(h, INFINITE);
+
+            DWORD tmp;
+            if(GetExitCodeProcess(h, &tmp)){
+                CloseHandle(h);
+                handles.erase(pid);
+                if (exit_code)
+                    *exit_code = tmp;
+                return true;
+            }else{
+                return false;
+            }
+#else
+            int tmp;
+            bool ret = (pid == waitpid(pid, &tmp, (block ? 0 : WNOHANG)));
+            if (exit_code)
+                *exit_code = WEXITSTATUS(tmp);
+            return ret;
+                
+#endif
+        }
         BSONObj StartMongoProgram( const BSONObj &a ) {
-            MongoProgramRunner r( a );
+            _nokillop = true;
+            ProgramRunner r( a );
             r.start();
             boost::thread t( r );
             return BSON( string( "" ) << int( r.pid() ) );
         }
 
         BSONObj RunMongoProgram( const BSONObj &a ) {
-            MongoProgramRunner r( a );
+            ProgramRunner r( a );
             r.start();
             boost::thread t( r );
-            int temp;
-            waitpid( r.pid() , &temp , 0 );
+            wait_for_pid(r.pid());
             shells.erase( r.pid() );
             return BSON( string( "" ) << int( r.pid() ) );
         }
 
+        BSONObj RunProgram(const BSONObj &a) {
+            ProgramRunner r( a, false );
+            r.start();
+            boost::thread t( r );
+            int exit_code;
+            wait_for_pid(r.pid(), true,  &exit_code);
+            shells.erase( r.pid() );
+            return BSON( string( "" ) << exit_code );
+        }
+
         BSONObj ResetDbpath( const BSONObj &a ) {
             assert( a.nFields() == 1 );
             string path = a.firstElement().valuestrsafe();
@@ -357,19 +470,73 @@ namespace mongo {
             return undefined_;
         }
         
-        void killDb( int port, pid_t _pid, int signal ) {
+        void copyDir( const path &from, const path &to ) {
+            directory_iterator end;
+            directory_iterator i( from );
+            while( i != end ) {
+                path p = *i;
+                if ( p.leaf() != "mongod.lock" ) {
+                    if ( is_directory( p ) ) {
+                        path newDir = to / p.leaf();
+                        boost::filesystem::create_directory( newDir );
+                        copyDir( p, newDir );
+                    } else {
+                        boost::filesystem::copy_file( p, to / p.leaf() );
+                    }
+                }
+                ++i;
+            }            
+        }
+        
+        // NOTE target dbpath will be cleared first
+        BSONObj CopyDbpath( const BSONObj &a ) {
+            assert( a.nFields() == 2 );
+            BSONObjIterator i( a );
+            string from = i.next().str();
+            string to = i.next().str();
+            assert( !from.empty() );
+            assert( !to.empty() );
+            if ( boost::filesystem::exists( to ) )
+                boost::filesystem::remove_all( to );
+            boost::filesystem::create_directory( to );
+            copyDir( from, to );
+            return undefined_;
+        }
+
+        inline void kill_wrapper(pid_t pid, int sig, int port){
+#ifdef _WIN32
+            if (sig == SIGKILL || port == 0){
+                assert( handles.count(pid) );
+                TerminateProcess(handles[pid], 1); // returns failure for "zombie" processes.
+            }else{
+                DBClientConnection conn;
+                conn.connect("127.0.0.1:" + BSONObjBuilder::numStr(port));
+                try {
+                    conn.simpleCommand("admin", NULL, "shutdown");
+                } catch (...) {
+                    //Do nothing. This command never returns data to the client and the driver doesn't like that.
+                }
+            }
+#else
+            assert( 0 == kill( pid, sig ) );
+#endif
+        }
+            
+        
+        int killDb( int port, pid_t _pid, int signal ) {
             pid_t pid;
+            int exitCode = 0;
             if ( port > 0 ) {
                 if( dbs.count( port ) != 1 ) {
                     cout << "No db started on port: " << port << endl;
-                    return;
+                    return 0;
                 }
                 pid = dbs[ port ].first;
             } else {
                 pid = _pid;
             }
             
-            assert( 0 == kill( pid, signal ) );
+            kill_wrapper( pid, signal, port );
             
             int i = 0;
             for( ; i < 65; ++i ) {
@@ -378,11 +545,9 @@ namespace mongo {
                     time_t_to_String(time(0), now);
                     now[ 20 ] = 0;
                     cout << now << " process on port " << port << ", with pid " << pid << " not terminated, sending sigkill" << endl;
-                    assert( 0 == kill( pid, SIGKILL ) );
+                    kill_wrapper( pid, SIGKILL, port );
                 }        
-                int temp;
-                int ret = waitpid( pid, &temp, WNOHANG );
-                if ( ret == pid )
+                if(wait_for_pid(pid, false, &exitCode))
                     break;
                 sleepms( 1000 );
             }
@@ -393,7 +558,7 @@ namespace mongo {
                 cout << now << " failed to terminate process on port " << port << ", with pid " << pid << endl;
                 assert( "Failed to terminate process" == 0 );
             }
-            
+
             if ( port > 0 ) {
                 close( dbs[ port ].second );
                 dbs.erase( port );
@@ -404,6 +569,8 @@ namespace mongo {
             if ( i > 4 || signal == SIGKILL ) {
                 sleepms( 4000 ); // allow operating system to reclaim resources
             }
+            
+            return exitCode;
         }
 
         int getSignal( const BSONObj &a ) {
@@ -422,18 +589,18 @@ namespace mongo {
             assert( a.nFields() == 1 || a.nFields() == 2 );
             assert( a.firstElement().isNumber() );
             int port = int( a.firstElement().number() );
-            killDb( port, 0, getSignal( a ) );
+            int code = killDb( port, 0, getSignal( a ) );
             cout << "shell: stopped mongo program on port " << port << endl;
-            return undefined_;
+            return BSON( "" << code );
         }        
         
         BSONObj StopMongoProgramByPid( const BSONObj &a ) {
             assert( a.nFields() == 1 || a.nFields() == 2 );
             assert( a.firstElement().isNumber() );
             int pid = int( a.firstElement().number() );            
-            killDb( 0, pid, getSignal( a ) );
+            int code = killDb( 0, pid, getSignal( a ) );
             cout << "shell: stopped mongo program on pid " << pid << endl;
-            return undefined_;            
+            return BSON( "" << code );
         }
                                                 
         void KillMongoProgramInstances() {
@@ -448,51 +615,99 @@ namespace mongo {
             for( vector< pid_t >::iterator i = pids.begin(); i != pids.end(); ++i )
                 killDb( 0, *i, SIGTERM );
         }
+#else // ndef MONGO_SAFE_SHELL
+        void KillMongoProgramInstances() {}
+#endif
         
         MongoProgramScope::~MongoProgramScope() {
-            try {
+            DESTRUCTOR_GUARD(
                 KillMongoProgramInstances();
-            } catch ( ... ) {
-                assert( false );
-            }
+                ClearRawMongoProgramOutput( BSONObj() );
+            )
         }
 
+        unsigned _randomSeed;
+        
+        BSONObj JSSrand( const BSONObj &a ) {
+            uassert( 12518, "srand requires a single numeric argument",
+                    a.nFields() == 1 && a.firstElement().isNumber() );
+            _randomSeed = (unsigned)a.firstElement().numberLong(); // grab least significant digits
+            return undefined_;
+        }
+        
+        BSONObj JSRand( const BSONObj &a ) {
+            uassert( 12519, "rand accepts no arguments", a.nFields() == 0 );
+            unsigned r;
+#if !defined(_WIN32)
+            r = rand_r( &_randomSeed );
 #else
-        MongoProgramScope::~MongoProgramScope() {}
-        void KillMongoProgramInstances() {}        
+            r = rand(); // seed not used in this case
 #endif
+            return BSON( "" << double( r ) / ( double( RAND_MAX ) + 1 ) );
+        }
 
-        BSONObj jsmd5( const BSONObj &a ){
-            uassert( "js md5 needs a string" , a.firstElement().type() == String );
-            const char * s = a.firstElement().valuestrsafe();
-            
-            md5digest d;
-            md5_state_t st;
-            md5_init(&st);
-            md5_append( &st , (const md5_byte_t*)s , strlen( s ) );
-            md5_finish(&st, d);
-            
-            return BSON( "" << digestToString( d ) );
+        BSONObj isWindows(const BSONObj& a){
+            uassert( 13006, "isWindows accepts no arguments", a.nFields() == 0 );
+#ifdef _WIN32
+            return BSON( "" << true );
+#else
+            return BSON( "" << false );
+#endif
         }
         
         void installShellUtils( Scope& scope ){
-            scope.injectNative( "listFiles" , listFiles );
             scope.injectNative( "sleep" , JSSleep );
             scope.injectNative( "quit", Quit );
             scope.injectNative( "getMemInfo" , JSGetMemInfo );
-            scope.injectNative( "version" , JSVersion );
-            scope.injectNative( "hex_md5" , jsmd5 );
-#if !defined(_WIN32)
-            scope.injectNative( "allocatePorts", AllocatePorts );
+            scope.injectNative( "_srand" , JSSrand );
+            scope.injectNative( "_rand" , JSRand );
+            scope.injectNative( "_isWindows" , isWindows );
+
+#ifndef MONGO_SAFE_SHELL
+            //can't launch programs
             scope.injectNative( "_startMongoProgram", StartMongoProgram );
+            scope.injectNative( "runProgram", RunProgram );
             scope.injectNative( "runMongoProgram", RunMongoProgram );
             scope.injectNative( "stopMongod", StopMongoProgram );
             scope.injectNative( "stopMongoProgram", StopMongoProgram );        
             scope.injectNative( "stopMongoProgramByPid", StopMongoProgramByPid );        
-            scope.injectNative( "resetDbpath", ResetDbpath );
             scope.injectNative( "rawMongoProgramOutput", RawMongoProgramOutput );
+            scope.injectNative( "clearRawMongoProgramOutput", ClearRawMongoProgramOutput );
+
+            //can't access filesystem
+            scope.injectNative( "removeFile" , removeFile );
+            scope.injectNative( "listFiles" , listFiles );
+            scope.injectNative( "resetDbpath", ResetDbpath );
+            scope.injectNative( "copyDbpath", CopyDbpath );
 #endif
         }
+
+        void initScope( Scope &scope ) {
+            scope.externalSetup();
+            mongo::shellUtils::installShellUtils( scope );
+            scope.execSetup( jsconcatcode_server , "setupServerCode" );
+            
+            if ( !_dbConnect.empty() ) {
+                uassert( 12513, "connect failed", scope.exec( _dbConnect , "(connect)" , false , true , false ) );
+                if ( !_dbAuth.empty() ) {
+                    installGlobalUtils( scope );
+                    uassert( 12514, "login failed", scope.exec( _dbAuth , "(auth)" , true , true , false ) );
+                }
+            }
+        }
         
+        map< const void*, string > _allMyUris;        
+        bool _nokillop = false;
+        void onConnect( DBClientWithCommands &c ) {
+            if ( _nokillop ) {
+                return;
+            }
+            BSONObj info;
+            if ( c.runCommand( "admin", BSON( "whatsmyuri" << 1 ), info ) ) {
+                // There's no way to explicitly disconnect a DBClientConnection, but we might allocate
+                // a new uri on automatic reconnect.  So just store one uri per connection.
+                _allMyUris[ &c ] = info[ "you" ].str();
+            }
+        }
     }
 }
