@@ -34,16 +34,4 @@ namespace mongo {
 #define malloc mongo::ourmalloc
 #define realloc mongo::ourrealloc
     
-#if defined(_WIN32)
-    inline void our_debug_free(void *p) {
-#if 0
-        // this is not safe if you malloc < 4 bytes so we don't use anymore
-        unsigned *u = (unsigned *) p;
-        u[0] = 0xEEEEEEEE;
-#endif
-        free(p);
-    }
-#define free our_debug_free
-#endif
-    
 } // namespace mongo
