@@ -41,4 +41,15 @@ namespace mongo {
         return -1;
     }
 
+    void ProcessInfo::getExtraInfo(BSONObjBuilder& info) {}
+    
+    bool ProcessInfo::blockCheckSupported(){
+        return false;
+    }
+    
+    bool ProcessInfo::blockInMemory( char * start ){
+        assert(0);
+        return true;
+    }
+
 }
